@@ -601,6 +601,9 @@ glfs_seek (struct glfs_fd *glfd, off_t offset, int whence)
                 glfd->offset = off;
 
 done:
+        if (fd)
+                fd_unref (fd);
+
         glfs_subvol_done (glfd->fs, subvol);
 
 out:
@@ -932,6 +935,8 @@ pub_glfs_preadv_async (struct glfs_fd *glfd, const struct iovec *iovec,
 
 out:
         if (ret) {
+                if (fd)
+                        fd_unref (fd);
                 if (glfd)
                         GF_REF_PUT (glfd);
                 if (gio) {
@@ -1247,6 +1252,8 @@ pub_glfs_pwritev_async (struct glfs_fd *glfd, const struct iovec *iovec,
         ret = 0;
 out:
         if (ret) {
+                if (fd)
+                        fd_unref (fd);
                 if (glfd)
                         GF_REF_PUT (glfd);
                 GF_FREE (gio);
@@ -1431,6 +1438,8 @@ glfs_fsync_async_common (struct glfs_fd *glfd, glfs_io_cbk fn, void *data,
 
 out:
         if (ret) {
+                if (fd)
+                        fd_unref (fd);
                 if (glfd)
                         GF_REF_PUT (glfd);
                 GF_FREE (gio);
@@ -1682,6 +1691,8 @@ pub_glfs_ftruncate_async (struct glfs_fd *glfd, off_t offset, glfs_io_cbk fn,
 
 out:
         if (ret) {
+                if (fd)
+                        fd_unref (fd);
                 if (glfd)
                         GF_REF_PUT (glfd);
                 GF_FREE (gio);
@@ -2538,6 +2549,8 @@ pub_glfs_discard_async (struct glfs_fd *glfd, off_t offset, size_t len,
         ret = 0;
 out:
         if (ret) {
+                if (fd)
+                        fd_unref (fd);
                 if (glfd)
                         GF_REF_PUT (glfd);
                 GF_FREE (gio);
@@ -2623,6 +2636,8 @@ pub_glfs_zerofill_async (struct glfs_fd *glfd, off_t offset, off_t len,
         ret = 0;
 out:
         if (ret) {
+                if (fd)
+                        fd_unref (fd);
                 if (glfd)
                         GF_REF_PUT (glfd);
                 GF_FREE (gio);
@@ -4325,7 +4340,8 @@ priv_glfs_process_upcall_event (struct glfs *fs, void *data)
         {
                 ctx = fs->ctx;
 
-                if (ctx->cleanup_started) {
+                /* if we're not interested in upcalls (anymore), skip them */
+                if (ctx->cleanup_started || !fs->cache_upcalls) {
                         pthread_mutex_unlock (&fs->mutex);
                         goto out;
                 }
