@@ -16,11 +16,6 @@
 #include <inttypes.h>
 #include <limits.h>
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "glusterfs.h"
 #include "logging.h"
 #include "stack.h"
@@ -91,6 +86,7 @@ glfs_refresh_inode_safe (xlator_t *subvol, inode_t *oldinode,
 	struct iatt  iatt = {0, };
 	inode_t     *newinode = NULL;
         gf_boolean_t lookup_needed = _gf_false;
+        uint64_t     ctx_value = LOOKUP_NOT_NEEDED;
 
 
 	if (!oldinode)
@@ -129,8 +125,16 @@ glfs_refresh_inode_safe (xlator_t *subvol, inode_t *oldinode,
 	}
 
 	newinode = inode_link (loc.inode, 0, 0, &iatt);
-	if (newinode)
-		inode_lookup (newinode);
+        if (newinode) {
+                if (newinode == loc.inode)
+                        inode_ctx_set (newinode, THIS, &ctx_value);
+                inode_lookup (newinode);
+        } else {
+                gf_msg (subvol->name, GF_LOG_WARNING, errno,
+                        API_MSG_INODE_LINK_FAILED,
+                        "inode linking of %s failed",
+                        uuid_utoa ((unsigned char *)&iatt.ia_gfid));
+        }
 
 	loc_wipe (&loc);
 
@@ -240,6 +244,7 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 	struct iatt  ciatt = {0, };
 	uuid_t       gfid;
 	dict_t      *xattr_req = NULL;
+        uint64_t     ctx_value = LOOKUP_NOT_NEEDED;
 
 	loc.name = component;
 
@@ -270,7 +275,7 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 		gf_uuid_copy (loc.gfid, loc.inode->gfid);
 		reval = 1;
 
-		if (!force_lookup) {
+                if (!(force_lookup || inode_needs_lookup (loc.inode, THIS))) {
 			inode = inode_ref (loc.inode);
 			ciatt.ia_type = inode->ia_type;
 			goto found;
@@ -313,6 +318,7 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
                         inode_unlink(loc.inode, loc.parent,
                                      loc.name);
 		inode_unref (loc.inode);
+	        gf_uuid_clear (loc.gfid);
 		loc.inode = inode_new (parent->table);
 		if (!loc.inode) {
 			errno = ENOMEM;
@@ -341,6 +347,15 @@ glfs_resolve_component (struct glfs *fs, xlator_t *subvol, inode_t *parent,
 		goto out;
 
 	inode = inode_link (loc.inode, loc.parent, component, &ciatt);
+
+        if (!inode) {
+                gf_msg (subvol->name, GF_LOG_WARNING, errno,
+                        API_MSG_INODE_LINK_FAILED,
+                        "inode linking of %s failed",
+                        uuid_utoa ((unsigned char *)&ciatt.ia_gfid));
+                goto out;
+        } else if (inode == loc.inode)
+                inode_ctx_set (inode, THIS, &ctx_value);
 found:
 	if (inode)
 		inode_lookup (inode);
@@ -769,7 +784,7 @@ glfs_resolve_fd (struct glfs *fs, xlator_t *subvol, struct glfs_fd *glfd)
 {
 	fd_t *fd = NULL;
 
-	glfs_lock (fs);
+        glfs_lock (fs, _gf_true);
 	{
 		fd = __glfs_resolve_fd (fs, subvol, glfd);
 	}
@@ -805,6 +820,13 @@ __glfs_migrate_openfds (struct glfs *fs, xlator_t *subvol)
 }
 
 
+/* Note that though it appears that this function executes under fs->mutex,
+ * it is not fully executed under fs->mutex. i.e. there are functions like
+ * __glfs_first_lookup, __glfs_refresh_inode, __glfs_migrate_openfds which
+ * unlocks fs->mutex before sending any network fop, and reacquire fs->mutex
+ * once the fop is complete. Hence the variable read from fs at the start of the
+ * function need not have the same value by the end of the function.
+ */
 xlator_t *
 __glfs_active_subvol (struct glfs *fs)
 {
@@ -815,7 +837,8 @@ __glfs_active_subvol (struct glfs *fs)
 	if (!fs->next_subvol)
 		return fs->active_subvol;
 
-	new_subvol = fs->next_subvol;
+	new_subvol = fs->mip_subvol = fs->next_subvol;
+        fs->next_subvol = NULL;
 
 	ret = __glfs_first_lookup (fs, new_subvol);
 	if (ret) {
@@ -849,8 +872,8 @@ __glfs_active_subvol (struct glfs *fs)
 	   should be atomic
 	*/
 	fs->old_subvol = fs->active_subvol;
-	fs->active_subvol = fs->next_subvol;
-	fs->next_subvol = NULL;
+	fs->active_subvol = fs->mip_subvol;
+	fs->mip_subvol = NULL;
 
 	if (new_cwd) {
 		__glfs_cwd_set (fs, new_cwd);
@@ -874,12 +897,17 @@ priv_glfs_subvol_done (struct glfs *fs, xlator_t *subvol)
 	if (!subvol)
 		return;
 
-	glfs_lock (fs);
+        /* For decrementing subvol->wind ref count we need not check/wait for
+         * migration-in-progress flag.
+         * Also glfs_subvol_done is called in call-back path therefore waiting
+         * fot migration-in-progress flag can lead to dead-lock.
+         */
+        glfs_lock (fs, _gf_false);
 	{
 		ref = (--subvol->winds);
 		active_subvol = fs->active_subvol;
 	}
-	glfs_unlock (fs);
+        glfs_unlock (fs);
 
 	if (ref == 0) {
 		assert (subvol != active_subvol);
@@ -896,7 +924,7 @@ priv_glfs_active_subvol (struct glfs *fs)
 	xlator_t      *subvol = NULL;
 	xlator_t      *old_subvol = NULL;
 
-	glfs_lock (fs);
+        glfs_lock (fs, _gf_true);
 	{
 		subvol = __glfs_active_subvol (fs);
 
@@ -945,7 +973,7 @@ glfs_cwd_set (struct glfs *fs, inode_t *inode)
 {
 	int ret = 0;
 
-	glfs_lock (fs);
+        glfs_lock (fs, _gf_true);
 	{
 		ret = __glfs_cwd_set (fs, inode);
 	}
@@ -978,7 +1006,7 @@ glfs_cwd_get (struct glfs *fs)
 {
 	inode_t *cwd = NULL;
 
-	glfs_lock (fs);
+        glfs_lock (fs, _gf_true);
 	{
 		cwd = __glfs_cwd_get (fs);
 	}
@@ -1018,7 +1046,7 @@ glfs_resolve_inode (struct glfs *fs, xlator_t *subvol,
 {
 	inode_t *inode = NULL;
 
-	glfs_lock (fs);
+        glfs_lock (fs, _gf_true);
 	{
 		inode = __glfs_resolve_inode(fs, subvol, object);
 	}
