@@ -19,11 +19,6 @@
 #include <sys/wait.h>
 #include <netinet/in.h>
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "cli.h"
 #include "cli-cmd.h"
 #include "cli-mem-types.h"
@@ -1109,90 +1104,6 @@ out:
         return ret;
 }
 
-static int
-gf_cli_create_auxiliary_mount (char *volname)
-{
-        int      ret                     = -1;
-        char     mountdir[PATH_MAX]      = {0,};
-        char     pidfile_path[PATH_MAX]  = {0,};
-        char     logfile[PATH_MAX]       = {0,};
-        char     qpid [16]               = {0,};
-
-        GLUSTERFS_GET_AUX_MOUNT_PIDFILE (pidfile_path, volname);
-
-        if (gf_is_service_running (pidfile_path, NULL)) {
-                gf_log ("cli", GF_LOG_DEBUG, "Aux mount of volume %s is running"
-                        " already", volname);
-                ret = 0;
-                goto out;
-        }
-
-        GLUSTERD_GET_QUOTA_AUX_MOUNT_PATH (mountdir, volname, "/");
-        ret = mkdir (mountdir, 0777);
-        if (ret && errno != EEXIST) {
-                gf_log ("cli", GF_LOG_ERROR, "Failed to create auxiliary mount "
-                        "directory %s. Reason : %s", mountdir,
-                        strerror (errno));
-                goto out;
-        }
-
-        snprintf (logfile, PATH_MAX-1, "%s/quota-mount-%s.log",
-                  DEFAULT_LOG_FILE_DIRECTORY, volname);
-        snprintf(qpid, 15, "%d", GF_CLIENT_PID_QUOTA_MOUNT);
-
-        ret = runcmd (SBIN_DIR"/glusterfs",
-                      "-s", "localhost",
-                      "--volfile-id", volname,
-                      "-l", logfile,
-                      "-p", pidfile_path,
-                      "--client-pid", qpid,
-                      mountdir,
-                      NULL);
-
-        if (ret) {
-                gf_log ("cli", GF_LOG_WARNING, "failed to mount glusterfs "
-                        "client. Please check the log file %s for more details",
-                        logfile);
-                ret = -1;
-                goto out;
-        }
-
-        ret = 0;
-
-out:
-        return ret;
-}
-
-static int
-cli_stage_quota_op (char *volname, int op_code)
-{
-        int ret = -1;
-
-        switch (op_code) {
-        case GF_QUOTA_OPTION_TYPE_ENABLE:
-        case GF_QUOTA_OPTION_TYPE_LIMIT_USAGE:
-        case GF_QUOTA_OPTION_TYPE_LIMIT_OBJECTS:
-        case GF_QUOTA_OPTION_TYPE_REMOVE:
-        case GF_QUOTA_OPTION_TYPE_REMOVE_OBJECTS:
-        case GF_QUOTA_OPTION_TYPE_LIST:
-                ret = gf_cli_create_auxiliary_mount (volname);
-                if (ret) {
-                        cli_err ("quota: Could not start quota "
-                                 "auxiliary mount");
-                        goto out;
-                }
-                ret = 0;
-                break;
-
-        default:
-                ret = 0;
-                break;
-        }
-
-out:
-        return ret;
-}
-
 int
 cli_get_soft_limit (dict_t *options, const char **words, dict_t *xdata)
 {
@@ -1258,9 +1169,10 @@ _limits_set_on_volume (char *volname, int type) {
         /* TODO: fix hardcoding; Need to perform an RPC call to glusterd
          * to fetch working directory
          */
-        sprintf (quota_conf_file, "%s/vols/%s/quota.conf",
-                 GLUSTERD_DEFAULT_WORKDIR,
-                 volname);
+        snprintf (quota_conf_file, sizeof quota_conf_file,
+                  "%s/vols/%s/quota.conf",
+                  GLUSTERD_DEFAULT_WORKDIR,
+                  volname);
         fd = open (quota_conf_file, O_RDONLY);
         if (fd == -1)
                 goto out;
@@ -1288,47 +1200,11 @@ _limits_set_on_volume (char *volname, int type) {
         }
 out:
         if (fd != -1)
-                close (fd);
+                sys_close (fd);
 
         return limits_set;
 }
 
-/* Checks if the mount is connected to the bricks
- *
- * Returns true if connected and false if not
- */
-gf_boolean_t
-_quota_aux_mount_online (char *volname)
-{
-        int         ret = 0;
-        char        mount_path[PATH_MAX + 1] = {0,};
-        struct stat buf = {0,};
-
-        GF_ASSERT (volname);
-
-        /* Try to create the aux mount before checking if bricks are online */
-        ret = gf_cli_create_auxiliary_mount (volname);
-        if (ret) {
-                cli_err ("quota: Could not start quota auxiliary mount");
-                return _gf_false;
-        }
-
-        GLUSTERD_GET_QUOTA_AUX_MOUNT_PATH (mount_path, volname, "/");
-
-        ret = sys_stat (mount_path, &buf);
-        if (ret) {
-                if (ENOTCONN == errno) {
-                        cli_err ("quota: Cannot connect to bricks. Check if "
-                                 "bricks are online.");
-                } else {
-                        cli_err ("quota: Error on quota auxiliary mount (%s).",
-                                 strerror (errno));
-                }
-                return _gf_false;
-        }
-        return _gf_true;
-}
-
 int
 cli_cmd_quota_handle_list_all (const char **words, dict_t *options)
 {
@@ -1350,6 +1226,7 @@ cli_cmd_quota_handle_list_all (const char **words, dict_t *options)
         int32_t                  type       = 0;
         char                     gfid_type  = 0;
         float                    version    = 0.0f;
+        int32_t                  max_count  = 0;
 
         xdata = dict_new ();
         if (!xdata) {
@@ -1399,12 +1276,6 @@ cli_cmd_quota_handle_list_all (const char **words, dict_t *options)
                 goto out;
         }
 
-        /* Check if the mount is online before doing any listing */
-        if (!_quota_aux_mount_online (volname)) {
-                ret = -1;
-                goto out;
-        }
-
         frame = create_frame (THIS, THIS->ctx->pool);
         if (!frame) {
                 ret = -1;
@@ -1426,9 +1297,10 @@ cli_cmd_quota_handle_list_all (const char **words, dict_t *options)
 
         //TODO: fix hardcoding; Need to perform an RPC call to glusterd
         //to fetch working directory
-        sprintf (quota_conf_file, "%s/vols/%s/quota.conf",
-                 GLUSTERD_DEFAULT_WORKDIR,
-                 volname);
+        snprintf (quota_conf_file, sizeof quota_conf_file,
+                  "%s/vols/%s/quota.conf",
+                  GLUSTERD_DEFAULT_WORKDIR,
+                  volname);
         fd = open (quota_conf_file, O_RDONLY);
         if (fd == -1) {
                 //This may because no limits were yet set on the volume
@@ -1450,6 +1322,41 @@ cli_cmd_quota_handle_list_all (const char **words, dict_t *options)
                 ret = -1;
                 goto out;
         }
+
+        for (count = 0;; count++) {
+                ret = quota_conf_read_gfid (fd, buf, &gfid_type, version);
+                if (ret == 0) {
+                        break;
+                } else if (ret < 0) {
+                        gf_log (THIS->name, GF_LOG_CRITICAL, "Quota "
+                                "configuration store may be corrupt.");
+                        goto out;
+                }
+
+                if ((type == GF_QUOTA_OPTION_TYPE_LIST &&
+                     gfid_type == GF_QUOTA_CONF_TYPE_OBJECTS) ||
+                    (type == GF_QUOTA_OPTION_TYPE_LIST_OBJECTS &&
+                     gfid_type == GF_QUOTA_CONF_TYPE_USAGE))
+                        continue;
+
+                max_count++;
+        }
+        ret = dict_set_int32 (xdata, "max_count", max_count);
+        if (ret) {
+                gf_log ("cli", GF_LOG_ERROR, "Failed to set max_count");
+                goto out;
+        }
+
+        ret = sys_lseek (fd, 0L, SEEK_SET);
+        if (ret < 0) {
+                gf_log (THIS->name, GF_LOG_ERROR, "failed to move offset to "
+                        "the beginning: %s", strerror (errno));
+                goto out;
+        }
+        ret = quota_conf_read_version (fd, &version);
+        if (ret)
+                goto out;
+
         for (count = 0;; count++) {
                 ret = quota_conf_read_gfid (fd, buf, &gfid_type, version);
                 if (ret == 0) {
@@ -1509,7 +1416,7 @@ out:
         }
 
         if (fd != -1) {
-                close (fd);
+                sys_close (fd);
         }
 
         GF_FREE (gfid_str);
@@ -1637,11 +1544,6 @@ cli_cmd_quota_cbk (struct cli_state *state, struct cli_cmd_word *word,
                 goto out;
         }
 
-        //create auxiliary mount need for quota commands that operate on path
-        ret = cli_stage_quota_op (volname, type);
-        if (ret)
-                goto out;
-
         frame = create_frame (THIS, THIS->ctx->pool);
         if (!frame) {
                 ret = -1;
@@ -2246,9 +2148,11 @@ cli_print_brick_status (cli_volume_status_t *status)
 }
 
 #define NEEDS_GLFS_HEAL(op) ((op == GF_SHD_OP_SBRAIN_HEAL_FROM_BIGGER_FILE) || \
+                             (op == GF_SHD_OP_SBRAIN_HEAL_FROM_LATEST_MTIME) ||\
                              (op == GF_SHD_OP_SBRAIN_HEAL_FROM_BRICK) ||      \
                              (op == GF_SHD_OP_INDEX_SUMMARY) ||               \
-                             (op == GF_SHD_OP_SPLIT_BRAIN_FILES))
+                             (op == GF_SHD_OP_SPLIT_BRAIN_FILES) ||           \
+                             (op == GF_SHD_OP_GRANULAR_ENTRY_HEAL_ENABLE))
 
 int
 cli_launch_glfs_heal (int heal_op, dict_t *options)
@@ -2269,11 +2173,18 @@ cli_launch_glfs_heal (int heal_op, dict_t *options)
 
         switch (heal_op) {
         case GF_SHD_OP_INDEX_SUMMARY:
+                if (global_state->mode & GLUSTER_MODE_XML) {
+                        runner_add_args (&runner, "xml", NULL);
+                }
                 break;
         case GF_SHD_OP_SBRAIN_HEAL_FROM_BIGGER_FILE:
                 ret = dict_get_str (options, "file", &filename);
                 runner_add_args (&runner, "bigger-file", filename, NULL);
                 break;
+        case GF_SHD_OP_SBRAIN_HEAL_FROM_LATEST_MTIME:
+                ret = dict_get_str (options, "file", &filename);
+                runner_add_args (&runner, "latest-mtime", filename, NULL);
+                break;
         case GF_SHD_OP_SBRAIN_HEAL_FROM_BRICK:
                 ret = dict_get_str (options, "heal-source-hostname",
                                     &hostname);
@@ -2286,6 +2197,13 @@ cli_launch_glfs_heal (int heal_op, dict_t *options)
                 break;
         case GF_SHD_OP_SPLIT_BRAIN_FILES:
                 runner_add_args (&runner, "split-brain-info", NULL);
+                if (global_state->mode & GLUSTER_MODE_XML) {
+                        runner_add_args (&runner, "xml", NULL);
+                }
+                break;
+        case GF_SHD_OP_GRANULAR_ENTRY_HEAL_ENABLE:
+        case GF_SHD_OP_GRANULAR_ENTRY_HEAL_DISABLE:
+                runner_add_args (&runner, "granular-entry-heal-op", NULL);
                 break;
         default:
                 ret = -1;
@@ -2298,11 +2216,11 @@ cli_launch_glfs_heal (int heal_op, dict_t *options)
                 printf ("%s", out);
         }
         ret = runner_end (&runner);
-        ret = WEXITSTATUS (ret);
 
 out:
         return ret;
 }
+
 int
 cli_cmd_volume_heal_cbk (struct cli_state *state, struct cli_cmd_word *word,
                           const char **words, int wordcount)
@@ -2339,24 +2257,26 @@ cli_cmd_volume_heal_cbk (struct cli_state *state, struct cli_cmd_word *word,
                 goto out;
         if (NEEDS_GLFS_HEAL (heal_op)) {
                 ret = cli_launch_glfs_heal (heal_op, options);
-                if (ret == -1)
+                if (ret < 0)
+                        goto out;
+                if (heal_op != GF_SHD_OP_GRANULAR_ENTRY_HEAL_ENABLE)
                         goto out;
         }
-        else {
-                proc = &cli_rpc_prog->proctable[GLUSTER_CLI_HEAL_VOLUME];
 
-                CLI_LOCAL_INIT (local, words, frame, options);
+        proc = &cli_rpc_prog->proctable[GLUSTER_CLI_HEAL_VOLUME];
 
-                if (proc->fn) {
-                        ret = proc->fn (frame, THIS, options);
-                }
+        CLI_LOCAL_INIT (local, words, frame, options);
+
+        if (proc->fn) {
+                ret = proc->fn (frame, THIS, options);
         }
-
 out:
         if (ret) {
                 cli_cmd_sent_status_get (&sent);
-                if ((sent == 0) && (parse_error == 0))
+                if ((sent == 0) && (parse_error == 0) &&
+                    !(global_state->mode & GLUSTER_MODE_XML)) {
                         cli_out ("Volume heal failed.");
+                }
         }
 
         CLI_STACK_DESTROY (frame);
@@ -2413,7 +2333,7 @@ cli_cmd_volume_statedump_cbk (struct cli_state *state, struct cli_cmd_word *word
 out:
         if (ret) {
                 cli_cmd_sent_status_get (&sent);
-                if ((sent == 0) && (parse_error = 0))
+                if ((sent == 0) && (parse_error == 0))
                         cli_out ("Volume statedump failed");
         }
 
@@ -2561,8 +2481,6 @@ out:
                         cli_err ("Volume barrier failed");
         }
         CLI_STACK_DESTROY (frame);
-        if (options)
-                dict_unref (options);
 
         return ret;
 }
@@ -2615,8 +2533,6 @@ out:
                         cli_err ("Volume get option failed");
         }
         CLI_STACK_DESTROY (frame);
-        if (options)
-                dict_unref (options);
         return ret;
 }
 
@@ -2653,7 +2569,9 @@ struct cli_cmd volume_cmds[] = {
           cli_cmd_volume_rename_cbk,
           "rename volume <VOLNAME> to <NEW-VOLNAME>"},*/
 
+#if !defined(__NetBSD__)
         { "volume tier <VOLNAME> status\n"
+        "volume tier <VOLNAME> start [force]\n"
         "volume tier <VOLNAME> attach [<replica COUNT>] <NEW-BRICK>...\n"
         "volume tier <VOLNAME> detach <start|stop|status|commit|[force]>\n",
         cli_cmd_volume_tier_cbk,
@@ -2671,8 +2589,10 @@ struct cli_cmd volume_cmds[] = {
           "NOTE: this is old syntax, will be depreciated in next release. "
           "Please use gluster volume tier <vol> detach "
           "{start|stop|commit} [force]"},
+#endif
 
-        { "volume add-brick <VOLNAME> [<stripe|replica> <COUNT>] <NEW-BRICK> ... [force]",
+        { "volume add-brick <VOLNAME> [<stripe|replica> <COUNT> "
+          "[arbiter <COUNT>]] <NEW-BRICK> ... [force]",
           cli_cmd_volume_add_brick_cbk,
           "add brick to volume <VOLNAME>"},
 
@@ -2720,8 +2640,8 @@ struct cli_cmd volume_cmds[] = {
          "reset all the reconfigured options"},
 
 #if (SYNCDAEMON_COMPILE)
-        {"volume "GEOREP" [<VOLNAME>] [<SLAVE-URL>] {create [[no-verify]|[push-pem]] [force]"
-         "|start [force]|stop [force]|pause [force]|resume [force]|config|status [detail]|delete} [options...]",
+        {"volume "GEOREP" [<VOLNAME>] [<SLAVE-URL>] {create [[ssh-port n] [[no-verify]|[push-pem]]] [force]"
+         "|start [force]|stop [force]|pause [force]|resume [force]|config|status [detail]|delete [reset-sync-time]} [options...]",
          cli_cmd_volume_gsync_set_cbk,
          "Geo-sync operations",
          cli_cmd_check_gsync_exists_cbk},
@@ -2757,8 +2677,9 @@ struct cli_cmd volume_cmds[] = {
         { "volume heal <VOLNAME> [enable | disable | full |"
           "statistics [heal-count [replica <HOSTNAME:BRICKNAME>]] |"
           "info [healed | heal-failed | split-brain] |"
-          "split-brain {bigger-file <FILE> |"
-                       "source-brick <HOSTNAME:BRICKNAME> [<FILE>]}]",
+          "split-brain {bigger-file <FILE> | latest-mtime <FILE> |"
+                       "source-brick <HOSTNAME:BRICKNAME> [<FILE>]} |"
+          "granular-entry-heal {enable | disable}]",
           cli_cmd_volume_heal_cbk,
           "self-heal commands on volume specified by <VOLNAME>"},
 
@@ -2788,7 +2709,7 @@ struct cli_cmd volume_cmds[] = {
          "volume bitrot <volname> scrub-throttle {lazy|normal|aggressive} |\n"
          "volume bitrot <volname> scrub-frequency {hourly|daily|weekly|biweekly"
          "|monthly} |\n"
-         "volume bitrot <volname> scrub {pause|resume}",
+         "volume bitrot <volname> scrub {pause|resume|status}",
          cli_cmd_bitrot_cbk,
          "Bitrot translator specific operation. For more information about "
          "bitrot command type  'man gluster'"
