@@ -29,11 +29,6 @@
 #include <semaphore.h>
 #include <errno.h>
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
@@ -159,8 +154,6 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         LOCK_INIT (&pool->lock);
         ctx->pool = pool;
 
-        pthread_mutex_init (&(ctx->lock), NULL);
-
         cmd_args = &ctx->cmd_args;
 
         INIT_LIST_HEAD (&cmd_args->xlator_options);
@@ -365,12 +358,18 @@ cli_opt_parse (char *opt, struct cli_state *state)
                 state->log_file = oarg;
                 return 0;
         }
+        oarg = strtail (opt, "timeout=");
+        if (oarg) {
+                cli_default_conn_timeout = atoi(oarg);
+                return 0;
+        }
 
         oarg = strtail (opt, "log-level=");
         if (oarg) {
-                state->log_level = glusterd_check_log_level(oarg);
-                if (state->log_level == -1)
+                int log_level = glusterd_check_log_level(oarg);
+                if (log_level == -1)
                         return -1;
+                state->log_level = (gf_loglevel_t) log_level;
                 return 0;
         }
 
@@ -410,7 +409,7 @@ parse_cmdline (int argc, char *argv[], struct cli_state *state)
         state->argv=&argv[1];
 
         /* Do this first so that an option can override. */
-        if (access(SECURE_ACCESS_FILE,F_OK) == 0) {
+        if (sys_access (SECURE_ACCESS_FILE, F_OK) == 0) {
                 state->ctx->secure_mgmt = 1;
         }
 
@@ -501,8 +500,10 @@ _cli_err (const char *fmt, ...)
         va_start (ap, fmt);
 
 #ifdef HAVE_READLINE
-        if (state->rl_enabled && !state->rl_processing)
-                return cli_rl_err(state, fmt, ap);
+        if (state->rl_enabled && !state->rl_processing) {
+                va_end (ap);
+                return cli_rl_err (state, fmt, ap);
+        }
 #endif
 
         ret = vfprintf (stderr, fmt, ap);
@@ -525,8 +526,10 @@ _cli_out (const char *fmt, ...)
         va_start (ap, fmt);
 
 #ifdef HAVE_READLINE
-        if (state->rl_enabled && !state->rl_processing)
-                return cli_rl_out(state, fmt, ap);
+        if (state->rl_enabled && !state->rl_processing) {
+                va_end (ap);
+                return cli_rl_out (state, fmt, ap);
+        }
 #endif
 
         ret = vprintf (fmt, ap);
@@ -658,6 +661,7 @@ cli_local_get ()
 
         local = GF_CALLOC (1, sizeof (*local), cli_mt_cli_local_t);
         LOCK_INIT (&local->lock);
+        INIT_LIST_HEAD (&local->dict_list);
 
         return local;
 }
@@ -702,6 +706,9 @@ main (int argc, char *argv[])
         if (ret)
                 goto out;
 
+        cli_default_conn_timeout = 120;
+        cli_ten_minutes_timeout = 600;
+
         ret = cli_state_init (&state);
         if (ret)
                 goto out;
