@@ -32,9 +32,15 @@
 #include <errno.h>
 #include <pwd.h>
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
+#ifdef GF_LINUX_HOST_OS
+#ifdef HAVE_LINUX_OOM_H
+#include <linux/oom.h>
+#else
+#define OOM_SCORE_ADJ_MIN (-1000)
+#define OOM_SCORE_ADJ_MAX   1000
+#define OOM_DISABLE       (-17)
+#define OOM_ADJUST_MAX      15
+#endif
 #endif
 
 #ifdef HAVE_MALLOC_H
@@ -101,8 +107,8 @@ static error_t parse_opts (int32_t key, char *arg, struct argp_state *_state);
 static struct argp_option gf_options[] = {
         {0, 0, 0, 0, "Basic options:"},
         {"volfile-server", ARGP_VOLFILE_SERVER_KEY, "SERVER", 0,
-         "Server to get the volume file from.  This option overrides "
-         "--volfile option"},
+         "Server to get the volume file from. Unix domain socket path when "
+         "transport type 'unix'. This option overrides --volfile option"},
         {"volfile", ARGP_VOLUME_FILE_KEY, "VOLFILE", 0,
          "File to use as VOLUME_FILE"},
         {"spec-file", ARGP_VOLUME_FILE_KEY, "VOLFILE", OPTION_HIDDEN,
@@ -155,7 +161,9 @@ static struct argp_option gf_options[] = {
         {"acl", ARGP_ACL_KEY, 0, 0,
          "Mount the filesystem with POSIX ACL support"},
         {"selinux", ARGP_SELINUX_KEY, 0, 0,
-         "Enable SELinux label (extened attributes) support on inodes"},
+         "Enable SELinux label (extended attributes) support on inodes"},
+        {"capability", ARGP_CAPABILITY_KEY, 0, 0,
+         "Enable Capability (extended attributes) support on inodes"},
 
         {"print-netgroups", ARGP_PRINT_NETGROUPS, "NETGROUP-FILE", 0,
          "Validate the netgroups file and print it out"},
@@ -201,16 +209,21 @@ static struct argp_option gf_options[] = {
          "Set attribute timeout to SECONDS for inodes in fuse kernel module "
          "[default: 1]"},
 	{"gid-timeout", ARGP_GID_TIMEOUT_KEY, "SECONDS", 0,
-	 "Set auxilary group list timeout to SECONDS for fuse translator "
+	 "Set auxiliary group list timeout to SECONDS for fuse translator "
 	 "[default: 300]"},
         {"resolve-gids", ARGP_RESOLVE_GIDS_KEY, 0, 0,
-         "Resolve all auxilary groups in fuse translator (max 32 otherwise)"},
+         "Resolve all auxiliary groups in fuse translator (max 32 otherwise)"},
 	{"background-qlen", ARGP_FUSE_BACKGROUND_QLEN_KEY, "N", 0,
 	 "Set fuse module's background queue length to N "
 	 "[default: 64]"},
 	{"congestion-threshold", ARGP_FUSE_CONGESTION_THRESHOLD_KEY, "N", 0,
 	 "Set fuse module's congestion threshold to N "
 	 "[default: 48]"},
+#ifdef GF_LINUX_HOST_OS
+        {"oom-score-adj", ARGP_OOM_SCORE_ADJ_KEY, "INTEGER", 0,
+         "Set oom_score_adj value for process"
+         "[default: 0]"},
+#endif
         {"client-pid", ARGP_CLIENT_PID_KEY, "PID", OPTION_HIDDEN,
          "client will authenticate itself with process id PID to server"},
         {"no-root-squash", ARGP_FUSE_NO_ROOT_SQUASH_KEY, "BOOL",
@@ -228,7 +241,7 @@ static struct argp_option gf_options[] = {
          "Extra mount options to pass to FUSE"},
         {"use-readdirp", ARGP_FUSE_USE_READDIRP_KEY, "BOOL", OPTION_ARG_OPTIONAL,
          "Use readdirp mode in fuse kernel module"
-         " [default: \"off\"]"},
+         " [default: \"yes\"]"},
         {"secure-mgmt", ARGP_SECURE_MGMT_KEY, "BOOL", OPTION_ARG_OPTIONAL,
          "Override default for secure (SSL) management connections"},
         {0, 0, 0, 0, "Miscellaneous Options:"},
@@ -371,6 +384,15 @@ set_fuse_mount_options (glusterfs_ctx_t *ctx, dict_t *options)
                 }
         }
 
+        if (cmd_args->capability) {
+                ret = dict_set_static_ptr (options, "capability", "on");
+                if (ret < 0) {
+                        gf_msg ("glusterfsd", GF_LOG_ERROR, 0, glusterfsd_msg_4,
+                                "capability");
+                        goto err;
+                }
+        }
+
         if (cmd_args->aux_gfid_mount) {
                 ret = dict_set_static_ptr (options, "virtual-gfid-access",
                                            "on");
@@ -762,20 +784,50 @@ out:
 }
 
 
+#ifdef GF_LINUX_HOST_OS
+static struct oom_api_info {
+        char *oom_api_file;
+        int32_t oom_min;
+        int32_t oom_max;
+} oom_api_info[] = {
+        { "/proc/self/oom_score_adj", OOM_SCORE_ADJ_MIN, OOM_SCORE_ADJ_MAX },
+        { "/proc/self/oom_adj",       OOM_DISABLE,       OOM_ADJUST_MAX },
+        { NULL, 0, 0 }
+};
+
+
+static struct oom_api_info *
+get_oom_api_info (void)
+{
+        struct oom_api_info *api = NULL;
+
+        for (api = oom_api_info; api->oom_api_file; api++) {
+                if (sys_access (api->oom_api_file, F_OK) != -1) {
+                        return api;
+                }
+        }
+
+        return NULL;
+}
+#endif
 
 static error_t
 parse_opts (int key, char *arg, struct argp_state *state)
 {
-        cmd_args_t   *cmd_args      = NULL;
-        uint32_t      n             = 0;
-        double        d             = 0.0;
-        gf_boolean_t  b             = _gf_false;
-        char         *pwd           = NULL;
-        char          tmp_buf[2048] = {0,};
-        char         *tmp_str       = NULL;
-        char         *port_str      = NULL;
-        struct passwd *pw           = NULL;
-        int           ret           = 0;
+        cmd_args_t          *cmd_args      = NULL;
+        uint32_t             n             = 0;
+#ifdef GF_LINUX_HOST_OS
+        int32_t              k             = 0;
+        struct oom_api_info *api           = NULL;
+#endif
+        double               d             = 0.0;
+        gf_boolean_t         b             = _gf_false;
+        char                *pwd           = NULL;
+        char                 tmp_buf[2048] = {0,};
+        char                *tmp_str       = NULL;
+        char                *port_str      = NULL;
+        struct passwd       *pw            = NULL;
+        int                  ret           = 0;
 
         cmd_args = state->input;
 
@@ -799,6 +851,10 @@ parse_opts (int key, char *arg, struct argp_state *state)
                 gf_remember_xlator_option ("*-md-cache.cache-selinux=true");
                 break;
 
+        case ARGP_CAPABILITY_KEY:
+                cmd_args->capability = 1;
+                break;
+
         case ARGP_AUX_GFID_MOUNT_KEY:
                 cmd_args->aux_gfid_mount = 1;
                 break;
@@ -1112,6 +1168,27 @@ parse_opts (int key, char *arg, struct argp_state *state)
                               "unknown congestion threshold option %s", arg);
                 break;
 
+#ifdef GF_LINUX_HOST_OS
+        case ARGP_OOM_SCORE_ADJ_KEY:
+                k = 0;
+
+                api = get_oom_api_info();
+                if (!api)
+                        goto no_oom_api;
+
+                if (gf_string2int (arg, &k) == 0 &&
+                    k >= api->oom_min && k <= api->oom_max) {
+                        cmd_args->oom_score_adj = gf_strdup (arg);
+                        break;
+                }
+
+                argp_failure (state, -1, 0,
+                              "unknown oom_score_adj value %s", arg);
+
+no_oom_api:
+                break;
+#endif
+
         case ARGP_FUSE_MOUNTOPTS_KEY:
                 cmd_args->fuse_mountopts = gf_strdup (arg);
                 break;
@@ -1159,13 +1236,12 @@ parse_opts (int key, char *arg, struct argp_state *state)
                 if (gf_string2uint32 (arg, &cmd_args->log_buf_size)) {
                         argp_failure (state, -1, 0,
                                       "unknown log buf size option %s", arg);
-                } else if ((cmd_args->log_buf_size < GF_LOG_LRU_BUFSIZE_MIN) ||
-                          (cmd_args->log_buf_size > GF_LOG_LRU_BUFSIZE_MAX)) {
-                            argp_failure (state, -1, 0,
-                                          "Invalid log buf size %s. "
-                                          "Valid range: ["
-                                          GF_LOG_LRU_BUFSIZE_MIN_STR","
-                                          GF_LOG_LRU_BUFSIZE_MAX_STR"]", arg);
+                } else if (cmd_args->log_buf_size > GF_LOG_LRU_BUFSIZE_MAX) {
+                        argp_failure (state, -1, 0,
+                                      "Invalid log buf size %s. "
+                                      "Valid range: ["
+                                      GF_LOG_LRU_BUFSIZE_MIN_STR","
+                                      GF_LOG_LRU_BUFSIZE_MAX_STR"]", arg);
                 }
 
                 break;
@@ -1204,6 +1280,21 @@ parse_opts (int key, char *arg, struct argp_state *state)
         return 0;
 }
 
+gf_boolean_t
+should_call_fini (glusterfs_ctx_t *ctx, xlator_t *trav)
+{
+        /* There's nothing to call, so the other checks don't matter. */
+        if (!trav->fini) {
+                return _gf_false;
+        }
+
+        /* This preserves previous behavior in glusterd. */
+        if (ctx->process_mode == GF_GLUSTERD_PROCESS) {
+                return _gf_true;
+        }
+
+        return _gf_false;
+}
 
 void
 cleanup_and_exit (int signum)
@@ -1276,21 +1367,18 @@ cleanup_and_exit (int signum)
 
         /*call fini for glusterd xlator */
         /* TODO : Invoke fini for rest of the xlators */
-        if (ctx->process_mode == GF_GLUSTERD_PROCESS) {
-
-                trav = NULL;
-                if (ctx->active)
-                        trav = ctx->active->top;
-                while (trav) {
-                        if (trav->fini) {
-                                THIS = trav;
-                                trav->fini (trav);
-                        }
-                        trav = trav->next;
+        trav = NULL;
+        if (ctx->active)
+                trav = ctx->active->top;
+        while (trav) {
+                if (should_call_fini(ctx,trav)) {
+                        THIS = trav;
+                        trav->fini (trav);
                 }
-
+                trav = trav->next;
         }
-        exit(0);
+
+        exit(signum);
 }
 
 
@@ -1327,8 +1415,8 @@ emancipate (glusterfs_ctx_t *ctx, int ret)
 {
         /* break free from the parent */
         if (ctx->daemon_pipe[1] != -1) {
-                write (ctx->daemon_pipe[1], (void *) &ret, sizeof (ret));
-                close (ctx->daemon_pipe[1]);
+                sys_write (ctx->daemon_pipe[1], (void *) &ret, sizeof (ret));
+                sys_close (ctx->daemon_pipe[1]);
                 ctx->daemon_pipe[1] = -1;
         }
 }
@@ -1441,7 +1529,7 @@ glusterfs_ctx_defaults_init (glusterfs_ctx_t *ctx)
         if (!ctx->logbuf_pool)
                 goto out;
 
-        pthread_mutex_init (&(ctx->lock), NULL);
+        LOCK_INIT (&ctx->lock);
         pthread_mutex_init (&ctx->notify_lock, NULL);
         pthread_cond_init (&ctx->notify_cond, NULL);
 
@@ -1771,7 +1859,7 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
         cmd_args = &ctx->cmd_args;
 
         /* Do this before argp_parse so it can be overridden. */
-        if (access(SECURE_ACCESS_FILE,F_OK) == 0) {
+        if (sys_access (SECURE_ACCESS_FILE, F_OK) == 0) {
                 cmd_args->secure_mgmt = 1;
         }
 
@@ -1823,7 +1911,7 @@ parse_cmdline (int argc, char *argv[], glusterfs_ctx_t *ctx)
 
                 /* Check if the volfile exists, if not give usage output
                    and exit */
-                ret = stat (cmd_args->volfile, &stbuf);
+                ret = sys_stat (cmd_args->volfile, &stbuf);
                 if (ret) {
                         gf_msg ("glusterfs", GF_LOG_CRITICAL, errno,
                                 glusterfsd_msg_16);
@@ -1928,7 +2016,7 @@ glusterfs_pidfile_cleanup (glusterfs_ctx_t *ctx)
                       cmd_args->pid_file);
 
         if (ctx->cmd_args.pid_file) {
-                unlink (ctx->cmd_args.pid_file);
+                sys_unlink (ctx->cmd_args.pid_file);
                 ctx->cmd_args.pid_file = NULL;
         }
 
@@ -1959,7 +2047,7 @@ glusterfs_pidfile_update (glusterfs_ctx_t *ctx)
                 return ret;
         }
 
-        ret = ftruncate (fileno (pidfp), 0);
+        ret = sys_ftruncate (fileno (pidfp), 0);
         if (ret) {
                 gf_msg ("glusterfsd", GF_LOG_ERROR, errno, glusterfsd_msg_20,
                         cmd_args->pid_file);
@@ -2091,7 +2179,7 @@ daemonize (glusterfs_ctx_t *ctx)
         int            ret = -1;
         cmd_args_t    *cmd_args = NULL;
         int            cstatus = 0;
-        int            err = 0;
+        int            err = 1;
 
         cmd_args = &ctx->cmd_args;
 
@@ -2118,8 +2206,8 @@ daemonize (glusterfs_ctx_t *ctx)
         switch (ret) {
         case -1:
                 if (ctx->daemon_pipe[0] != -1) {
-                        close (ctx->daemon_pipe[0]);
-                        close (ctx->daemon_pipe[1]);
+                        sys_close (ctx->daemon_pipe[0]);
+                        sys_close (ctx->daemon_pipe[1]);
                 }
 
                 gf_msg ("daemonize", GF_LOG_ERROR, errno, glusterfsd_msg_24);
@@ -2127,24 +2215,27 @@ daemonize (glusterfs_ctx_t *ctx)
         case 0:
                 /* child */
                 /* close read */
-                close (ctx->daemon_pipe[0]);
+                sys_close (ctx->daemon_pipe[0]);
                 break;
         default:
                 /* parent */
                 /* close write */
-                close (ctx->daemon_pipe[1]);
+                sys_close (ctx->daemon_pipe[1]);
 
                 if (ctx->mnt_pid > 0) {
                         ret = waitpid (ctx->mnt_pid, &cstatus, 0);
-                        if (!(ret == ctx->mnt_pid && cstatus == 0)) {
+                        if (!(ret == ctx->mnt_pid)) {
+                                if (WIFEXITED(cstatus)) {
+                                        err = WEXITSTATUS(cstatus);
+                                } else {
+                                        err = cstatus;
+                                }
                                 gf_msg ("daemonize", GF_LOG_ERROR, 0,
                                         glusterfsd_msg_25);
-                                exit (1);
+                                exit (err);
                         }
                 }
-
-                err = 1;
-                read (ctx->daemon_pipe[0], (void *)&err, sizeof (err));
+                sys_read (ctx->daemon_pipe[0], (void *)&err, sizeof (err));
                 _exit (err);
         }
 
@@ -2161,12 +2252,55 @@ out:
 }
 
 
+#ifdef GF_LINUX_HOST_OS
+static int
+set_oom_score_adj (glusterfs_ctx_t *ctx)
+{
+        int                  ret           = -1;
+        cmd_args_t          *cmd_args      =  NULL;
+        int                  fd            = -1;
+        size_t               oom_score_len =  0;
+        struct oom_api_info *api           =  NULL;
+
+        cmd_args = &ctx->cmd_args;
+
+        if (!cmd_args->oom_score_adj)
+                goto success;
+
+        api = get_oom_api_info();
+        if (!api)
+                goto out;
+
+        fd = open (api->oom_api_file, O_WRONLY);
+        if (fd < 0)
+                goto out;
+
+        oom_score_len = strlen (cmd_args->oom_score_adj);
+        if (sys_write (fd,
+                  cmd_args->oom_score_adj, oom_score_len) != oom_score_len) {
+                sys_close (fd);
+                goto out;
+        }
+
+        if (sys_close (fd) < 0)
+                goto out;
+
+success:
+        ret = 0;
+
+out:
+        return ret;
+}
+#endif
+
+
 int
 glusterfs_process_volfp (glusterfs_ctx_t *ctx, FILE *fp)
 {
         glusterfs_graph_t  *graph = NULL;
         int                 ret = -1;
         xlator_t           *trav = NULL;
+        int                 err = 0;
 
         graph = glusterfs_graph_construct (fp);
         if (!graph) {
@@ -2184,14 +2318,12 @@ glusterfs_process_volfp (glusterfs_ctx_t *ctx, FILE *fp)
 
         ret = glusterfs_graph_prepare (graph, ctx);
         if (ret) {
-                glusterfs_graph_destroy (graph);
                 goto out;
         }
 
         ret = glusterfs_graph_activate (graph, ctx);
 
         if (ret) {
-                glusterfs_graph_destroy (graph);
                 goto out;
         }
 
@@ -2203,8 +2335,11 @@ out:
                 fclose (fp);
 
         if (ret && !ctx->active) {
+                glusterfs_graph_destroy (graph);
                 /* there is some error in setting up the first graph itself */
-                cleanup_and_exit (0);
+                err = -ret;
+                sys_write (ctx->daemon_pipe[1], (void *) &err, sizeof (err));
+                cleanup_and_exit (err);
         }
 
         return ret;
@@ -2312,7 +2447,8 @@ main (int argc, char *argv[])
                 strcpy (cmdlinestr, argv[0]);
                 for (i = 1; i < argc; i++) {
                         strcat (cmdlinestr, " ");
-                        strcat (cmdlinestr, argv[i]);
+                        strncat (cmdlinestr, argv[i],
+                                 (sizeof (cmdlinestr) - 1));
                 }
                 gf_msg (argv[0], GF_LOG_INFO, 0, glusterfsd_msg_30,
                         argv[0], PACKAGE_VERSION, cmdlinestr);
@@ -2330,6 +2466,12 @@ main (int argc, char *argv[])
         if (ret)
                 goto out;
 
+#ifdef GF_LINUX_HOST_OS
+        ret = set_oom_score_adj (ctx);
+        if (ret)
+                goto out;
+#endif
+
 	ctx->env = syncenv_new (0, 0, 0);
         if (!ctx->env) {
                 gf_msg ("", GF_LOG_ERROR, 0, glusterfsd_msg_31);
