@@ -11,11 +11,6 @@
 #ifndef _COMMON_UTILS_H
 #define _COMMON_UTILS_H
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdint.h>
 #include <sys/uio.h>
 #include <netdb.h>
@@ -30,6 +25,10 @@
 #include <limits.h>
 #include <fnmatch.h>
 
+#ifndef ffsll
+#define ffsll(x) __builtin_ffsll(x)
+#endif
+
 void trap (void);
 
 #define GF_UNIVERSAL_ANSWER 42    /* :O */
@@ -42,6 +41,7 @@ void trap (void);
 #include "locking.h"
 #include "mem-pool.h"
 #include "compat-uuid.h"
+#include "iatt.h"
 #include "uuid.h"
 #include "libglusterfs-messages.h"
 
@@ -87,7 +87,10 @@ void trap (void);
  * nfs port in volume status.
  */
 #define GF_NFS3_PORT    2049
+
 #define GF_CLIENT_PORT_CEILING 1024
+#define GF_IANA_PRIV_PORTS_START 49152 /* RFC 6335 */
+#define GF_CLNT_INSECURE_PORT_CEILING (GF_IANA_PRIV_PORTS_START - 1)
 #define GF_PORT_MAX 65535
 
 #define GF_MINUTE_IN_SECONDS 60
@@ -101,6 +104,17 @@ void trap (void);
 /* Default value of signing waiting time to sign a file for bitrot */
 #define SIGNING_TIMEOUT "120"
 
+/* Shard */
+#define GF_XATTR_SHARD_FILE_SIZE  "trusted.glusterfs.shard.file-size"
+#define SHARD_ROOT_GFID "be318638-e8a0-4c6d-977d-7a937aa84806"
+
+/* Lease: buffer length for stringified lease id
+ * Format: 4hexnum-4hexnum-4hexnum-4hexnum-4hexnum-4hexnum-4hexnum-4hexnum
+ * Eg:6c69-6431-2d63-6c6e-7431-0000-0000-0000
+ */
+#define GF_LEASE_ID_BUF_SIZE  ((LEASE_ID_SIZE * 2) +     \
+                               (LEASE_ID_SIZE / 2))
+
 enum _gf_boolean
 {
 	_gf_false = 0,
@@ -111,10 +125,9 @@ enum _gf_boolean
  * we could have initialized these as +ve values and treated
  * them as negative while comparing etc.. (which would have
  * saved us with the pain of assigning values), but since we
- * only have a couple of clients that use this feature, it's
- * okay.
+ * only have a few clients that use this feature, it's okay.
  */
-enum _gf_client_pid
+enum _gf_special_pid
 {
         GF_CLIENT_PID_MAX               =  0,
         GF_CLIENT_PID_GSYNCD            = -1,
@@ -122,11 +135,12 @@ enum _gf_client_pid
         GF_CLIENT_PID_DEFRAG            = -3,
         GF_CLIENT_PID_NO_ROOT_SQUASH    = -4,
         GF_CLIENT_PID_QUOTA_MOUNT       = -5,
-        GF_CLIENT_PID_AFR_SELF_HEALD    = -6,
+        GF_CLIENT_PID_SELF_HEALD        = -6,
         GF_CLIENT_PID_GLFS_HEAL         = -7,
         GF_CLIENT_PID_BITD              = -8,
         GF_CLIENT_PID_SCRUB             = -9,
-        GF_CLIENT_PID_TIER_DEFRAG       = -10
+        GF_CLIENT_PID_TIER_DEFRAG       = -10,
+        GF_SERVER_PID_TRASH             = -11
 };
 
 enum _gf_xlator_ipc_targets {
@@ -135,7 +149,7 @@ enum _gf_xlator_ipc_targets {
 };
 
 typedef enum _gf_boolean gf_boolean_t;
-typedef enum _gf_client_pid gf_client_pid_t;
+typedef enum _gf_special_pid gf_special_pid_t;
 typedef enum _gf_xlator_ipc_targets _gf_xlator_ipc_targets_t;
 
 /* The DHT file rename operation is not a straightforward rename.
@@ -164,6 +178,42 @@ typedef struct dht_changelog_rename_info {
 
 typedef int (*gf_cmp) (void *, void *);
 
+struct _dict;
+
+struct dnscache {
+        struct _dict *cache_dict;
+        time_t ttl;
+};
+
+struct dnscache_entry {
+        char *ip;
+        char *fqdn;
+        time_t timestamp;
+};
+
+struct dnscache6 {
+        struct addrinfo *first;
+        struct addrinfo *next;
+};
+
+struct list_node {
+        void *ptr;
+        struct list_head list;
+};
+
+struct list_node *list_node_add (void *ptr, struct list_head *list);
+struct list_node *list_node_add_order (void *ptr, struct list_head *list,
+                                       int (*compare)(struct list_head *,
+                                            struct list_head *));
+void list_node_del (struct list_node *node);
+
+struct dnscache *gf_dnscache_init (time_t ttl);
+struct dnscache_entry *gf_dnscache_entry_init ();
+void gf_dnscache_entry_deinit (struct dnscache_entry *entry);
+char *gf_rev_dns_lookup_cached (const char *ip, struct dnscache *dnscache);
+
+char *gf_resolve_path_parent (const char *path);
+
 void gf_global_variable_init(void);
 
 int32_t gf_resolve_ip6 (const char *hostname, uint16_t port, int family,
@@ -339,6 +389,8 @@ union gf_sock_union {
 };
 
 #define GF_HIDDEN_PATH ".glusterfs"
+#define GF_UNLINK_PATH GF_HIDDEN_PATH"/unlink"
+#define GF_LANDFILL_PATH GF_HIDDEN_PATH"/landfill"
 
 #define IOV_MIN(n) min(IOV_MAX,n)
 
@@ -346,11 +398,11 @@ union gf_sock_union {
         do {\
                 entry = NULL;\
                 if (dir) { \
-                        entry = readdir (dir); \
+                        entry = sys_readdir (dir); \
                         while (entry && (!strcmp (entry->d_name, ".") || \
                             !fnmatch ("*.tmp", entry->d_name, 0) || \
                             !strcmp (entry->d_name, ".."))) { \
-                                entry = readdir (dir); \
+                                entry = sys_readdir (dir); \
                         } \
                 } \
         } while (0)
@@ -410,10 +462,12 @@ iov_subset (struct iovec *orig, int orig_count,
 	int    i;
 	off_t  offset = 0;
 	size_t start_offset = 0;
-	size_t end_offset = 0;
+	size_t end_offset = 0, origin_iov_len = 0;
 
 
 	for (i = 0; i < orig_count; i++) {
+                origin_iov_len = orig[i].iov_len;
+
 		if ((offset + orig[i].iov_len < src_offset)
 		    || (offset > dst_offset)) {
 			goto not_subset;
@@ -441,7 +495,7 @@ iov_subset (struct iovec *orig, int orig_count,
 		new_count++;
 
 	not_subset:
-		offset += orig[i].iov_len;
+		offset += origin_iov_len;
 	}
 
 	return new_count;
@@ -689,6 +743,8 @@ char *uuid_utoa (uuid_t uuid);
 char *uuid_utoa_r (uuid_t uuid, char *dst);
 char *lkowner_utoa (gf_lkowner_t *lkowner);
 char *lkowner_utoa_r (gf_lkowner_t *lkowner, char *dst, int len);
+char *leaseid_utoa (const char *lease_id);
+gf_boolean_t is_valid_lease_id (const char *lease_id);
 
 void gf_array_insertionsort (void *a, int l, int r, size_t elem_size,
                              gf_cmp cmp);
@@ -715,6 +771,9 @@ int gf_set_timestamp  (const char *src, const char* dest);
 
 int gf_thread_create (pthread_t *thread, const pthread_attr_t *attr,
                       void *(*start_routine)(void *), void *arg);
+int gf_thread_create_detached (pthread_t *thread,
+                      void *(*start_routine)(void *), void *arg);
+
 gf_boolean_t
 gf_is_service_running (char *pidfile, int *pid);
 int
@@ -769,4 +828,22 @@ gf_nwrite (int fd, const void *buf, size_t count);
 void _mask_cancellation (void);
 void _unmask_cancellation (void);
 
+gf_boolean_t
+gf_is_zero_filled_stat (struct iatt *buf);
+
+void
+gf_zero_fill_stat (struct iatt *buf);
+
+gf_boolean_t
+gf_is_valid_xattr_namespace (char *k);
+
+const char *
+gf_inode_type_to_str (ia_type_t type);
+
+int32_t
+gf_bits_count (uint64_t n);
+
+int32_t
+gf_bits_index (uint64_t n);
+
 #endif /* _COMMON_UTILS_H */
