@@ -8,11 +8,6 @@
   cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include <errno.h>
 #include <pthread.h>
 #include <stdio.h>
@@ -31,6 +26,8 @@
 
 #include <sys/stat.h>
 
+#include "syscall.h"
+
 #define GF_JSON_MSG_LENGTH      8192
 #define GF_SYSLOG_CEE_FORMAT    \
         "@cee: {\"msg\": \"%s\", \"gf_code\": \"%u\", \"gf_message\": \"%s\"}"
@@ -90,8 +87,10 @@ gf_log_logrotate (int signum)
 
         ctx = THIS->ctx;
 
-        if (ctx)
+        if (ctx) {
                 ctx->log.logrotate = 1;
+                ctx->log.cmd_history_logrotate = 1;
+        }
 }
 
 void
@@ -374,7 +373,7 @@ gf_log_rotate(glusterfs_ctx_t *ctx)
                                 "logfile");
                         return;
                 }
-                close (fd);
+                sys_close (fd);
 
                 new_logfile = fopen (ctx->log.filename, "a");
                 if (!new_logfile) {
@@ -643,13 +642,13 @@ gf_syslog (int facility_priority, char *format, ...)
 }
 
 void
-gf_log_globals_init (void *data)
+gf_log_globals_init (void *data, gf_loglevel_t level)
 {
         glusterfs_ctx_t *ctx = data;
 
         pthread_mutex_init (&ctx->log.logfile_mutex, NULL);
 
-        ctx->log.loglevel         = GF_LOG_INFO;
+        ctx->log.loglevel         = level;
         ctx->log.gf_log_syslog    = 1;
         ctx->log.sys_log_level    = GF_LOG_CRITICAL;
         ctx->log.logger           = gf_logger_glusterlog;
@@ -694,7 +693,7 @@ gf_log_init (void *data, const char *file, const char *ident)
                 gf_openlog (NULL, -1, LOG_DAEMON);
         }
         /* TODO: make FACILITY configurable than LOG_DAEMON */
-        if (stat (GF_LOG_CONTROL_FILE, &buf) == 0) {
+        if (sys_stat (GF_LOG_CONTROL_FILE, &buf) == 0) {
                 /* use syslog logging */
                 ctx->log.log_control_file_found = 1;
         } else {
@@ -746,7 +745,7 @@ gf_log_init (void *data, const char *file, const char *ident)
                          " \"%s\" (%s)\n", file, strerror (errno));
                 return -1;
         }
-        close (fd);
+        sys_close (fd);
 
         ctx->log.logfile = fopen (file, "a");
         if (!ctx->log.logfile) {
@@ -915,6 +914,8 @@ out:
 
         FREE (str2);
 
+        va_end (ap);
+
         return ret;
 }
 
@@ -1239,7 +1240,7 @@ _gf_msg_nomem (const char *domain, const char *file,
 
                         /* write directly to the fd to prevent out of order
                          * message and stack */
-                        ret = write (fd, msg, wlen);
+                        ret = sys_write (fd, msg, wlen);
                         if (ret == -1) {
                                 pthread_mutex_unlock (&ctx->log.logfile_mutex);
                                 goto out;
@@ -2184,7 +2185,7 @@ _gf_log (const char *domain, const char *file, const char *function, int line,
                                 "failed to open logfile");
                         return -1;
                 }
-                close (fd);
+                sys_close (fd);
 
                 new_logfile = fopen (ctx->log.filename, "a");
                 if (!new_logfile) {
@@ -2270,6 +2271,7 @@ err:
         FREE (str2);
 
 out:
+        va_end (ap);
         return (0);
 }
 
@@ -2319,7 +2321,9 @@ out:
         if (str2)
                 FREE (str2);
 
-         return ret;
+        va_end (ap);
+
+        return ret;
 }
 
 int
@@ -2358,7 +2362,7 @@ gf_cmd_log_init (const char *filename)
                         LG_MSG_FILE_OP_FAILED, "failed to open cmd_log_file");
                 return -1;
         }
-        close (fd);
+        sys_close (fd);
 
         ctx->log.cmdlogfile = fopen (ctx->log.cmd_log_filename, "a");
         if (!ctx->log.cmdlogfile){
@@ -2382,6 +2386,7 @@ gf_cmd_log (const char *domain, const char *fmt, ...)
         char          *msg  = NULL;
         size_t         len  = 0;
         int            ret  = 0;
+        int            fd   = -1;
         glusterfs_ctx_t *ctx = NULL;
 
         ctx = THIS->ctx;
@@ -2430,6 +2435,36 @@ gf_cmd_log (const char *domain, const char *fmt, ...)
         strcpy (msg, str1);
         strcpy (msg + len, str2);
 
+        /* close and reopen cmdlogfile fd for in case of log rotate*/
+        if (ctx->log.cmd_history_logrotate) {
+                ctx->log.cmd_history_logrotate = 0;
+
+                if (ctx->log.cmdlogfile) {
+                        fclose (ctx->log.cmdlogfile);
+                        ctx->log.cmdlogfile = NULL;
+                }
+
+                fd = open (ctx->log.cmd_log_filename,
+                           O_CREAT | O_WRONLY | O_APPEND, S_IRUSR | S_IWUSR);
+                if (fd < 0) {
+                        gf_msg (THIS->name, GF_LOG_CRITICAL, errno,
+                                LG_MSG_FILE_OP_FAILED, "failed to open "
+                                "logfile \"%s\" \n", ctx->log.cmd_log_filename);
+                        ret = -1;
+                        goto out;
+                }
+
+                ctx->log.cmdlogfile = fdopen (fd, "a");
+                if (!ctx->log.cmdlogfile) {
+                        gf_msg (THIS->name, GF_LOG_CRITICAL, errno,
+                                LG_MSG_FILE_OP_FAILED,
+                                "failed to open logfile \"%s\""
+                                " \n", ctx->log.cmd_log_filename);
+                        ret = -1;
+                        goto out;
+                }
+        }
+
         fprintf (ctx->log.cmdlogfile, "%s\n", msg);
         fflush (ctx->log.cmdlogfile);
 
@@ -2440,5 +2475,7 @@ out:
 
         FREE (str2);
 
-        return (0);
+        va_end (ap);
+
+        return ret;
 }
