@@ -226,6 +226,7 @@ gf_vasprintf (char **string_ptr, const char *format, va_list arg)
         rv = vsnprintf (str, size, format, arg_save);
 
         *string_ptr = str;
+        va_end (arg_save);
         return (rv);
 }
 
@@ -317,8 +318,8 @@ __gf_free (void *free_ptr)
         {
                 mem_acct->rec[header->type].size -= header->size;
                 mem_acct->rec[header->type].num_allocs--;
-                /* If all the instaces are freed up then ensure typestr is
-                 * set to NULL */
+                /* If all the instances are freed up then ensure typestr is set
+                 * to NULL */
                 if (!mem_acct->rec[header->type].num_allocs)
                         mem_acct->rec[header->type].typestr = NULL;
         }
@@ -337,6 +338,26 @@ free:
 }
 
 
+/* Based on the mem-type that is used for the allocation, GF_FREE can be
+ * called, or something more intelligent for the structure can be done.
+ */
+int
+gf_get_mem_type (void *ptr)
+{
+        struct mem_header *header = NULL;
+
+        if (!ptr || !THIS->ctx->mem_acct_enable)
+                return 0;
+
+        header = (struct mem_header *) (ptr - GF_MEM_HEADER_SIZE);
+
+        /* Possible corruption, assert here */
+        GF_ASSERT (GF_MEM_HEADER_MAGIC == header->magic);
+
+        return header->type;
+}
+
+
 
 struct mem_pool *
 mem_pool_new_fn (unsigned long sizeof_type,
@@ -401,7 +422,11 @@ mem_pool_new_fn (unsigned long sizeof_type,
         if (!ctx)
                 goto out;
 
-        list_add (&mem_pool->global_list, &ctx->mempool_list);
+        LOCK (&ctx->lock);
+        {
+                list_add (&mem_pool->global_list, &ctx->mempool_list);
+        }
+        UNLOCK (&ctx->lock);
 
 out:
         return mem_pool;
