@@ -16,17 +16,12 @@
 #ifndef _STACK_H
 #define _STACK_H
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 struct _call_stack_t;
 typedef struct _call_stack_t call_stack_t;
 struct _call_frame_t;
 typedef struct _call_frame_t call_frame_t;
-struct _call_pool_t;
-typedef struct _call_pool_t call_pool_t;
+struct call_pool;
+typedef struct call_pool call_pool_t;
 
 #include <sys/time.h>
 
@@ -36,6 +31,8 @@ typedef struct _call_pool_t call_pool_t;
 #include "common-utils.h"
 #include "globals.h"
 #include "lkowner.h"
+#include "client_t.h"
+#include "libglusterfs-messages.h"
 
 #define NFS_PID 1
 #define LOW_PRIO_PROC_PID -1
@@ -46,7 +43,7 @@ typedef int32_t (*ret_fn_t) (call_frame_t *frame,
                              int32_t op_errno,
                              ...);
 
-struct _call_pool_t {
+struct call_pool {
         union {
                 struct list_head   all_frames;
                 struct {
@@ -63,8 +60,7 @@ struct _call_pool_t {
 struct _call_frame_t {
         call_stack_t *root;        /* stack root */
         call_frame_t *parent;      /* previous BP */
-        call_frame_t *next;
-        call_frame_t *prev;        /* maintenance list */
+        struct list_head frames;
         void         *local;       /* local variables */
         xlator_t     *this;        /* implicit object */
         ret_fn_t      ret;         /* op_return address */
@@ -94,12 +90,13 @@ struct _call_stack_t {
         };
         call_pool_t                  *pool;
         gf_lock_t                     stack_lock;
-        void                         *trans;
+        client_t                     *client;
         uint64_t                      unique;
         void                         *state;  /* pointer to request state */
         uid_t                         uid;
         gid_t                         gid;
         pid_t                         pid;
+        char                          identifier[UNIX_PATH_MAX];
         uint16_t                      ngrps;
         uint32_t                      groups_small[SMALL_GROUP_COUNT];
 	uint32_t                     *groups_large;
@@ -107,7 +104,8 @@ struct _call_stack_t {
         gf_lkowner_t                  lk_owner;
         glusterfs_ctx_t              *ctx;
 
-        call_frame_t                  frames;
+        struct list_head              myframes; /* List of call_frame_t that go
+                                                   to make the call stack */
 
         int32_t                       op;
         int8_t                        type;
@@ -137,10 +135,8 @@ static inline void
 FRAME_DESTROY (call_frame_t *frame)
 {
         void *local = NULL;
-        if (frame->next)
-                frame->next->prev = frame->prev;
-        if (frame->prev)
-                frame->prev->next = frame->next;
+
+        list_del_init (&frame->frames);
         if (frame->local) {
                 local = frame->local;
                 frame->local = NULL;
@@ -159,6 +155,8 @@ static inline void
 STACK_DESTROY (call_stack_t *stack)
 {
         void *local = NULL;
+        call_frame_t *frame = NULL;
+        call_frame_t *tmp = NULL;
 
         LOCK (&stack->pool->lock);
         {
@@ -167,16 +165,10 @@ STACK_DESTROY (call_stack_t *stack)
         }
         UNLOCK (&stack->pool->lock);
 
-        if (stack->frames.local) {
-                local = stack->frames.local;
-                stack->frames.local = NULL;
-        }
-
-        LOCK_DESTROY (&stack->frames.lock);
         LOCK_DESTROY (&stack->stack_lock);
 
-        while (stack->frames.next) {
-                FRAME_DESTROY (stack->frames.next);
+        list_for_each_entry_safe (frame, tmp, &stack->myframes, frames) {
+                FRAME_DESTROY (frame);
         }
 
 	GF_FREE (stack->groups_large);
@@ -191,14 +183,28 @@ static inline void
 STACK_RESET (call_stack_t *stack)
 {
         void *local = NULL;
+        call_frame_t *frame = NULL;
+        call_frame_t *tmp = NULL;
+        call_frame_t *last = NULL;
+        struct list_head toreset = {0};
 
-        if (stack->frames.local) {
-                local = stack->frames.local;
-                stack->frames.local = NULL;
+        INIT_LIST_HEAD (&toreset);
+
+        /* We acquire call_pool->lock only to remove the frames from this stack
+         * to preserve atomicity. This synchronizes across concurrent requests
+         * like statedump, STACK_DESTROY etc. */
+
+        LOCK (&stack->pool->lock);
+        {
+                last = list_last_entry (&stack->myframes, call_frame_t, frames);
+                list_del_init (&last->frames);
+                list_splice_init (&stack->myframes, &toreset);
+                list_add (&last->frames, &stack->myframes);
         }
+        UNLOCK (&stack->pool->lock);
 
-        while (stack->frames.next) {
-                FRAME_DESTROY (stack->frames.next);
+        list_for_each_entry_safe (frame, tmp, &toreset, frames) {
+                FRAME_DESTROY (frame);
         }
 
         if (local)
@@ -232,7 +238,6 @@ STACK_RESET (call_stack_t *stack)
                                                                         \
                 _new = mem_get0 (frame->root->pool->frame_mem_pool);    \
                 if (!_new) {                                            \
-                        gf_log ("stack", GF_LOG_ERROR, "alloc failed"); \
                         break;                                          \
                 }                                                       \
                 typeof(fn##_cbk) tmp_cbk = rfn;                         \
@@ -248,16 +253,17 @@ STACK_RESET (call_stack_t *stack)
                 LOCK_INIT (&_new->lock);                                \
                 LOCK(&frame->root->stack_lock);                         \
                 {                                                       \
-                        _new->next = frame->root->frames.next;          \
-                        _new->prev = &frame->root->frames;              \
-                        if (frame->root->frames.next)                   \
-                                frame->root->frames.next->prev = _new;  \
-                        frame->root->frames.next = _new;                \
+                        list_add (&_new->frames, &frame->root->myframes);\
                         frame->ref_count++;                             \
                 }                                                       \
                 UNLOCK(&frame->root->stack_lock);                       \
                 old_THIS = THIS;                                        \
                 THIS = obj;                                             \
+                gf_msg_trace ("stack-trace", 0,                         \
+                              "stack-address: %p, "                     \
+                              "winding from %s to %s",                  \
+                              frame->root, old_THIS->name,              \
+                              THIS->name);                              \
                 if (frame->this->ctx->measure_latency)                  \
                         gf_latency_begin (_new, fn);                    \
                 fn (_new, obj, params);                                 \
@@ -269,12 +275,19 @@ STACK_RESET (call_stack_t *stack)
 #define STACK_WIND_TAIL(frame, obj, fn, params ...)                     \
         do {                                                            \
                 xlator_t     *old_THIS = NULL;                          \
+                xlator_t     *next_xl = obj;                            \
+                typeof(fn)    next_xl_fn = fn;                          \
                                                                         \
-                frame->this = obj;                                      \
+                frame->this = next_xl;                                  \
                 frame->wind_to = #fn;                                   \
                 old_THIS = THIS;                                        \
-                THIS = obj;                                             \
-                fn (frame, obj, params);                                \
+                THIS = next_xl;                                         \
+                gf_msg_trace ("stack-trace", 0,                         \
+                              "stack-address: %p, "                     \
+                              "winding from %s to %s",                  \
+                              frame->root, old_THIS->name,              \
+                              THIS->name);                              \
+                next_xl_fn (frame, next_xl, params);                    \
                 THIS = old_THIS;                                        \
         } while (0)
 
@@ -287,7 +300,6 @@ STACK_RESET (call_stack_t *stack)
                                                                         \
                 _new = mem_get0 (frame->root->pool->frame_mem_pool);    \
                 if (!_new) {                                            \
-                        gf_log ("stack", GF_LOG_ERROR, "alloc failed"); \
                         break;                                          \
                 }                                                       \
                 typeof(fn##_cbk) tmp_cbk = rfn;                         \
@@ -302,17 +314,18 @@ STACK_RESET (call_stack_t *stack)
                 LOCK_INIT (&_new->lock);                                \
                 LOCK(&frame->root->stack_lock);                         \
                 {                                                       \
+                        list_add (&_new->frames, &frame->root->myframes);\
                         frame->ref_count++;                             \
-                        _new->next = frame->root->frames.next;          \
-                        _new->prev = &frame->root->frames;              \
-                        if (frame->root->frames.next)                   \
-                                frame->root->frames.next->prev = _new;  \
-                        frame->root->frames.next = _new;                \
                 }                                                       \
                 UNLOCK(&frame->root->stack_lock);                       \
                 fn##_cbk = rfn;                                         \
                 old_THIS = THIS;                                        \
                 THIS = obj;                                             \
+                gf_msg_trace ("stack-trace", 0,                         \
+                              "stack-address: %p, "                     \
+                              "winding from %s to %s",                  \
+                              frame->root, old_THIS->name,              \
+                              THIS->name);                              \
                 if (obj->ctx->measure_latency)                          \
                         gf_latency_begin (_new, fn);                    \
                 fn (_new, obj, params);                                 \
@@ -321,15 +334,30 @@ STACK_RESET (call_stack_t *stack)
 
 
 /* return from function */
-#define STACK_UNWIND(frame, params ...)                                 \
+#define STACK_UNWIND(frame, op_ret, op_errno, params ...)               \
         do {                                                            \
                 ret_fn_t      fn = NULL;                                \
                 call_frame_t *_parent = NULL;                           \
                 xlator_t     *old_THIS = NULL;                          \
                 if (!frame) {                                           \
-                        gf_log ("stack", GF_LOG_CRITICAL, "!frame");    \
+                        gf_msg ("stack", GF_LOG_CRITICAL, 0,            \
+                                LG_MSG_FRAME_ERROR, "!frame");          \
                         break;                                          \
                 }                                                       \
+                if (op_ret < 0) {                                       \
+                        gf_msg_debug ("stack-trace", op_errno,          \
+                                      "stack-address: %p, "             \
+                                      "%s returned %d error: %s",       \
+                                      frame->root, THIS->name,          \
+                                      (int32_t)op_ret,                  \
+                                      strerror(op_errno));              \
+                } else {                                                \
+                        gf_msg_trace ("stack-trace", 0,                 \
+                                      "stack-address: %p, "             \
+                                      "%s returned %d",                 \
+                                      frame->root, THIS->name,          \
+                                      (int32_t)op_ret);                 \
+                }                                                       \
                 fn = frame->ret;                                        \
                 _parent = frame->parent;                                \
                 LOCK(&frame->root->stack_lock);                         \
@@ -343,22 +371,38 @@ STACK_RESET (call_stack_t *stack)
                 frame->unwind_from = __FUNCTION__;                      \
                 if (frame->this->ctx->measure_latency)                  \
                         gf_latency_end (frame);                         \
-                fn (_parent, frame->cookie, _parent->this, params);     \
+                fn (_parent, frame->cookie, _parent->this, op_ret,      \
+                    op_errno, params);                                  \
                 THIS = old_THIS;                                        \
         } while (0)
 
 
 /* return from function in type-safe way */
-#define STACK_UNWIND_STRICT(op, frame, params ...)                      \
+#define STACK_UNWIND_STRICT(op, frame, op_ret, op_errno, params ...)    \
         do {                                                            \
                 fop_##op##_cbk_t      fn = NULL;                        \
                 call_frame_t *_parent = NULL;                           \
                 xlator_t     *old_THIS = NULL;                          \
                                                                         \
                 if (!frame) {                                           \
-                        gf_log ("stack", GF_LOG_CRITICAL, "!frame");    \
+                        gf_msg ("stack", GF_LOG_CRITICAL, 0,            \
+                                LG_MSG_FRAME_ERROR, "!frame");          \
                         break;                                          \
                 }                                                       \
+                if (op_ret < 0) {                                       \
+                        gf_msg_debug ("stack-trace", op_errno,          \
+                                      "stack-address: %p, "             \
+                                      "%s returned %d error: %s",       \
+                                      frame->root, THIS->name,          \
+                                      (int32_t)op_ret,                  \
+                                      strerror(op_errno));              \
+                } else {                                                \
+                        gf_msg_trace ("stack-trace", 0,                 \
+                                      "stack-address: %p, "             \
+                                      "%s returned %d",                 \
+                                      frame->root, THIS->name,          \
+                                      (int32_t)op_ret);                 \
+                }                                                       \
                 fn = (fop_##op##_cbk_t )frame->ret;                     \
                 _parent = frame->parent;                                \
                 LOCK(&frame->root->stack_lock);                         \
@@ -372,7 +416,8 @@ STACK_RESET (call_stack_t *stack)
                 frame->unwind_from = __FUNCTION__;                      \
                 if (frame->this->ctx->measure_latency)                  \
                         gf_latency_end (frame);                         \
-                fn (_parent, frame->cookie, _parent->this, params);     \
+                fn (_parent, frame->cookie, _parent->this, op_ret,      \
+                    op_errno, params);                                  \
                 THIS = old_THIS;                                        \
         } while (0)
 
@@ -395,11 +440,27 @@ call_stack_alloc_groups (call_stack_t *stack, int ngrps)
 	return 0;
 }
 
+static inline
+int call_frames_count (call_stack_t *call_stack)
+{
+        call_frame_t *pos;
+        int32_t count = 0;
+
+        if (!call_stack)
+                return count;
+
+        list_for_each_entry (pos, &call_stack->myframes, frames)
+                count++;
+
+        return count;
+}
+
 static inline call_frame_t *
 copy_frame (call_frame_t *frame)
 {
         call_stack_t *newstack = NULL;
         call_stack_t *oldstack = NULL;
+        call_frame_t *newframe = NULL;
 
         if (!frame) {
                 return NULL;
@@ -410,6 +471,19 @@ copy_frame (call_frame_t *frame)
                 return NULL;
         }
 
+        INIT_LIST_HEAD (&newstack->myframes);
+
+        newframe = mem_get0 (frame->root->pool->frame_mem_pool);
+        if (!newframe) {
+                mem_put (newstack);
+                return NULL;
+        }
+
+        newframe->this = frame->this;
+        newframe->root = newstack;
+        INIT_LIST_HEAD (&newframe->frames);
+        list_add (&newframe->frames, &newstack->myframes);
+
         oldstack = frame->root;
 
         newstack->uid = oldstack->uid;
@@ -425,22 +499,20 @@ copy_frame (call_frame_t *frame)
         memcpy (newstack->groups, oldstack->groups,
                 sizeof (gid_t) * oldstack->ngrps);
         newstack->unique = oldstack->unique;
-
-        newstack->frames.this = frame->this;
-        newstack->frames.root = newstack;
         newstack->pool = oldstack->pool;
         newstack->lk_owner = oldstack->lk_owner;
         newstack->ctx = oldstack->ctx;
 
         if (newstack->ctx->measure_latency) {
                 if (gettimeofday (&newstack->tv, NULL) == -1)
-                        gf_log ("stack", GF_LOG_ERROR, "gettimeofday () failed."
-                                " (%s)", strerror (errno));
-                memcpy (&newstack->frames.begin, &newstack->tv,
+                        gf_msg ("stack", GF_LOG_ERROR, errno,
+                                LG_MSG_GETTIMEOFDAY_FAILED,
+                                "gettimeofday () failed.");
+                memcpy (&newframe->begin, &newstack->tv,
                         sizeof (newstack->tv));
         }
 
-        LOCK_INIT (&newstack->frames.lock);
+        LOCK_INIT (&newframe->lock);
         LOCK_INIT (&newstack->stack_lock);
 
         LOCK (&oldstack->pool->lock);
@@ -450,7 +522,7 @@ copy_frame (call_frame_t *frame)
         }
         UNLOCK (&oldstack->pool->lock);
 
-        return &newstack->frames;
+        return newframe;
 }
 
 void gf_proc_dump_pending_frames(call_pool_t *call_pool);
