@@ -8,11 +8,6 @@
   cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "xlator.h"
 #include <dlfcn.h>
 #include <netdb.h>
@@ -85,6 +80,10 @@ fill_defaults (xlator_t *xl)
 	SET_DEFAULT_FOP (discard);
         SET_DEFAULT_FOP (zerofill);
         SET_DEFAULT_FOP (ipc);
+        SET_DEFAULT_FOP (seek);
+        SET_DEFAULT_FOP (lease);
+        SET_DEFAULT_FOP (getactivelk);
+        SET_DEFAULT_FOP (setactivelk);
 
         SET_DEFAULT_FOP (getspec);
 
@@ -128,7 +127,12 @@ xlator_volopt_dynload (char *xlator_type, void **dl_handle,
 
         GF_VALIDATE_OR_GOTO ("xlator", xlator_type, out);
 
-        ret = gf_asprintf (&name, "%s/%s.so", XLATORDIR, xlator_type);
+        /* socket.so doesn't fall under the default xlator directory, hence we
+         * need this check */
+        if (!strstr(xlator_type, "rpc-transport"))
+                ret = gf_asprintf (&name, "%s/%s.so", XLATORDIR, xlator_type);
+        else
+                ret = gf_asprintf (&name, "%s/%s.so", XLATORPARENTDIR, xlator_type);
         if (-1 == ret) {
                 goto out;
         }
@@ -414,6 +418,7 @@ xlator_init (xlator_t *xl)
         if (xl->mem_acct_init)
                 xl->mem_acct_init (xl);
 
+        xl->instance_name = NULL;
         if (!xl->init) {
                 gf_msg (xl->name, GF_LOG_WARNING, 0, LG_MSG_INIT_FAILED,
                         "No init() found");
@@ -757,6 +762,23 @@ out:
         return;
 }
 
+void
+loc_pargfid (loc_t *loc, uuid_t gfid)
+{
+        if (!gfid)
+                goto out;
+        gf_uuid_clear (gfid);
+
+        if (!loc)
+                goto out;
+        else if (!gf_uuid_is_null (loc->pargfid))
+                gf_uuid_copy (gfid, loc->pargfid);
+        else if (loc->parent && (!gf_uuid_is_null (loc->parent->gfid)))
+                gf_uuid_copy (gfid, loc->parent->gfid);
+out:
+        return;
+}
+
 char*
 loc_gfid_utoa (loc_t *loc)
 {
