@@ -11,11 +11,6 @@
 #ifndef _XLATOR_H
 #define _XLATOR_H
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdio.h>
 #include <stdint.h>
 #include <inttypes.h>
@@ -27,8 +22,10 @@
 #include "compat.h"
 #include "list.h"
 #include "latency.h"
+#include "compat-uuid.h"
 
 #define FIRST_CHILD(xl) (xl->children->xlator)
+#define SECOND_CHILD(xl) (xl->children->next->xlator)
 
 #define GF_SET_ATTR_MODE  0x1
 #define GF_SET_ATTR_UID   0x2
@@ -442,6 +439,34 @@ typedef int32_t (*fop_zerofill_cbk_t) (call_frame_t *frame,
                                       struct iatt *preop_stbuf,
                                       struct iatt *postop_stbuf, dict_t *xdata);
 
+typedef int32_t (*fop_ipc_cbk_t) (call_frame_t *frame, void *cookie,
+                                 xlator_t *this, int32_t op_ret,
+                                 int32_t op_errno, dict_t *xdata);
+
+typedef int32_t (*fop_seek_cbk_t) (call_frame_t *frame, void *cookie,
+                                   xlator_t *this, int32_t op_ret,
+                                   int32_t op_errno, off_t offset,
+                                   dict_t *xdata);
+
+typedef int32_t (*fop_lease_cbk_t) (call_frame_t *frame, void *cookie,
+                                    xlator_t *this, int32_t op_ret,
+                                    int32_t op_errno, struct gf_lease *lease,
+                                    dict_t *xdata);
+typedef int32_t (*fop_compound_cbk_t) (call_frame_t *frame,
+                                       void *cookie, xlator_t *this,
+                                       int32_t op_ret, int32_t op_errno,
+                                       void *data, dict_t *xdata);
+
+typedef int32_t (*fop_getactivelk_cbk_t) (call_frame_t *frame, void *cookie,
+                                           xlator_t *this, int32_t op_ret,
+                                           int32_t op_errno,
+                                           lock_migration_info_t *locklist,
+                                           dict_t *xdata);
+
+typedef int32_t (*fop_setactivelk_cbk_t) (call_frame_t *frame, void *cookie,
+                                            xlator_t *this, int32_t op_ret,
+                                            int32_t op_errno, dict_t *xdata);
+
 typedef int32_t (*fop_lookup_t) (call_frame_t *frame,
                                  xlator_t *this,
                                  loc_t *loc,
@@ -673,6 +698,7 @@ typedef int32_t (*fop_discard_t) (call_frame_t *frame,
 				  off_t offset,
 				  size_t len,
                                   dict_t *xdata);
+
 typedef int32_t (*fop_zerofill_t) (call_frame_t *frame,
                                   xlator_t *this,
                                   fd_t *fd,
@@ -680,6 +706,27 @@ typedef int32_t (*fop_zerofill_t) (call_frame_t *frame,
                                   off_t len,
                                   dict_t *xdata);
 
+typedef int32_t (*fop_ipc_t) (call_frame_t *frame, xlator_t *this, int32_t op,
+                              dict_t *xdata);
+
+typedef int32_t (*fop_seek_t) (call_frame_t *frame, xlator_t *this, fd_t *fd,
+                               off_t offset, gf_seek_what_t what,
+                               dict_t *xdata);
+
+typedef int32_t (*fop_lease_t) (call_frame_t *frame, xlator_t *this, loc_t *loc,
+                                struct gf_lease *lease, dict_t *xdata);
+
+typedef int32_t (*fop_compound_t) (call_frame_t *frame, xlator_t *this,
+                                   void *args, dict_t *xdata);
+
+typedef int32_t (*fop_getactivelk_t) (call_frame_t *frame, xlator_t *this,
+                                       loc_t *loc, dict_t *xdata);
+
+typedef int32_t (*fop_setactivelk_t) (call_frame_t *frame, xlator_t *this,
+                                      loc_t *loc,
+                                      lock_migration_info_t *locklist,
+                                      dict_t *xdata);
+
 struct xlator_fops {
         fop_lookup_t         lookup;
         fop_stat_t           stat;
@@ -726,6 +773,12 @@ struct xlator_fops {
 	fop_fallocate_t	     fallocate;
 	fop_discard_t	     discard;
         fop_zerofill_t       zerofill;
+        fop_ipc_t            ipc;
+        fop_seek_t           seek;
+        fop_lease_t          lease;
+        fop_compound_t       compound;
+        fop_getactivelk_t   getactivelk;
+        fop_setactivelk_t  setactivelk;
 
         /* these entries are used for a typechecking hack in STACK_WIND _only_ */
         fop_lookup_cbk_t         lookup_cbk;
@@ -773,6 +826,12 @@ struct xlator_fops {
 	fop_fallocate_cbk_t	 fallocate_cbk;
 	fop_discard_cbk_t	 discard_cbk;
         fop_zerofill_cbk_t       zerofill_cbk;
+        fop_ipc_cbk_t            ipc_cbk;
+        fop_seek_cbk_t           seek_cbk;
+        fop_lease_cbk_t          lease_cbk;
+        fop_compound_cbk_t       compound_cbk;
+        fop_getactivelk_cbk_t   getactivelk_cbk;
+        fop_setactivelk_cbk_t  setactivelk_cbk;
 };
 
 typedef int32_t (*cbk_forget_t) (xlator_t *this,
@@ -785,6 +844,9 @@ typedef int32_t (*cbk_invalidate_t)(xlator_t *this, inode_t *inode);
 
 typedef int32_t (*cbk_client_t)(xlator_t *this, client_t *client);
 
+typedef void (*cbk_ictxmerge_t) (xlator_t *this, fd_t *fd,
+                                 inode_t *inode, inode_t *linked_inode);
+
 struct xlator_cbks {
         cbk_forget_t             forget;
         cbk_release_t            release;
@@ -792,6 +854,7 @@ struct xlator_cbks {
 	cbk_invalidate_t         invalidate;
         cbk_client_t             client_destroy;
         cbk_client_t             client_disconnect;
+        cbk_ictxmerge_t          ictxmerge;
 };
 
 typedef int32_t (*dumpop_priv_t) (xlator_t *this);
@@ -842,6 +905,7 @@ struct _xlator {
         /* Built during parsing */
         char          *name;
         char          *type;
+        char          *instance_name;  /* Used for multi NFSd */
         xlator_t      *next;
         xlator_t      *prev;
         xlator_list_t *parents;
@@ -873,7 +937,7 @@ struct _xlator {
         inode_table_t      *itable;
         char                init_succeeded;
         void               *private;
-        struct mem_acct     mem_acct;
+        struct mem_acct    *mem_acct;
         uint64_t            winds;
         char                switched;
 
@@ -918,7 +982,8 @@ int xlator_init (xlator_t *this);
 int xlator_destroy (xlator_t *xl);
 
 int32_t xlator_tree_init (xlator_t *xl);
-int32_t xlator_tree_free (xlator_t *xl);
+int32_t xlator_tree_free_members (xlator_t *xl);
+int32_t xlator_tree_free_memacct (xlator_t *xl);
 
 void xlator_tree_fini (xlator_t *xl);
 
@@ -934,6 +999,9 @@ void xlator_foreach_depth_first (xlator_t *this,
 
 xlator_t *xlator_search_by_name (xlator_t *any, const char *name);
 
+void
+xlator_set_inode_lru_limit (xlator_t *this, void *data);
+
 void inode_destroy_notify (inode_t *inode, const char *xlname);
 
 int loc_copy (loc_t *dst, loc_t *src);
@@ -943,8 +1011,10 @@ int loc_copy_overload_parent (loc_t *dst,
 void loc_wipe (loc_t *loc);
 int loc_path (loc_t *loc, const char *bname);
 void loc_gfid (loc_t *loc, uuid_t gfid);
+void loc_pargfid (loc_t *loc, uuid_t pargfid);
 char* loc_gfid_utoa (loc_t *loc);
 gf_boolean_t loc_is_root (loc_t *loc);
+int32_t loc_build_child (loc_t *child, loc_t *parent, char *name);
 int xlator_mem_acct_init (xlator_t *xl, int num_types);
 int is_gf_log_command (xlator_t *trans, const char *name, char *value);
 int glusterd_check_log_level (const char *value);
@@ -959,4 +1029,17 @@ is_graph_topology_equal (glusterfs_graph_t *graph1, glusterfs_graph_t *graph2);
 int
 glusterfs_volfile_reconfigure (int oldvollen, FILE *newvolfile_fp,
                                glusterfs_ctx_t *ctx, const char *oldvolfile);
+
+int
+loc_touchup (loc_t *loc, const char *name);
+
+int
+glusterfs_leaf_position(xlator_t *tgt);
+
+int
+glusterfs_reachable_leaves(xlator_t *base, dict_t *leaves);
+
+int
+xlator_subvolume_count (xlator_t *this);
+
 #endif /* _XLATOR_H */
