@@ -1412,7 +1412,8 @@ afr_handle_split_brain_commands (xlator_t *this, call_frame_t *frame,
                 }
                 data->spb_child_index = spb_child_index;
                 data->frame = frame;
-                data->loc = loc;
+                loc_copy (&local->loc, loc);
+                data->loc = &local->loc;
                 ret = synctask_new (this->ctx->env,
                                     afr_can_set_split_brain_choice,
                                     afr_set_split_brain_choice, NULL, data);
