@@ -64,7 +64,6 @@ afr_read_txn_refresh_done (call_frame_t *frame, xlator_t *this, int err)
 {
 	afr_local_t *local = NULL;
 	int read_subvol = 0;
-	int event_generation = 0;
 	inode_t *inode = NULL;
 	int ret = -1;
         int spb_choice = -1;
@@ -76,18 +75,12 @@ afr_read_txn_refresh_done (call_frame_t *frame, xlator_t *this, int err)
                 local->op_errno = -err;
                 local->op_ret = -1;
                 read_subvol = -1;
+                gf_msg (this->name, GF_LOG_ERROR, EIO, AFR_MSG_SPLIT_BRAIN,
+                        "Failing %s on gfid %s: split-brain observed.",
+                        gf_fop_list[local->op], uuid_utoa (inode->gfid));
                 goto readfn;
         }
 
-	ret = afr_inode_get_readable (frame, inode, this, local->readable,
-			              &event_generation,
-				      local->transaction.type);
-
-	if (ret == -1 || !event_generation)
-		/* Even after refresh, we don't have a good
-		   read subvolume. Time to bail */
-                AFR_READ_TXN_SET_ERROR_AND_GOTO (-1, EIO, -1, readfn);
-
 	read_subvol = afr_read_subvol_select_by_policy (inode, this,
 							local->readable, NULL);
 	if (read_subvol == -1)
@@ -121,7 +114,7 @@ afr_read_txn_continue (call_frame_t *frame, xlator_t *this, int subvol)
 
 	if (!local->refreshed) {
 		local->refreshed = _gf_true;
-		afr_inode_refresh (frame, this, local->inode,
+		afr_inode_refresh (frame, this, local->inode, NULL,
 				   afr_read_txn_refresh_done);
 	} else {
 		afr_read_txn_next_subvol (frame, this);
@@ -208,6 +201,7 @@ afr_read_txn (call_frame_t *frame, xlator_t *this, inode_t *inode,
 
 	local->readfn = readfn;
 	local->inode = inode_ref (inode);
+        local->is_read_txn = _gf_true;
 
         if (priv->quorum_reads &&
             priv->quorum_count && !afr_has_quorum (priv->child_up, this)) {
@@ -218,18 +212,12 @@ afr_read_txn (call_frame_t *frame, xlator_t *this, inode_t *inode,
         }
 
 	local->transaction.type = type;
-        if (local->op == GF_FOP_FSTAT || local->op == GF_FOP_STAT) {
-                ret = afr_inode_read_subvol_get (inode, this, data, metadata,
-                                                 &event_generation);
-                AFR_INTERSECT (local->readable, data, metadata,
-                               priv->child_count);
-        } else {
-                ret = afr_inode_read_subvol_type_get (inode, this, local->readable,
-                                                      &event_generation, type);
-        }
+        ret = afr_inode_read_subvol_get (inode, this, data, metadata,
+                                         &event_generation);
 	if (ret == -1)
 		/* very first transaction on this inode */
 		goto refresh;
+        AFR_INTERSECT (local->readable, data, metadata, priv->child_count);
 
         gf_msg_debug (this->name, 0, "%s: generation now vs cached: %d, "
                       "%d", uuid_utoa (inode->gfid), local->event_generation,
@@ -268,7 +256,7 @@ read:
 	return 0;
 
 refresh:
-	afr_inode_refresh (frame, this, inode, afr_read_txn_refresh_done);
+	afr_inode_refresh (frame, this, inode, NULL, afr_read_txn_refresh_done);
 
 	return 0;
 }
