@@ -9,11 +9,6 @@
 */
 
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "afr.h"
 #include "afr-self-heal.h"
 #include "byte-order.h"
@@ -94,9 +89,15 @@ __afr_can_skip_data_block_heal (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
 	priv = this->private;
 	local = frame->local;
+
         xdata = dict_new();
-        if (xdata)
-                i = dict_set_int32 (xdata, "check-zero-filled", 1);
+        if (!xdata)
+                goto out;
+        if (dict_set_int32 (xdata, "check-zero-filled", 1)) {
+                dict_unref (xdata);
+                goto out;
+        }
+
 	wind_subvols = alloca0 (priv->child_count);
 	for (i = 0; i < priv->child_count; i++) {
 		if (i == source || healed_sinks[i])
@@ -135,7 +136,7 @@ __afr_can_skip_data_block_heal (call_frame_t *frame, xlator_t *this, fd_t *fd,
                 else
                         return _gf_true;
         }
-
+out:
         return _gf_false;
 }
 
@@ -303,6 +304,9 @@ afr_selfheal_data_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	local = frame->local;
 	priv = this->private;
 
+        if (!priv->ensure_durability)
+                return 0;
+
 	AFR_ONLIST (healed_sinks, frame, attr_cbk, fsync, fd, 0, NULL);
 
 	for (i = 0; i < priv->child_count; i++)
@@ -371,15 +375,22 @@ afr_selfheal_data_do (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	int type = AFR_SELFHEAL_DATA_FULL;
 	int ret = -1;
 	call_frame_t *iter_frame = NULL;
+        unsigned char arbiter_sink_status = 0;
 
 	priv = this->private;
+        if (priv->arbiter_count) {
+                arbiter_sink_status = healed_sinks[ARBITER_BRICK_INDEX];
+                healed_sinks[ARBITER_BRICK_INDEX] = 0;
+        }
 
         type = afr_data_self_heal_type_get (priv, healed_sinks, source,
                                             replies);
 
 	iter_frame = afr_copy_frame (frame);
-	if (!iter_frame)
-		return -ENOMEM;
+	if (!iter_frame) {
+                ret = -ENOMEM;
+                goto out;
+        }
 
 	for (off = 0; off < replies[source].poststat.ia_size; off += block) {
                 if (AFR_COUNT (healed_sinks, priv->child_count) == 0) {
@@ -400,12 +411,12 @@ afr_selfheal_data_do (call_frame_t *frame, xlator_t *this, fd_t *fd,
                 }
 	}
 
-	afr_selfheal_data_restore_time (frame, this, fd->inode, source,
-					healed_sinks, replies);
-
 	ret = afr_selfheal_data_fsync (frame, this, fd, healed_sinks);
 
 out:
+        if (arbiter_sink_status)
+                healed_sinks[ARBITER_BRICK_INDEX] = arbiter_sink_status;
+
 	if (iter_frame)
 		AFR_STACK_DESTROY (iter_frame);
 	return ret;
@@ -419,11 +430,17 @@ __afr_selfheal_truncate_sinks (call_frame_t *frame, xlator_t *this,
 {
 	afr_local_t *local = NULL;
 	afr_private_t *priv = NULL;
+        unsigned char arbiter_sink_status = 0;
 	int i = 0;
 
 	local = frame->local;
 	priv = this->private;
 
+        if (priv->arbiter_count) {
+                arbiter_sink_status = healed_sinks[ARBITER_BRICK_INDEX];
+                healed_sinks[ARBITER_BRICK_INDEX] = 0;
+        }
+
 	AFR_ONLIST (healed_sinks, frame, attr_cbk, ftruncate, fd, size, NULL);
 
 	for (i = 0; i < priv->child_count; i++)
@@ -432,6 +449,9 @@ __afr_selfheal_truncate_sinks (call_frame_t *frame, xlator_t *this,
 			   as successfully healed. Mark it so.
 			*/
 			healed_sinks[i] = 0;
+
+        if (arbiter_sink_status)
+                healed_sinks[ARBITER_BRICK_INDEX] = arbiter_sink_status;
 	return 0;
 }
 
@@ -472,6 +492,10 @@ afr_does_size_mismatch (xlator_t *this, unsigned char *sources,
                 if (!sources[i])
                         continue;
 
+                if (AFR_IS_ARBITER_BRICK (priv, i) &&
+                    (replies[i].poststat.ia_size == 0))
+                        continue;
+
                 if (!min)
                         min = &replies[i].poststat;
 
@@ -550,14 +574,15 @@ afr_mark_newest_file_as_source (xlator_t *this, unsigned char *sources,
 
 static int
 __afr_selfheal_data_finalize_source (call_frame_t *frame, xlator_t *this,
+                                     inode_t *inode,
                                      unsigned char *sources,
                                      unsigned char *sinks,
 				     unsigned char *healed_sinks,
 				     unsigned char *locked_on,
+				     unsigned char *undid_pending,
 				     struct afr_reply *replies,
                                      uint64_t *witness)
 {
-	int i = 0;
 	afr_private_t *priv = NULL;
 	int source = -1;
 	int sources_count = 0;
@@ -568,13 +593,19 @@ __afr_selfheal_data_finalize_source (call_frame_t *frame, xlator_t *this,
 	if ((AFR_CMP (locked_on, healed_sinks, priv->child_count) == 0)
             || !sources_count) {
 		/* split brain */
-                source = afr_mark_split_brain_source_sinks (frame, this,
+                source = afr_mark_split_brain_source_sinks (frame, this, inode,
                                                             sources, sinks,
                                                             healed_sinks,
                                                             locked_on, replies,
                                                           AFR_DATA_TRANSACTION);
                 if (source < 0)
                         return -EIO;
+
+                _afr_fav_child_reset_sink_xattrs (frame, this, inode, source,
+                                                 healed_sinks, undid_pending,
+                                                 AFR_DATA_TRANSACTION,
+                                                 locked_on, replies);
+
                 return source;
 	}
 
@@ -594,13 +625,9 @@ __afr_selfheal_data_finalize_source (call_frame_t *frame, xlator_t *this,
 
 out:
         afr_mark_active_sinks (this, sources, locked_on, healed_sinks);
+        source = afr_choose_source_by_policy (priv, sources,
+                                              AFR_DATA_TRANSACTION);
 
-        for (i = 0; i < priv->child_count; i++) {
-                if (sources[i]) {
-                        source = i;
-                        break;
-                }
-        }
 	return source;
 }
 
@@ -618,7 +645,8 @@ __afr_selfheal_data_prepare (call_frame_t *frame, xlator_t *this,
                              inode_t *inode, unsigned char *locked_on,
                              unsigned char *sources, unsigned char *sinks,
                              unsigned char *healed_sinks,
-			     struct afr_reply *replies)
+                             unsigned char *undid_pending,
+			     struct afr_reply *replies, gf_boolean_t *pflag)
 {
 	int ret = -1;
 	int source = -1;
@@ -636,7 +664,8 @@ __afr_selfheal_data_prepare (call_frame_t *frame, xlator_t *this,
         witness = alloca0(priv->child_count * sizeof (*witness));
 	ret = afr_selfheal_find_direction (frame, this, replies,
 					   AFR_DATA_TRANSACTION,
-					   locked_on, sources, sinks, witness);
+					   locked_on, sources, sinks, witness,
+                                           pflag);
 	if (ret)
 		return ret;
 
@@ -649,10 +678,11 @@ __afr_selfheal_data_prepare (call_frame_t *frame, xlator_t *this,
         */
         AFR_INTERSECT (healed_sinks, sinks, locked_on, priv->child_count);
 
-	source = __afr_selfheal_data_finalize_source (frame, this, sources,
-                                                      sinks, healed_sinks,
-                                                      locked_on, replies,
-                                                      witness);
+	source = __afr_selfheal_data_finalize_source (frame, this, inode,
+                                                      sources, sinks,
+                                                      healed_sinks,
+                                                      locked_on, undid_pending,
+                                                      replies, witness);
 	if (source < 0)
 		return -EIO;
 
@@ -670,11 +700,11 @@ __afr_selfheal_data (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	unsigned char *sinks = NULL;
 	unsigned char *data_lock = NULL;
 	unsigned char *healed_sinks = NULL;
+	unsigned char *undid_pending = NULL;
 	struct afr_reply *locked_replies = NULL;
 	int source = -1;
-	gf_boolean_t compat = _gf_false;
         gf_boolean_t did_sh = _gf_true;
-	unsigned char *compat_lock = NULL;
+        gf_boolean_t is_arbiter_the_only_sink = _gf_false;
 
 	priv = this->private;
 
@@ -682,7 +712,7 @@ __afr_selfheal_data (call_frame_t *frame, xlator_t *this, fd_t *fd,
 	sinks = alloca0 (priv->child_count);
 	healed_sinks = alloca0 (priv->child_count);
 	data_lock = alloca0 (priv->child_count);
-	compat_lock = alloca0 (priv->child_count);
+        undid_pending = alloca0 (priv->child_count);
 
 	locked_replies = alloca0 (sizeof (*locked_replies) * priv->child_count);
 
@@ -702,8 +732,8 @@ __afr_selfheal_data (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
 		ret = __afr_selfheal_data_prepare (frame, this, fd->inode,
                                                    data_lock, sources, sinks,
-                                                   healed_sinks,
-						   locked_replies);
+                                                   healed_sinks, undid_pending,
+                                                   locked_replies, NULL);
 		if (ret < 0)
 			goto unlock;
 
@@ -714,12 +744,18 @@ __afr_selfheal_data (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
 		source = ret;
 
-                if (AFR_IS_ARBITER_BRICK(priv, source) &&
-                    AFR_COUNT (sources, priv->child_count) == 1) {
+                if (AFR_IS_ARBITER_BRICK(priv, source)) {
                         did_sh = _gf_false;
                         goto unlock;
                 }
 
+                if (priv->arbiter_count &&
+                    AFR_COUNT (healed_sinks, priv->child_count) == 1 &&
+                    healed_sinks[ARBITER_BRICK_INDEX]) {
+                        is_arbiter_the_only_sink = _gf_true;
+                        goto restore_time;
+                }
+
 		ret = __afr_selfheal_truncate_sinks (frame, this, fd, healed_sinks,
 						     locked_replies[source].poststat.ia_size);
 		if (ret < 0)
@@ -727,18 +763,6 @@ __afr_selfheal_data (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
 		ret = 0;
 
-                if (priv->arbiter_count == 0) {/*TODO: delete this code after
-                                                 3.5.x goes out of support*/
-                        /* Locking from (LLONG_MAX - 2) to (LLONG_MAX - 1) is
-                           for compatibility with older self-heal clients which
-                           do not hold a lock in the @priv->sh_domain domain to
-                           guard against concurrent ongoing self-heals
-                        */
-                        afr_selfheal_inodelk (frame, this, fd->inode,
-                                        this->name, LLONG_MAX - 2, 1,
-                                        compat_lock);
-                        compat = _gf_true;
-                }
 	}
 unlock:
 	afr_selfheal_uninodelk (frame, this, fd->inode, this->name, 0, 0,
@@ -752,19 +776,32 @@ unlock:
 	ret = afr_selfheal_data_do (frame, this, fd, source, healed_sinks,
 				    locked_replies);
 	if (ret)
-		goto out;
+                goto out;
+restore_time:
+	afr_selfheal_data_restore_time (frame, this, fd->inode, source,
+					healed_sinks, locked_replies);
 
-	ret = afr_selfheal_undo_pending (frame, this, fd->inode, sources, sinks,
-					 healed_sinks, AFR_DATA_TRANSACTION,
-					 locked_replies, data_lock);
+        if (!is_arbiter_the_only_sink) {
+                ret = afr_selfheal_inodelk (frame, this, fd->inode, this->name,
+                                            0, 0, data_lock);
+                if (ret < AFR_SH_MIN_PARTICIPANTS) {
+                        ret = -ENOTCONN;
+                        did_sh = _gf_false;
+                        goto skip_undo_pending;
+                }
+        }
+        ret = afr_selfheal_undo_pending (frame, this, fd->inode,
+                                         sources, sinks, healed_sinks,
+                                         undid_pending, AFR_DATA_TRANSACTION,
+                                         locked_replies, data_lock);
+skip_undo_pending:
+	afr_selfheal_uninodelk (frame, this, fd->inode, this->name, 0, 0,
+				data_lock);
 out:
-	if (compat)
-		afr_selfheal_uninodelk (frame, this, fd->inode, this->name,
-					LLONG_MAX - 2, 1, compat_lock);
 
         if (did_sh)
                 afr_log_selfheal (fd->inode->gfid, this, ret, "data", source,
-                                  healed_sinks);
+                                  sources, healed_sinks);
         else
                 ret = 1;
 
@@ -775,31 +812,32 @@ out:
 }
 
 
-static fd_t *
-afr_selfheal_data_open (xlator_t *this, inode_t *inode)
+int
+afr_selfheal_data_open (xlator_t *this, inode_t *inode, fd_t **fd)
 {
-	loc_t loc = {0,};
-	int ret = 0;
-	fd_t *fd = NULL;
+	int         ret    = 0;
+        fd_t       *fd_tmp = NULL;
+	loc_t       loc    = {0,};
 
-	fd = fd_create (inode, 0);
-	if (!fd)
-		return NULL;
+	fd_tmp = fd_create (inode, 0);
+	if (!fd_tmp)
+		return -ENOMEM;
 
 	loc.inode = inode_ref (inode);
 	gf_uuid_copy (loc.gfid, inode->gfid);
 
-	ret = syncop_open (this, &loc, O_RDWR|O_LARGEFILE, fd, NULL, NULL);
-	if (ret) {
-		fd_unref (fd);
-		fd = NULL;
+	ret = syncop_open (this, &loc, O_RDWR|O_LARGEFILE, fd_tmp, NULL, NULL);
+	if (ret < 0) {
+		fd_unref (fd_tmp);
+                goto out;
 	} else {
-		fd_bind (fd);
+		fd_bind (fd_tmp);
 	}
 
-	loc_wipe (&loc);
-
-	return fd;
+        *fd = fd_tmp;
+out:
+        loc_wipe (&loc);
+	return ret;
 }
 
 int
@@ -812,17 +850,18 @@ afr_selfheal_data (call_frame_t *frame, xlator_t *this, inode_t *inode)
 
 	priv = this->private;
 
-	fd = afr_selfheal_data_open (this, inode);
+	ret = afr_selfheal_data_open (this, inode, &fd);
 	if (!fd) {
-                gf_msg_debug (this->name, 0, "%s: Failed to open",
+                gf_msg_debug (this->name, -ret, "%s: Failed to open",
                               uuid_utoa (inode->gfid));
                 return -EIO;
         }
 
 	locked_on = alloca0 (priv->child_count);
 
-	ret = afr_selfheal_tryinodelk (frame, this, inode, priv->sh_domain, 0, 0,
-				       locked_on);
+	ret = afr_selfheal_tie_breaker_inodelk (frame, this, inode,
+	                                        priv->sh_domain, 0, 0,
+				                locked_on);
 	{
 		if (ret < AFR_SH_MIN_PARTICIPANTS) {
                         gf_msg_debug (this->name, 0, "%s: Skipping "
@@ -841,7 +880,8 @@ afr_selfheal_data (call_frame_t *frame, xlator_t *this, inode_t *inode)
 		ret = __afr_selfheal_data (frame, this, fd, locked_on);
 	}
 unlock:
-	afr_selfheal_uninodelk (frame, this, inode, priv->sh_domain, 0, 0, locked_on);
+	afr_selfheal_uninodelk (frame, this, inode, priv->sh_domain, 0, 0,
+	                        locked_on);
 
 	if (fd)
 		fd_unref (fd);
