@@ -1175,6 +1175,16 @@ afr_changelog_populate_xdata (call_frame_t *frame, afr_xattrop_type_t op,
                                 need_entry_key_set = _gf_false;
                                 break;
                         }
+                        /* If the transaction itself did not fail and there
+                         * are no failed subvolumes, check whether the fop
+                         * failed due to a symmetric error. If it did, do
+                         * not set the ENTRY_OUT xattr which would end up
+                         * deleting a name index which was created possibly by
+                         * an earlier entry txn that may have failed on some
+                         * of the sub-volumes.
+                         */
+                        if (local->op_ret)
+                                need_entry_key_set = _gf_false;
                 } else {
                         key = GF_XATTROP_ENTRY_IN_KEY;
                 }
@@ -2251,19 +2261,12 @@ afr_write_txn_refresh_done (call_frame_t *frame, xlator_t *this, int err)
         if (err) {
                 local->op_errno = -err;
                 local->op_ret = -1;
-                goto fail;
-        }
-	ret = afr_inode_get_readable (frame, local->inode, this,
-                                      local->readable, NULL,
-				      local->transaction.type);
-        if (ret < 0) {
                 gf_msg (this->name, GF_LOG_ERROR, -ret, AFR_MSG_SPLIT_BRAIN,
                         "Failing %s on gfid %s: split-brain observed.",
                         gf_fop_list[local->op], uuid_utoa (local->inode->gfid));
-                local->op_ret = -1;
-                local->op_errno = -ret;
                 goto fail;
         }
+
         afr_transaction_start (frame, this);
         return 0;
 fail:
