@@ -810,7 +810,8 @@ dht_fix_dir_xattr (call_frame_t *frame, loc_t *loc, dht_layout_t *layout)
 
         local->call_cnt = count = conf->subvolume_cnt;
 
-        dht_log_new_layout_for_dir_selfheal (this, loc, layout);
+        if (gf_log_get_loglevel () >= GF_LOG_DEBUG)
+                dht_log_new_layout_for_dir_selfheal (this, loc, layout);
 
         for (i = 0; i < layout->cnt; i++) {
                 dht_selfheal_dir_xattr_persubvol (frame, loc, layout, i, NULL);
@@ -884,7 +885,8 @@ dht_selfheal_dir_xattr (call_frame_t *frame, loc_t *loc, dht_layout_t *layout)
 
         local->call_cnt = missing_xattr;
 
-        dht_log_new_layout_for_dir_selfheal (this, loc, layout);
+        if (gf_log_get_loglevel () >= GF_LOG_DEBUG)
+                dht_log_new_layout_for_dir_selfheal (this, loc, layout);
 
         for (i = 0; i < layout->cnt; i++) {
                 if (layout->list[i].err != -1 || !layout->list[i].stop)
@@ -1011,7 +1013,8 @@ dht_selfheal_dir_xattr_for_nameless_lookup (call_frame_t *frame, loc_t *loc,
 
         local->call_cnt = missing_xattr;
 
-        dht_log_new_layout_for_dir_selfheal (this, loc, layout);
+        if (gf_log_get_loglevel () >= GF_LOG_DEBUG)
+                dht_log_new_layout_for_dir_selfheal (this, loc, layout);
 
         for (i = 0; i < layout->cnt; i++) {
                 if (layout->list[i].err != -1 || !layout->list[i].stop)
@@ -1703,6 +1706,7 @@ dht_fix_layout_of_directory (call_frame_t *frame, loc_t *loc,
         dht_local_t  *local        = NULL;
         uint32_t      subvol_down  = 0;
         int           ret          = 0;
+        gf_boolean_t  maximize_overlap = _gf_true;
 
         this  = frame->this;
         priv  = this->private;
@@ -1744,14 +1748,23 @@ dht_fix_layout_of_directory (call_frame_t *frame, loc_t *loc,
 
         if (priv->du_stats) {
                 for (i = 0; i < priv->subvolume_cnt; ++i) {
-                        gf_msg (this->name, GF_LOG_INFO, 0,
+                        gf_msg (this->name, GF_LOG_DEBUG, 0,
                                 DHT_MSG_SUBVOL_INFO,
                                 "subvolume %d (%s): %u chunks", i,
                                 priv->subvolumes[i]->name,
                                 priv->du_stats[i].chunks);
+
+                        /* Maximize overlap if the bricks are all the same
+                         *  size.
+                         * This is probably not going to be very common on
+                         * live setups but will benefit our regression tests
+                         */
+                        if (i && (priv->du_stats[i].chunks
+                                  != priv->du_stats[0].chunks)) {
+                                maximize_overlap = _gf_false;
+                        }
                 }
-        }
-        else {
+        } else {
                 gf_msg (this->name, GF_LOG_WARNING, 0,
                         DHT_MSG_NO_DISK_USAGE_STATUS, "no du stats ?!?");
         }
@@ -1761,9 +1774,16 @@ dht_fix_layout_of_directory (call_frame_t *frame, loc_t *loc,
         dht_layout_sort_volname (new_layout);
 	dht_selfheal_layout_new_directory (frame, loc, new_layout);
 
-	/* Now selectively re-assign ranges only when it helps */
-	dht_selfheal_layout_maximize_overlap (frame, loc, new_layout, layout);
 
+        /* Maximize overlap if weighted-rebalance is disabled */
+        if (!priv->do_weighting)
+                maximize_overlap = _gf_true;
+
+	/* Now selectively re-assign ranges only when it helps */
+        if (maximize_overlap) {
+                dht_selfheal_layout_maximize_overlap (frame, loc, new_layout,
+                                                      layout);
+        }
 done:
         if (new_layout) {
                 /* Now that the new layout has all the proper layout, change the
@@ -2237,11 +2257,19 @@ dht_dir_attr_heal (void *data)
 
         for (i = 0; i < call_cnt; i++) {
                 subvol = conf->subvolumes[i];
-                if (!subvol || (subvol == dht_first_up_subvol (this)))
+                if (!subvol)
                         continue;
-                ret = syncop_setattr (subvol, &local->loc, &local->stbuf,
-                                      (GF_SET_ATTR_UID | GF_SET_ATTR_GID),
-                                      NULL, NULL, NULL, NULL);
+
+                if (__is_root_gfid (local->stbuf.ia_gfid)) {
+                        ret = syncop_setattr (subvol, &local->loc, &local->stbuf,
+                                              (GF_SET_ATTR_UID | GF_SET_ATTR_GID | GF_SET_ATTR_MODE),
+                                              NULL, NULL, NULL, NULL);
+                } else {
+                        ret = syncop_setattr (subvol, &local->loc, &local->stbuf,
+                                              (GF_SET_ATTR_UID | GF_SET_ATTR_GID),
+                                              NULL, NULL, NULL, NULL);
+                }
+
                 if (ret) {
                         gf_uuid_unparse(local->loc.gfid, gfid);
 
