@@ -9,11 +9,6 @@
 */
 
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 /* TODO: add NS locking */
 #include "statedump.h"
 #include "dht-common.h"
@@ -159,7 +154,7 @@ dht_priv_dump (xlator_t *this)
         gf_proc_dump_write("unhashed_sticky_bit", "%d", conf->unhashed_sticky_bit);
         gf_proc_dump_write("use-readdirp", "%d", conf->use_readdirp);
 
-        if (conf->du_stats) {
+        if (conf->du_stats && conf->subvolume_status) {
                 for (i = 0; i < conf->subvolume_cnt; i++) {
                         if (!conf->subvolume_status[i])
                                 continue;
@@ -341,9 +336,9 @@ out:
 }
 void
 dht_init_regex (xlator_t *this, dict_t *odict, char *name,
-                regex_t *re, gf_boolean_t *re_valid)
+                regex_t *re, gf_boolean_t *re_valid, dht_conf_t *conf)
 {
-        char    *temp_str;
+        char       *temp_str = NULL;
 
         if (dict_get_str (odict, name, &temp_str) != 0) {
                 if (strcmp(name,"rsync-hash-regex")) {
@@ -352,25 +347,29 @@ dht_init_regex (xlator_t *this, dict_t *odict, char *name,
                 temp_str = "^\\.(.+)\\.[^.]+$";
         }
 
-        if (*re_valid) {
-                regfree(re);
-                *re_valid = _gf_false;
-        }
+        LOCK (&conf->lock);
+        {
+                if (*re_valid) {
+                        regfree(re);
+                        *re_valid = _gf_false;
+                }
 
-        if (!strcmp(temp_str,"none")) {
-                return;
-        }
+                if (!strcmp(temp_str, "none")) {
+                        goto unlock;
+                }
 
-        if (regcomp(re,temp_str,REG_EXTENDED) == 0) {
-                gf_msg_debug (this->name, 0,
-                              "using regex %s = %s", name, temp_str);
-                *re_valid = _gf_true;
-        }
-        else {
-                gf_msg (this->name, GF_LOG_WARNING, 0,
-                        DHT_MSG_REGEX_INFO,
-                        "compiling regex %s failed", temp_str);
+                if (regcomp(re, temp_str, REG_EXTENDED) == 0) {
+                        gf_msg_debug (this->name, 0,
+                                      "using regex %s = %s", name, temp_str);
+                        *re_valid = _gf_true;
+                } else {
+                        gf_msg (this->name, GF_LOG_WARNING, 0,
+                                DHT_MSG_REGEX_INFO,
+                                "compiling regex %s failed", temp_str);
+                }
         }
+unlock:
+        UNLOCK (&conf->lock);
 }
 
 int
@@ -460,7 +459,13 @@ dht_reconfigure (xlator_t *this, dict_t *options)
         GF_OPTION_RECONF ("rebal-throttle", conf->dthrottle, options,
                           str, out);
 
+        GF_OPTION_RECONF ("lock-migration", conf->lock_migration_enabled,
+                          options, bool, out);
+
         if (conf->defrag) {
+                conf->defrag->lock_migration_enabled =
+                                        conf->lock_migration_enabled;
+
                 GF_DECIDE_DEFRAG_THROTTLE_COUNT (throttle_count, conf);
                 gf_msg ("DHT", GF_LOG_INFO, 0,
                         DHT_MSG_REBAL_THROTTLE_INFO,
@@ -485,9 +490,9 @@ dht_reconfigure (xlator_t *this, dict_t *options)
         }
 
         dht_init_regex (this, options, "rsync-hash-regex",
-                        &conf->rsync_regex, &conf->rsync_regex_valid);
+                        &conf->rsync_regex, &conf->rsync_regex_valid, conf);
         dht_init_regex (this, options, "extra-hash-regex",
-                        &conf->extra_regex, &conf->extra_regex_valid);
+                        &conf->extra_regex, &conf->extra_regex_valid, conf);
 
         GF_OPTION_RECONF ("weighted-rebalance", conf->do_weighting, options,
                           bool, out);
@@ -569,6 +574,30 @@ out:
         return ret;
 }
 
+
+
+int
+dht_init_methods (xlator_t *this)
+{
+        int ret                  = -1;
+        dht_conf_t      *conf    = NULL;
+        dht_methods_t   *methods = NULL;
+
+        GF_VALIDATE_OR_GOTO ("dht", this, err);
+
+        conf = this->private;
+        methods = &(conf->methods);
+
+        methods->migration_get_dst_subvol = dht_migration_get_dst_subvol;
+        methods->migration_needed = dht_migration_needed;
+        methods->migration_other  = NULL;
+        methods->layout_search    = dht_layout_search;
+
+        ret = 0;
+err:
+        return ret;
+}
+
 int
 dht_init (xlator_t *this)
 {
@@ -602,6 +631,10 @@ dht_init (xlator_t *this)
                 goto err;
         }
 
+        LOCK_INIT (&conf->subvolume_lock);
+        LOCK_INIT (&conf->layout_lock);
+        LOCK_INIT (&conf->lock);
+
         /* We get the commit-hash to set only for rebalance process */
         if (dict_get_uint32 (this->options,
                              "commit-hash", &commit_hash) == 0) {
@@ -701,8 +734,14 @@ dht_init (xlator_t *this)
 
         GF_OPTION_INIT ("readdir-optimize", conf->readdir_optimize, bool, err);
 
+
+        GF_OPTION_INIT ("lock-migration", conf->lock_migration_enabled,
+                         bool, err);
+
         if (defrag) {
-                GF_OPTION_INIT ("rebalance-stats", defrag->stats, bool, err);
+              defrag->lock_migration_enabled = conf->lock_migration_enabled;
+
+              GF_OPTION_INIT ("rebalance-stats", defrag->stats, bool, err);
                 if (dict_get_str (this->options, "rebalance-filter", &temp_str)
                     == 0) {
                         if (gf_defrag_pattern_list_fill (this, defrag, temp_str)
@@ -745,17 +784,15 @@ dht_init (xlator_t *this)
         }
 
         dht_init_regex (this, this->options, "rsync-hash-regex",
-                        &conf->rsync_regex, &conf->rsync_regex_valid);
+                        &conf->rsync_regex, &conf->rsync_regex_valid, conf);
         dht_init_regex (this, this->options, "extra-hash-regex",
-                        &conf->extra_regex, &conf->extra_regex_valid);
+                        &conf->extra_regex, &conf->extra_regex_valid, conf);
 
         ret = dht_layouts_init (this, conf);
         if (ret == -1) {
                 goto err;
         }
 
-        LOCK_INIT (&conf->subvolume_lock);
-        LOCK_INIT (&conf->layout_lock);
 
         conf->gen = 1;
 
@@ -808,7 +845,8 @@ dht_init (xlator_t *this)
         if (dht_set_subvol_range(this))
                 goto err;
 
-        conf->methods = &dht_methods;
+        if (dht_init_methods (this))
+                goto err;
 
         return 0;
 
@@ -981,7 +1019,7 @@ struct volume_options options[] = {
 
         { .key  = {"tier-demote-frequency"},
           .type = GF_OPTION_TYPE_INT,
-          .default_value = "120",
+          .default_value = "3600",
           .description = "Frequency to demote files to slow tier"
         },
 
@@ -1008,11 +1046,15 @@ struct volume_options options[] = {
         },
         { .key         = {"tier-max-mb"},
           .type = GF_OPTION_TYPE_INT,
-          .default_value = "1000",
+          .default_value = "4000",
+        },
+        { .key         = {"tier-max-promote-file-size"},
+          .type = GF_OPTION_TYPE_INT,
+          .default_value = "0",
         },
         { .key         = {"tier-max-files"},
           .type = GF_OPTION_TYPE_INT,
-          .default_value = "5000",
+          .default_value = "10000",
         },
         /* switch option */
         { .key  = {"pattern.switch.case"},
@@ -1040,5 +1082,12 @@ struct volume_options options[] = {
                          "max of [($(processing units) - 4) / 2), 4]"
         },
 
+        { .key =  {"lock-migration"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "off",
+          .description = " If enabled this feature will migrate the posix locks"
+                         " associated with a file during rebalance"
+        },
+
         { .key  = {NULL} },
 };
