@@ -74,8 +74,8 @@ int32_t ec_heal_report(call_frame_t * frame, void * cookie, xlator_t * this,
             gf_msg (this->name, GF_LOG_INFO, 0,
                     EC_MSG_HEAL_SUCCESS, "Heal succeeded on %d/%d "
                     "subvolumes",
-                    ec_bits_count(mask & ~(good | bad)),
-                    ec_bits_count(mask & ~good));
+                    gf_bits_count(mask & ~(good | bad)),
+                    gf_bits_count(mask & ~good));
         }
     }
 
@@ -154,11 +154,12 @@ void ec_lock_update_good(ec_lock_t *lock, ec_fop_data_t *fop)
         return;
     }
 
-    /* When updating the good mask of the lock, we only take into
-     * consideration those bits corresponding to the bricks where
-     * the fop has been executed. */
-    lock->good_mask &= ~fop->mask | fop->remaining;
-    lock->good_mask |= fop->good;
+    /* When updating the good mask of the lock, we only take into consideration
+     * those bits corresponding to the bricks where the fop has been executed.
+     * Bad bricks are removed from good_mask, but once marked as bad it's never
+     * set to good until the lock is released and reacquired */
+
+    lock->good_mask &= fop->good | fop->remaining;
 }
 
 void __ec_fop_set_error(ec_fop_data_t * fop, int32_t error)
@@ -332,7 +333,7 @@ void ec_complete(ec_fop_data_t * fop)
         if (fop->answer == NULL) {
             if (!list_empty(&fop->cbk_list)) {
                 cbk = list_entry(fop->cbk_list.next, ec_cbk_data_t, list);
-                healing_count = ec_bits_count (cbk->mask & fop->healing);
+                healing_count = gf_bits_count (cbk->mask & fop->healing);
                     /* fop shouldn't be treated as success if it is not
                      * successful on at least fop->minimum good copies*/
                 if ((cbk->count - healing_count) >= fop->minimum) {
@@ -423,7 +424,7 @@ int32_t ec_child_select(ec_fop_data_t * fop)
     switch (fop->minimum)
     {
         case EC_MINIMUM_ALL:
-            fop->minimum = ec_bits_count(fop->mask);
+            fop->minimum = gf_bits_count(fop->mask);
             if (fop->minimum >= ec->fragments)
             {
                 break;
@@ -450,12 +451,12 @@ int32_t ec_child_select(ec_fop_data_t * fop)
 
     ec_trace("SELECT", fop, "");
 
-    num = ec_bits_count(fop->mask);
+    num = gf_bits_count(fop->mask);
     if ((num < fop->minimum) && (num < ec->fragments))
     {
         gf_msg (ec->xl->name, GF_LOG_ERROR, 0,
                 EC_MSG_CHILDS_INSUFFICIENT,
-                "Insufficient available childs "
+                "Insufficient available children "
                 "for this request (have %d, need "
                 "%d)", num, fop->minimum);
 
@@ -499,7 +500,7 @@ void ec_dispatch_mask(ec_fop_data_t * fop, uintptr_t mask)
     ec_t * ec = fop->xl->private;
     int32_t count, idx;
 
-    count = ec_bits_count(mask);
+    count = gf_bits_count(mask);
 
     LOCK(&fop->lock);
 
@@ -577,7 +578,7 @@ void ec_dispatch_inc(ec_fop_data_t * fop)
 
     if (ec_child_select(fop))
     {
-        fop->expected = ec_bits_count(fop->remaining);
+        fop->expected = gf_bits_count(fop->remaining);
         fop->first = 0;
 
         ec_dispatch_next(fop, 0);
@@ -590,7 +591,7 @@ ec_dispatch_all (ec_fop_data_t *fop)
         ec_dispatch_start(fop);
 
         if (ec_child_select(fop)) {
-                fop->expected = ec_bits_count(fop->remaining);
+                fop->expected = gf_bits_count(fop->remaining);
                 fop->first = 0;
 
                 ec_dispatch_mask(fop, fop->remaining);
@@ -644,6 +645,7 @@ ec_lock_t *ec_lock_allocate(ec_fop_data_t *fop, loc_t *loc)
     if (lock != NULL)
     {
         lock->good_mask = -1ULL;
+        INIT_LIST_HEAD(&lock->owners);
         INIT_LIST_HEAD(&lock->waiting);
         INIT_LIST_HEAD(&lock->frozen);
         err = ec_loc_from_loc(fop->xl, &lock->loc, loc);
@@ -709,8 +711,7 @@ void ec_lock_insert(ec_fop_data_t *fop, ec_lock_t *lock, uint32_t flags,
     link->update[EC_METADATA_TXN] = (flags & EC_UPDATE_META) != 0;
     link->base = base;
 
-    lock->refs++;
-    lock->inserted++;
+    lock->refs_pending++;
 }
 
 void ec_lock_prepare_inode_internal(ec_fop_data_t *fop, loc_t *loc,
@@ -890,7 +891,8 @@ ec_prepare_update_cbk (call_frame_t *frame, void *cookie,
                        xlator_t *this, int32_t op_ret, int32_t op_errno,
                        dict_t *dict, dict_t *xdata)
 {
-    ec_fop_data_t *fop = cookie, *parent;
+    struct list_head list;
+    ec_fop_data_t *fop = cookie, *parent, *tmp;
     ec_lock_link_t *link = fop->data;
     ec_lock_t *lock = NULL;
     ec_inode_t *ctx;
@@ -899,16 +901,26 @@ ec_prepare_update_cbk (call_frame_t *frame, void *cookie,
     parent = link->fop;
     ctx = lock->ctx;
 
+    INIT_LIST_HEAD(&list);
+
+    LOCK(&lock->loc.inode->lock);
+
+    list_for_each_entry(link, &lock->owners, owner_list) {
+        if ((link->fop->flags & EC_FLAG_WAITING_SIZE) != 0) {
+            link->fop->flags ^= EC_FLAG_WAITING_SIZE;
+
+            list_add_tail(&link->fop->cbk_list, &list);
+        }
+    }
+
     if (op_ret < 0) {
         gf_msg (this->name, GF_LOG_WARNING, op_errno,
                 EC_MSG_SIZE_VERS_GET_FAIL,
                 "Failed to get size and version");
 
-        goto out;
+        goto unlock;
     }
 
-    LOCK(&lock->loc.inode->lock);
-
     op_errno = -ec_dict_del_array(dict, EC_XATTR_VERSION, ctx->pre_version,
                                   EC_VERSION_SIZE);
     if (op_errno != 0) {
@@ -923,37 +935,46 @@ ec_prepare_update_cbk (call_frame_t *frame, void *cookie,
 
     ctx->have_version = _gf_true;
 
-    if (lock->loc.inode->ia_type == IA_IFREG) {
+    if (lock->loc.inode->ia_type == IA_IFREG ||
+        lock->loc.inode->ia_type == IA_INVAL) {
         op_errno = -ec_dict_del_number(dict, EC_XATTR_SIZE, &ctx->pre_size);
         if (op_errno != 0) {
-            gf_msg (this->name, GF_LOG_ERROR, op_errno,
-                    EC_MSG_SIZE_XATTR_GET_FAIL, "Unable to get size xattr");
+            if (lock->loc.inode->ia_type == IA_IFREG) {
+                gf_msg (this->name, GF_LOG_ERROR, op_errno,
+                        EC_MSG_SIZE_XATTR_GET_FAIL,
+                        "Unable to get size xattr");
 
-            goto unlock;
+                goto unlock;
+            }
+        } else {
+            ctx->post_size = ctx->pre_size;
+
+            ctx->have_size = _gf_true;
         }
-        ctx->post_size = ctx->pre_size;
-
-        ctx->have_size = _gf_true;
 
         op_errno = -ec_dict_del_config(dict, EC_XATTR_CONFIG, &ctx->config);
         if (op_errno != 0) {
-            gf_msg (this->name, GF_LOG_ERROR, op_errno,
-                    EC_MSG_CONFIG_XATTR_GET_FAIL,
-                    "Unable to get config xattr");
+            if ((lock->loc.inode->ia_type == IA_IFREG) ||
+                (op_errno != ENODATA)) {
+                gf_msg (this->name, GF_LOG_ERROR, op_errno,
+                        EC_MSG_CONFIG_XATTR_GET_FAIL,
+                        "Unable to get config xattr");
 
-            goto unlock;
+                goto unlock;
+            }
+        } else {
+            if (!ec_config_check(parent, &ctx->config)) {
+                gf_msg (this->name, GF_LOG_ERROR, EINVAL,
+                        EC_MSG_CONFIG_XATTR_INVALID,
+                        "Invalid config xattr");
+
+                op_errno = EINVAL;
+
+                goto unlock;
+            }
+
+            ctx->have_config = _gf_true;
         }
-        if (!ec_config_check(parent, &ctx->config)) {
-            gf_msg (this->name, GF_LOG_ERROR, EINVAL,
-                    EC_MSG_CONFIG_XATTR_INVALID,
-                    "Invalid config xattr");
-
-            op_errno = EINVAL;
-
-            goto unlock;
-        }
-
-        ctx->have_config = _gf_true;
     }
 
     ctx->have_info = _gf_true;
@@ -961,12 +982,15 @@ ec_prepare_update_cbk (call_frame_t *frame, void *cookie,
     op_errno = 0;
 
 unlock:
+    lock->getting_size = _gf_false;
+
     UNLOCK(&lock->loc.inode->lock);
-out:
+
     if (op_errno == 0) {
         /* We don't allow the main fop to be executed on bricks that have not
          * succeeded the initial xattrop. */
         parent->mask &= fop->good;
+        ec_lock_update_good (lock, fop);
 
         /*As of now only data healing marks bricks as healing*/
         lock->healing |= fop->healing;
@@ -977,6 +1001,24 @@ out:
         ec_fop_set_error(parent, op_errno);
     }
 
+    while (!list_empty(&list)) {
+        tmp = list_entry(list.next, ec_fop_data_t, cbk_list);
+        list_del_init(&tmp->cbk_list);
+
+        if (op_errno == 0) {
+            tmp->mask &= fop->good;
+
+            /*As of now only data healing marks bricks as healing*/
+            if (ec_is_data_fop (tmp->id)) {
+                tmp->healing |= fop->healing;
+            }
+        } else {
+            ec_fop_set_error(tmp, op_errno);
+        }
+
+        ec_resume(tmp, 0);
+    }
+
     return 0;
 }
 
@@ -987,9 +1029,8 @@ void ec_get_size_version(ec_lock_link_t *link)
     ec_inode_t *ctx;
     ec_fop_data_t *fop;
     dict_t *dict = NULL;
-    uid_t uid;
-    gid_t gid;
     int32_t error = -ENOMEM;
+    gf_boolean_t getting_size;
     uint64_t allzero[EC_VERSION_SIZE] = {0, 0};
 
     lock = link->lock;
@@ -1006,15 +1047,32 @@ void ec_get_size_version(ec_lock_link_t *link)
 
     /* Determine if there's something we need to retrieve for the current
      * operation. */
-    if (!lock->query && (lock->loc.inode->ia_type != IA_IFREG)) {
+    if (!lock->query &&
+        (lock->loc.inode->ia_type != IA_IFREG) &&
+        (lock->loc.inode->ia_type != IA_INVAL)) {
         return;
     }
 
-    uid = fop->frame->root->uid;
-    gid = fop->frame->root->gid;
-
     memset(&loc, 0, sizeof(loc));
 
+    LOCK(&lock->loc.inode->lock);
+
+    getting_size = lock->getting_size;
+    lock->getting_size = _gf_true;
+    if (getting_size) {
+        fop->flags |= EC_FLAG_WAITING_SIZE;
+
+        ec_sleep(fop);
+    }
+
+    UNLOCK(&lock->loc.inode->lock);
+
+    if (getting_size) {
+        error = 0;
+
+        goto out;
+    }
+
     dict = dict_new();
     if (dict == NULL) {
         goto out;
@@ -1032,7 +1090,8 @@ void ec_get_size_version(ec_lock_link_t *link)
         goto out;
     }
 
-    if (lock->loc.inode->ia_type == IA_IFREG) {
+    if (lock->loc.inode->ia_type == IA_IFREG ||
+        lock->loc.inode->ia_type == IA_INVAL) {
         error = ec_dict_set_number(dict, EC_XATTR_SIZE, 0);
         if (error == 0) {
             error = ec_dict_set_number(dict, EC_XATTR_CONFIG, 0);
@@ -1079,8 +1138,8 @@ void ec_get_size_version(ec_lock_link_t *link)
     error = 0;
 
 out:
-    fop->frame->root->uid = uid;
-    fop->frame->root->gid = gid;
+    fop->frame->root->uid = fop->uid;
+    fop->frame->root->gid = fop->gid;
 
     loc_wipe(&loc);
 
@@ -1240,34 +1299,119 @@ out:
     }
 }
 
+static void
+ec_lock_update_fd(ec_lock_t *lock, ec_fop_data_t *fop)
+{
+    /* If the fop has an fd available, attach it to the lock structure to be
+     * able to do fxattrop calls instead of xattrop. */
+    if (fop->use_fd && (lock->fd == NULL)) {
+        lock->fd = __fd_ref(fop->fd);
+    }
+}
+
+static void
+ec_lock_wake_shared(ec_lock_t *lock, struct list_head *list)
+{
+    ec_fop_data_t *fop;
+    ec_lock_link_t *link;
+    gf_boolean_t exclusive = _gf_false;
+
+    while (!exclusive && !list_empty(&lock->waiting)) {
+        link = list_entry(lock->waiting.next, ec_lock_link_t, wait_list);
+        fop = link->fop;
+
+        /* If lock is not acquired, at most one fop can be assigned as owner.
+         * The following fops will need to wait in the lock->waiting queue
+         * until the lock has been fully acquired. */
+        exclusive = !lock->acquired;
+
+        /* If the fop is not shareable, only this fop can be assigned as owner.
+         * Other fops will need to wait until this one finishes. */
+        if ((fop->flags & EC_FLAG_LOCK_SHARED) == 0) {
+            exclusive = _gf_true;
+
+            /* Avoid other requests to be assigned as owners. */
+            lock->exclusive = 1;
+        }
+
+        /* If only one fop is allowed, it can be assigned as the owner of the
+         * lock only if there weren't any other owner. */
+        if (exclusive && !list_empty(&lock->owners)) {
+            break;
+        }
+
+        list_move_tail(&link->wait_list, list);
+
+        list_add_tail(&link->owner_list, &lock->owners);
+        lock->refs_owners++;
+
+        ec_lock_update_fd(lock, fop);
+    }
+}
+
+static void
+ec_lock_apply(ec_lock_link_t *link)
+{
+    ec_fop_data_t *fop = link->fop;
+
+    fop->mask &= link->lock->good_mask;
+    fop->locked++;
+
+    ec_get_size_version(link);
+    ec_get_real_size(link);
+}
+
+gf_boolean_t ec_lock_acquire(ec_lock_link_t *link);
+
+static void
+ec_lock_resume_shared(struct list_head *list)
+{
+    ec_lock_link_t *link;
+
+    while (!list_empty(list)) {
+        link = list_entry(list->next, ec_lock_link_t, wait_list);
+        list_del_init(&link->wait_list);
+
+        if (link->lock->acquired) {
+            ec_lock_apply(link);
+            ec_lock(link->fop);
+        } else {
+            GF_ASSERT(list_empty(list));
+
+            ec_lock_acquire(link);
+        }
+
+        ec_resume(link->fop, 0);
+    }
+}
+
 void ec_lock_acquired(ec_lock_link_t *link)
 {
+    struct list_head list;
     ec_lock_t *lock;
     ec_fop_data_t *fop;
 
     lock = link->lock;
     fop = link->fop;
 
-    ec_trace("LOCKED", link->fop, "lock=%p", lock);
+    ec_trace("LOCKED", fop, "lock=%p", lock);
+
+    INIT_LIST_HEAD(&list);
+
+    LOCK(&lock->loc.inode->lock);
 
-    /* If the fop has an fd available, attach it to the lock structure to be
-     * able to do fxattrop calls instead of xattrop. It's safe to change this
-     * here because no xattrop using the fd can start concurrently at this
-     * point. */
-    if (fop->use_fd) {
-        if (lock->fd != NULL) {
-            fd_unref(lock->fd);
-        }
-        lock->fd = fd_ref(fop->fd);
-    }
     lock->acquired = _gf_true;
 
-    fop->mask &= lock->good_mask;
+    ec_lock_update_fd(lock, fop);
+    if ((fop->flags & EC_FLAG_LOCK_SHARED) != 0) {
+        ec_lock_wake_shared(lock, &list);
+    }
 
-    fop->locked++;
+    UNLOCK(&lock->loc.inode->lock);
 
-    ec_get_size_version(link);
-    ec_get_real_size(link);
+    ec_lock_apply(link);
+
+    ec_lock_resume_shared(&list);
 }
 
 int32_t ec_locked(call_frame_t *frame, void *cookie, xlator_t *this,
@@ -1301,6 +1445,7 @@ gf_boolean_t ec_lock_acquire(ec_lock_link_t *link)
 
     lock = link->lock;
     fop = link->fop;
+
     if (!lock->acquired) {
         ec_owner_set(fop->frame, lock);
 
@@ -1322,11 +1467,200 @@ gf_boolean_t ec_lock_acquire(ec_lock_link_t *link)
     return _gf_true;
 }
 
+static gf_boolean_t
+ec_lock_assign_owner(ec_lock_link_t *link)
+{
+    ec_fop_data_t *fop;
+    ec_lock_t *lock;
+    ec_lock_link_t *timer_link = NULL;
+    gf_boolean_t assigned = _gf_false;
+
+    /* The link cannot be in any list because we have just finished preparing
+     * it. */
+    GF_ASSERT(list_empty(&link->wait_list));
+
+    fop = link->fop;
+    lock = link->lock;
+
+    LOCK(&lock->loc.inode->lock);
+
+    /* Since the link has just been prepared but it's not active yet, the
+     * refs_pending must be one at least (the ref owned by this link). */
+    GF_ASSERT (lock->refs_pending > 0);
+    /* The link is not pending any more. It will be assigned to the owner,
+     * waiting or frozen list. */
+    lock->refs_pending--;
+
+    if (lock->release) {
+        ec_trace("LOCK_QUEUE_FREEZE", fop, "lock=%p", lock);
+
+        /* When lock->release is set, we'll unlock the lock as soon as
+         * possible, meaning that we won't use a timer. */
+        GF_ASSERT(lock->timer == NULL);
+
+        /* The lock is marked to be released. We can still have owners and fops
+         * in the waiting ilist f they have been added before the lock has been
+         * marked to be released. However new fops are put into the frozen list
+         * to wait for the next unlock/lock cycle. */
+        list_add_tail(&link->wait_list, &lock->frozen);
+
+        goto unlock;
+    }
+
+    /* The lock is not marked to be released, so the frozen list should be
+     * empty. */
+    GF_ASSERT(list_empty(&lock->frozen));
+
+    if (lock->timer != NULL) {
+        /* We are trying to acquire a lock that has an unlock timer active.
+         * This means that the lock must be idle, i.e. no fop can be in the
+         * owner, waiting or frozen lists. It also means that the lock cannot
+         * have been marked as being released (this is done without timers)
+         * and it must not be exclusive. There should only be one owner
+         * reference, but it's possible that some fops are being prepared to
+         * use this lock. */
+        GF_ASSERT ((lock->exclusive == 0) && (lock->refs_owners == 1) &&
+                   list_empty(&lock->owners) && list_empty(&lock->waiting));
+
+        /* We take the timer_link before cancelling the timer, since a
+         * successful cancellation will destroy it. It must not be NULL
+         * because it references the fop responsible for the delayed unlock
+         * that we are currently trying to cancel. */
+        timer_link = lock->timer->data;
+        GF_ASSERT(timer_link != NULL);
+
+        if (gf_timer_call_cancel(fop->xl->ctx, lock->timer) < 0) {
+            /* It's too late to avoid the execution of the timer callback.
+             * Since we need to be sure that the callback has access to all
+             * needed resources, we cannot resume the execution of the timer
+             * fop now. This will be done in the callback.
+             */
+            timer_link = NULL;
+        } else {
+            /* The timer has been cancelled, so we need to release the owner
+             * reference that was held by the fop waiting for the timer. This
+             * can be the last reference, but we'll immediately increment it
+             * for the current fop, so no need to check it.
+             */
+            lock->refs_owners--;
+
+            ec_trace("UNLOCK_CANCELLED", timer_link->fop, "lock=%p", lock);
+        }
+
+        /* We have two options here:
+         *
+         * 1. The timer has been successfully cancelled.
+         *
+         *    This is the easiest case and we can continue with the currently
+         *    acquired lock.
+         *
+         * 2. The timer callback has already been fired.
+         *
+         *    In this case we have not been able to cancel the timer before
+         *    the timer callback has been fired, but we also know that
+         *    lock->timer != NULL. This means that the timer callback is still
+         *    trying to acquire the inode mutex that we currently own. We are
+         *    safe until we release it. In this case we can safely clear
+         *    lock->timer. This will cause that the timer callback does nothing
+         *    once it acquires the mutex.
+         */
+        lock->timer = NULL;
+    }
+
+    lock->exclusive |= (fop->flags & EC_FLAG_LOCK_SHARED) == 0;
+
+    if (!list_empty(&lock->owners)) {
+        /* There are other owners of this lock. We can only take ownership if
+         * the lock is already acquired and can be shared. Otherwise we need
+         * to wait. */
+        if (!lock->acquired || (lock->exclusive != 0)) {
+            ec_trace("LOCK_QUEUE_WAIT", fop, "lock=%p", lock);
+
+            list_add_tail(&link->wait_list, &lock->waiting);
+
+            goto unlock;
+        }
+    }
+
+    list_add_tail(&link->owner_list, &lock->owners);
+    lock->refs_owners++;
+
+    assigned = _gf_true;
+
+unlock:
+    if (!assigned) {
+        /* We have not been able to take ownership of this lock. The fop must
+         * be put to sleep. */
+        ec_sleep(fop);
+    }
+
+    UNLOCK(&lock->loc.inode->lock);
+
+    /* If we have cancelled the timer, we need to resume the fop that was
+     * waiting for it. */
+    if (timer_link != NULL) {
+        ec_resume(timer_link->fop, 0);
+    }
+
+    return assigned;
+}
+
+static void
+ec_lock_next_owner(ec_lock_link_t *link, ec_cbk_data_t *cbk,
+                   gf_boolean_t release)
+{
+    struct list_head list;
+    ec_lock_t *lock = link->lock;
+    ec_fop_data_t *fop = link->fop;
+    ec_inode_t *ctx = lock->ctx;
+    ec_t *ec = fop->xl->private;
+
+    INIT_LIST_HEAD(&list);
+
+    LOCK(&lock->loc.inode->lock);
+
+    ec_trace("LOCK_DONE", fop, "lock=%p", lock);
+
+    /* Current link must belong to the owner list of the lock. We don't
+     * decrement lock->refs_owners here because the inode mutex is released
+     * before ec_unlock() is called and we need to know when the last owner
+     * unlocks the lock to do proper cleanup. lock->refs_owners is used for
+     * this task. */
+    GF_ASSERT((lock->refs_owners > 0) && !list_empty(&link->owner_list));
+    list_del_init(&link->owner_list);
+
+    lock->release |= release;
+
+    if ((fop->error == 0) && (cbk != NULL) && (cbk->op_ret >= 0)) {
+        if (link->update[0]) {
+            ctx->post_version[0]++;
+            if (ec->node_mask & ~fop->good) {
+                ctx->dirty[0]++;
+            }
+        }
+        if (link->update[1]) {
+            ctx->post_version[1]++;
+            if (ec->node_mask & ~fop->good) {
+                ctx->dirty[1]++;
+            }
+        }
+    }
+
+    ec_lock_update_good(lock, fop);
+
+    lock->exclusive -= (fop->flags & EC_FLAG_LOCK_SHARED) == 0;
+    if (list_empty(&lock->owners)) {
+        ec_lock_wake_shared(lock, &list);
+    }
+
+    UNLOCK(&lock->loc.inode->lock);
+
+    ec_lock_resume_shared(&list);
+}
+
 void ec_lock(ec_fop_data_t *fop)
 {
     ec_lock_link_t *link;
-    ec_lock_link_t *timer_link = NULL;
-    ec_lock_t *lock;
 
     /* There is a chance that ec_resume is called on fop even before ec_sleep.
      * Which can result in refs == 0 for fop leading to use after free in this
@@ -1338,125 +1672,67 @@ void ec_lock(ec_fop_data_t *fop)
         /* Since there are only up to 2 locks per fop, this xor will change
          * the order of the locks if fop->first_lock is 1. */
         link = &fop->locks[fop->locked ^ fop->first_lock];
-        lock = link->lock;
-
-        timer_link = NULL;
-
-        LOCK(&lock->loc.inode->lock);
-        GF_ASSERT (lock->inserted > 0);
-        lock->inserted--;
-
-        if (lock->timer != NULL) {
-            GF_ASSERT (lock->release == _gf_false);
-            timer_link = lock->timer->data;
-            if (gf_timer_call_cancel(fop->xl->ctx, lock->timer) == 0) {
-                    ec_trace("UNLOCK_CANCELLED", timer_link->fop,
-                             "lock=%p", lock);
-                    lock->timer = NULL;
-                    lock->refs--;
-                    /* There should remain at least 1 ref, the current one. */
-                    GF_ASSERT(lock->refs > 0);
-            } else {
-                    /* Timer expired and on the way to unlock.
-                     * Set lock->release to _gf_true, so that this
-                     * lock will be put in frozen list*/
-                    timer_link = NULL;
-                    lock->release = _gf_true;
-            }
-        }
-
-        GF_ASSERT(list_empty(&link->wait_list));
-
-        if ((lock->owner != NULL) || lock->release) {
-            if (lock->release) {
-                ec_trace("LOCK_QUEUE_FREEZE", fop, "lock=%p", lock);
-
-                list_add_tail(&link->wait_list, &lock->frozen);
-
-                /* The lock is frozen, so we move the current reference to
-                 * refs_frozen. After that, there should remain at least one
-                 * ref belonging to the lock that is processing the release. */
-                lock->refs--;
-                GF_ASSERT(lock->refs > 0);
-                lock->refs_frozen++;
-            } else {
-                ec_trace("LOCK_QUEUE_WAIT", fop, "lock=%p", lock);
-
-                list_add_tail(&link->wait_list, &lock->waiting);
-            }
-
-            UNLOCK(&lock->loc.inode->lock);
-
-            ec_sleep(fop);
 
+        if (!ec_lock_assign_owner(link) || !ec_lock_acquire(link)) {
             break;
         }
-
-        lock->owner = fop;
-
-        UNLOCK(&lock->loc.inode->lock);
-
-        if (!ec_lock_acquire(link)) {
-            break;
-        }
-
-        if (timer_link != NULL) {
-            ec_resume(timer_link->fop, 0);
-            timer_link = NULL;
-        }
     }
-    ec_resume (fop, 0);
 
-    if (timer_link != NULL) {
-        ec_resume(timer_link->fop, 0);
-    }
+    ec_resume(fop, 0);
 }
 
 void
 ec_lock_unfreeze(ec_lock_link_t *link)
 {
+    struct list_head list;
     ec_lock_t *lock;
+    gf_boolean_t destroy = _gf_false;
 
     lock = link->lock;
 
+    INIT_LIST_HEAD(&list);
+
     LOCK(&lock->loc.inode->lock);
 
-    lock->acquired = _gf_false;
+    /* The lock must be marked to be released here, since we have just released
+     * it and any attempt to assign it to more fops must have added them to the
+     * frozen list. We can only have one active reference here: the one that
+     * is processing this unfreeze. */
+    GF_ASSERT(lock->release && (lock->refs_owners == 1));
     lock->release = _gf_false;
+    lock->refs_owners = 0;
 
-    lock->refs--;
-    GF_ASSERT (lock->refs == lock->inserted);
+    lock->acquired = _gf_false;
 
-    GF_ASSERT(list_empty(&lock->waiting) && (lock->owner == NULL));
+    /* We are unfreezing a lock. This means that the lock has already been
+     * released. In this state it shouldn't be exclusive nor have a pending
+     * timer nor have any owner, and the waiting list should be empty. Only
+     * the frozen list can contain some fop. */
+    GF_ASSERT((lock->exclusive == 0) && (lock->timer == NULL) &&
+              list_empty(&lock->waiting) && list_empty(&lock->owners));
 
+    /* We move all frozen fops to the waiting list. */
     list_splice_init(&lock->frozen, &lock->waiting);
-    lock->refs += lock->refs_frozen;
-    lock->refs_frozen = 0;
 
-    if (!list_empty(&lock->waiting)) {
-        link = list_entry(lock->waiting.next, ec_lock_link_t, wait_list);
-        list_del_init(&link->wait_list);
-
-        lock->owner = link->fop;
-
-        UNLOCK(&lock->loc.inode->lock);
-
-        ec_trace("LOCK_UNFREEZE", link->fop, "lock=%p", lock);
-
-        if (ec_lock_acquire(link)) {
-            ec_lock(link->fop);
-        }
-        ec_resume(link->fop, 0);
-    } else if (lock->refs == 0) {
+    /* If we don't have any fop waiting nor there are any prepared fops using
+     * this lock, we can finally dispose it. */
+    destroy = list_empty(&lock->waiting) && (lock->refs_pending == 0);
+    if (destroy) {
         ec_trace("LOCK_DESTROY", link->fop, "lock=%p", lock);
 
         lock->ctx->inode_lock = NULL;
-
-        UNLOCK(&lock->loc.inode->lock);
-
-        ec_lock_destroy(lock);
     } else {
-        UNLOCK(&lock->loc.inode->lock);
+        ec_trace("LOCK_UNFREEZE", link->fop, "lock=%p", lock);
+
+        ec_lock_wake_shared(lock, &list);
+    }
+
+    UNLOCK(&lock->loc.inode->lock);
+
+    ec_lock_resume_shared(&list);
+
+    if (destroy) {
+        ec_lock_destroy(lock);
     }
 }
 
@@ -1564,8 +1840,6 @@ ec_update_size_version(ec_lock_link_t *link, uint64_t *version,
     ec_lock_t *lock;
     ec_inode_t *ctx;
     dict_t * dict;
-    uid_t uid;
-    gid_t gid;
     int32_t err = -ENOMEM;
 
     fop = link->fop;
@@ -1611,16 +1885,13 @@ ec_update_size_version(ec_lock_link_t *link, uint64_t *version,
         }
     }
 
-    /* If config information is not know, we request it now. */
+    /* If config information is not known, we request it now. */
     if ((lock->loc.inode->ia_type == IA_IFREG) && !ctx->have_config) {
         /* A failure requesting this xattr is ignored because it's not
          * absolutely required right now. */
         ec_dict_set_number(dict, EC_XATTR_CONFIG, 0);
     }
 
-    uid = fop->frame->root->uid;
-    gid = fop->frame->root->gid;
-
     fop->frame->root->uid = 0;
     fop->frame->root->gid = 0;
 
@@ -1634,8 +1905,8 @@ ec_update_size_version(ec_lock_link_t *link, uint64_t *version,
                        GF_XATTROP_ADD_ARRAY64, dict, NULL);
     }
 
-    fop->frame->root->uid = uid;
-    fop->frame->root->gid = gid;
+    fop->frame->root->uid = fop->uid;
+    fop->frame->root->gid = fop->gid;
 
     dict_unref(dict);
 
@@ -1650,6 +1921,13 @@ out:
 
     gf_msg (fop->xl->name, GF_LOG_ERROR, -err, EC_MSG_SIZE_VERS_UPDATE_FAIL,
             "Unable to update version and size");
+
+    if ((fop->parent->id != GF_FOP_FLUSH) &&
+        (fop->parent->id != GF_FOP_FSYNC) &&
+        (fop->parent->id != GF_FOP_FSYNCDIR)) {
+        ec_unlock_lock(fop->data);
+    }
+
 }
 
 gf_boolean_t
@@ -1697,49 +1975,85 @@ ec_unlock_now(ec_lock_link_t *link)
     ec_resume(link->fop, 0);
 }
 
+void ec_unlock_timer_add(ec_lock_link_t *link);
+
 void
 ec_unlock_timer_del(ec_lock_link_t *link)
 {
-        int32_t before = 0;
         ec_lock_t *lock;
         inode_t *inode;
         gf_boolean_t now = _gf_false;
 
+        /* If we are here, it means that the timer has expired before having
+         * been cancelled. This guarantees that 'link' is still valid because
+         * the fop that contains it must be pending (if timer cancellation in
+         * ec_lock_assign_owner() fails, the fop is left sleeping).
+         *
+         * At the same time, the fop still has a reference to the lock, so
+         * it must also be valid.
+         */
         lock = link->lock;
 
-        /* A race condition can happen if timer expires, calls this function
-         * and the lock is released (lock->loc is wiped) but the fop is not
-         * fully completed yet (it's still on the list of pending fops). In
-         * this case, this function can also be called if ec_unlock_force() is
-         * called. */
+        /* 'lock' must have a valid inode since it can only be destroyed
+         * when the lock itself is destroyed, but we have a reference to the
+         * lock to avoid this.
+         */
         inode = lock->loc.inode;
-        if (inode == NULL) {
-                return;
-        }
 
         LOCK(&inode->lock);
 
         if (lock->timer != NULL) {
                 ec_trace("UNLOCK_DELAYED", link->fop, "lock=%p", lock);
 
+                /* The unlock timer has expired without anyone cancelling it.
+                 * This means that it shouldn't have any owner, and the
+                 * waiting and frozen lists should be empty. It shouldn't have
+                 * been marked as release nor be exclusive either. It must have
+                 * only one owner reference, but there can be fops being
+                 * prepared though. */
+                GF_ASSERT(!lock->release && (lock->exclusive == 0) &&
+                          (lock->refs_owners == 1) &&
+                          list_empty(&lock->owners) &&
+                          list_empty(&lock->waiting) &&
+                          list_empty(&lock->frozen));
+
                 gf_timer_call_cancel(link->fop->xl->ctx, lock->timer);
                 lock->timer = NULL;
 
+                /* Any fop being processed from now on, will need to wait
+                 * until the next unlock/lock cycle. */
                 lock->release = now = _gf_true;
-
-                before = lock->refs + lock->refs_frozen;
-                list_splice_init(&lock->waiting, &lock->frozen);
-                lock->refs_frozen += lock->refs - lock->inserted - 1;
-                lock->refs = 1 + lock->inserted;
-                /* We moved around the locks, so total number of locks shouldn't
-                 * change by this operation*/
-                GF_ASSERT (before == (lock->refs + lock->refs_frozen));
         }
 
         UNLOCK(&inode->lock);
 
         if (now) {
                 ec_unlock_now(link);
+        } else {
+                /* The timer has been cancelled just after firing it but before
+                 * getting here. This means that another fop has used the lock
+                 * and everything should be handled as if this callback were
+                 * have not been executed. However we still have an owner
+                 * reference.
+                 *
+                 * We need to release our reference. If this is not the last
+                 * reference (the most common case because another fop has
+                 * taken another ref) we only need to decrement the counter.
+                 * Otherwise we have been delayed enough so that the other fop
+                 * has had time to acquire the reference, do its operation and
+                 * release it. At the time of releasing it, the fop did found
+                 * that the ref counter was > 1 (our reference), so the delayed
+                 * unlock timer wasn't started. We need to start it again if we
+                 * are the last reference.
+                 *
+                 * ec_unlock_timer_add() handles both cases.
+                 */
+                ec_unlock_timer_add(link);
+
+                /* We need to resume the fop that was waiting for the delayed
+                 * unlock.
+                 */
+                ec_resume(link->fop, 0);
         }
 }
 
@@ -1757,22 +2071,50 @@ void ec_unlock_timer_add(ec_lock_link_t *link)
 
     LOCK(&lock->loc.inode->lock);
 
-    GF_ASSERT(lock->timer == NULL);
+    /* We are trying to unlock the lock. We can have multiple scenarios here,
+     * but all of them need to have lock->timer == NULL:
+     *
+     * 1. There are other owners currently running that can call ec_unlock().
+     *
+     *    None of them can have started the timer until the last one. But this
+     *    call should be the consequence of this lastest one.
+     *
+     * 2. There are fops in the waiting or frozen lists.
+     *
+     *    These fops cannot call ec_unlock(). So we should be here.
+     *
+     * We must reach here with at least one owner reference.
+     */
+    GF_ASSERT((lock->timer == NULL) && (lock->refs_owners > 0));
 
-    if ((lock->refs - lock->inserted) > 1) {
+    /* If the fop detects that a heal is needed, we mark the lock to be
+     * released as soon as possible. */
+    lock->release |= ec_fop_needs_heal(fop);
+
+    if (lock->refs_owners > 1) {
         ec_trace("UNLOCK_SKIP", fop, "lock=%p", lock);
 
-        lock->refs--;
+        /* If there are other owners we cannot do anything else with the lock.
+         * Note that the current fop has already been removed from the owners
+         * list in ec_lock_reuse(). */
+        lock->refs_owners--;
 
         UNLOCK(&lock->loc.inode->lock);
     } else if (lock->acquired) {
+        /* There are no other owners and the lock is acquired. If there were
+         * fops waiting, at least one of them should have been promoted to an
+         * owner, so the waiting list should be empty. */
+        GF_ASSERT(list_empty(&lock->owners) && list_empty(&lock->waiting));
+
         ec_t *ec = fop->xl->private;
 
+        /* If everything goes as expected this fop will be put to sleep until
+         * the timer callback is executed. */
         ec_sleep(fop);
 
-        /* If healing is needed, the lock needs to be released due to
-         * contention, or ec is shutting down, do not delay lock release. */
-        if (!lock->release && !ec_fop_needs_heal(fop) && !ec->shutdown) {
+        /* If the lock needs to be released, or ec is shutting down, do not
+         * delay lock release. */
+        if (!lock->release && !ec->shutdown) {
             ec_trace("UNLOCK_DELAY", fop, "lock=%p, release=%d", lock,
                      lock->release);
 
@@ -1783,9 +2125,10 @@ void ec_unlock_timer_add(ec_lock_link_t *link)
             if (lock->timer == NULL) {
                 gf_msg(fop->xl->name, GF_LOG_WARNING, ENOMEM,
                        EC_MSG_UNLOCK_DELAY_FAILED,
-                       "Unable to delay an "
-                       "unlock");
+                       "Unable to delay an unlock");
 
+                /* We are unable to create a new timer. We immediately release
+                 * the lock. */
                 lock->release = now = _gf_true;
             }
         } else {
@@ -1800,6 +2143,15 @@ void ec_unlock_timer_add(ec_lock_link_t *link)
             ec_unlock_now(link);
         }
     } else {
+        /* There are no owners and the lock is not acquired. This can only
+         * happen if a lock attempt has failed and we get to the unlock step
+         * of the fop. As in the previous case, the waiting list must be
+         * empty. */
+        GF_ASSERT(list_empty(&lock->owners) && list_empty(&lock->waiting));
+
+        /* We need to mark the lock to be released to correctly handle fops
+         * that may get in after we release the inode mutex but before
+         * ec_lock_unfreeze() is processed. */
         lock->release = _gf_true;
 
         UNLOCK(&lock->loc.inode->lock);
@@ -1817,7 +2169,7 @@ void ec_unlock(ec_fop_data_t *fop)
     }
 }
 
-void ec_flush_size_version(ec_fop_data_t *fop)
+void ec_flush_size_version(ec_fop_data_t * fop)
 {
     GF_ASSERT(fop->lock_count == 1);
     ec_update_info(&fop->locks[0]);
@@ -1825,16 +2177,14 @@ void ec_flush_size_version(ec_fop_data_t *fop)
 
 void ec_lock_reuse(ec_fop_data_t *fop)
 {
-    ec_t *ec;
     ec_cbk_data_t *cbk;
-    ec_lock_t *lock;
-    ec_lock_link_t *link;
-    ec_inode_t *ctx;
+    ec_t *ec = NULL;
     int32_t i, count;
     gf_boolean_t release = _gf_false;
-
+    ec = fop->xl->private;
     cbk = fop->answer;
-    if (cbk != NULL) {
+
+    if (ec->eager_lock && cbk != NULL) {
         if (cbk->xdata != NULL) {
             if ((dict_get_int32(cbk->xdata, GLUSTERFS_INODELK_COUNT,
                                 &count) == 0) && (count > 1)) {
@@ -1846,62 +2196,14 @@ void ec_lock_reuse(ec_fop_data_t *fop)
             }
         }
     } else {
-        /* If we haven't get an answer with enough quorum, we always release
+        /* If eager lock is disabled or if we haven't get
+         * an answer with enough quorum, we always release
          * the lock. */
         release = _gf_true;
     }
 
-    ec = fop->xl->private;
-
-    for (i = 0; i < fop->lock_count; i++)
-    {
-        link = &fop->locks[i];
-        lock = link->lock;
-        ctx = lock->ctx;
-
-        LOCK(&lock->loc.inode->lock);
-
-        ec_trace("LOCK_DONE", fop, "lock=%p", lock);
-
-        GF_ASSERT(lock->owner == fop);
-        lock->owner = NULL;
-        lock->release |= release;
-
-        if ((fop->error == 0) && (cbk != NULL) && (cbk->op_ret >= 0)) {
-            if (link->update[0]) {
-                ctx->post_version[0]++;
-                if (ec->node_mask & ~fop->good) {
-                    ctx->dirty[0]++;
-                }
-            }
-            if (link->update[1]) {
-                ctx->post_version[1]++;
-                if (ec->node_mask & ~fop->good) {
-                    ctx->dirty[1]++;
-                }
-            }
-        }
-
-        ec_lock_update_good(lock, fop);
-
-        link = NULL;
-        if (!list_empty(&lock->waiting))
-        {
-            link = list_entry(lock->waiting.next, ec_lock_link_t, wait_list);
-            list_del_init(&link->wait_list);
-
-            lock->owner = link->fop;
-        }
-
-        UNLOCK(&lock->loc.inode->lock);
-
-        if (link != NULL)
-        {
-            if (ec_lock_acquire(link)) {
-                ec_lock(link->fop);
-            }
-            ec_resume(link->fop, 0);
-        }
+    for (i = 0; i < fop->lock_count; i++) {
+        ec_lock_next_owner(&fop->locks[i], cbk, release);
     }
 }
 
