@@ -133,25 +133,39 @@ union _ec_cbk
     fop_xattrop_cbk_t      xattrop;
     fop_fxattrop_cbk_t     fxattrop;
     fop_zerofill_cbk_t     zerofill;
+    fop_seek_cbk_t         seek;
 };
 
 struct _ec_lock
 {
     ec_inode_t        *ctx;
     gf_timer_t        *timer;
-    struct list_head   waiting; /* Queue of requests being serviced. */
-    struct list_head   frozen;  /* Queue of requests that will be serviced in
-                                   the next unlock/lock cycle. */
+
+    /* List of owners of this lock. All fops added to this list are running
+     * concurrently. */
+    struct list_head   owners;
+
+    /* List of fops waiting to be an owner of the lock. Fops are added to this
+     * list when the current owner has an incompatible access (shared vs
+     * exclusive) or the lock is not acquired yet. */
+    struct list_head   waiting;
+
+    /* List of fops that will wait until the next unlock/lock cycle. This
+     * happens when the currently acquired lock is decided to be released as
+     * soon as possible. In this case, all frozen fops will be continued only
+     * after the lock is reacquired. */
+    struct list_head   frozen;
+
+    int32_t            exclusive;
     uintptr_t          mask;
     uintptr_t          good_mask;
     uintptr_t          healing;
-    int32_t            refs;
-    int32_t            refs_frozen;
-    int32_t            inserted;
+    uint32_t           refs_owners;  /* Refs for fops owning the lock */
+    uint32_t           refs_pending; /* Refs assigned to fops being prepared */
     gf_boolean_t       acquired;
+    gf_boolean_t       getting_size;
     gf_boolean_t       release;
     gf_boolean_t       query;
-    ec_fop_data_t     *owner;
     fd_t              *fd;
     loc_t              loc;
     union
@@ -165,6 +179,7 @@ struct _ec_lock_link
 {
     ec_lock_t        *lock;
     ec_fop_data_t    *fop;
+    struct list_head  owner_list;
     struct list_head  wait_list;
     gf_boolean_t      update[2];
     loc_t            *base;
@@ -205,6 +220,9 @@ struct _ec_fop_data
     uintptr_t          received; /* Mask of responses */
     uintptr_t          good;
 
+    uid_t              uid;
+    gid_t              gid;
+
     ec_wind_f          wind;
     ec_handler_f       handler;
     ec_resume_f        resume;
@@ -237,6 +255,7 @@ struct _ec_fop_data
     struct gf_flock    flock;
     struct iovec      *vector;
     struct iobref     *buffers;
+    gf_seek_what_t     seek;
 };
 
 struct _ec_cbk_data
@@ -267,6 +286,8 @@ struct _ec_cbk_data
     struct iobref *  buffers;
     char            *str;
     gf_dirent_t      entries;
+    off_t            offset;
+    gf_seek_what_t   what;
 };
 
 struct _ec_heal
