@@ -195,4 +195,8 @@ void ec_fxattrop(call_frame_t * frame, xlator_t * this, uintptr_t target,
                  fd_t * fd, gf_xattrop_flags_t optype, dict_t * xattr,
                  dict_t * xdata);
 
+void ec_seek(call_frame_t *frame, xlator_t *this, uintptr_t target,
+             int32_t minimum, fop_seek_cbk_t func, void *data, fd_t *fd,
+             off_t offset, gf_seek_what_t what, dict_t *xdata);
+
 #endif /* __EC_FOPS_H__ */
