@@ -1148,7 +1148,8 @@ ec_create_name (call_frame_t *frame, ec_t *ec, inode_t *parent, char *name,
                                      replies, output, frame, ec->xl,
                                      &loc, st_mode_from_ia (ia->ia_prot,
                                                            ia->ia_type),
-                                     ia->ia_rdev, 0, xdata);
+                         makedev(ia_major(ia->ia_rdev), ia_minor(ia->ia_rdev)),
+                         0, xdata);
                 break;
         }
 
@@ -1369,7 +1370,7 @@ ec_heal_names (call_frame_t *frame, ec_t *ec, inode_t *inode,
                 if (!participants[i])
                         continue;
                 syncop_dir_scan (ec->xl_list[i], &loc,
-                                GF_CLIENT_PID_AFR_SELF_HEALD, &name_data,
+                                GF_CLIENT_PID_SELF_HEALD, &name_data,
                                 ec_name_heal_handler);
                 for (j = 0; j < ec->nodes; j++)
                         if (name_data.failed_on[j])
@@ -2325,6 +2326,8 @@ ec_heal_do (xlator_t *this, void *data, loc_t *loc, int32_t partial)
         /*Do heal as root*/
         frame->root->uid = 0;
         frame->root->gid = 0;
+        /*Mark the fops as internal*/
+        frame->root->pid = GF_CLIENT_PID_SELF_HEALD;
         participants = alloca0(ec->nodes);
         ec_mask_to_char_array (ec->xl_up, participants, ec->nodes);
         if (loc->name && strlen (loc->name)) {
@@ -2552,3 +2555,63 @@ fail:
     if (func)
             func (frame, NULL, this, -1, err, 0, 0, 0, NULL);
 }
+
+int
+ec_replace_heal_done (int ret, call_frame_t *heal, void *opaque)
+{
+        ec_t *ec = opaque;
+
+        gf_msg_debug (ec->xl->name, 0,
+                "getxattr on bricks is done ret %d", ret);
+        return 0;
+}
+
+int32_t
+ec_replace_heal (ec_t *ec, inode_t *inode)
+{
+        loc_t        loc = {0};
+        int          ret = 0;
+
+        loc.inode = inode_ref (inode);
+        gf_uuid_copy (loc.gfid, inode->gfid);
+        ret = syncop_getxattr (ec->xl, &loc, NULL, EC_XATTR_HEAL,
+                               NULL, NULL);
+        if (ret < 0)
+                gf_msg_debug (ec->xl->name, 0,
+                        "Heal failed for replace brick ret = %d", ret);
+
+        loc_wipe (&loc);
+        return ret;
+}
+
+int32_t
+ec_replace_brick_heal_wrap (void  *opaque)
+{
+         ec_t *ec = opaque;
+         inode_table_t   *itable = NULL;
+         int32_t         ret     = -1;
+
+         if (ec->xl->itable)
+                 itable = ec->xl->itable;
+         else
+                 goto out;
+         ret = ec_replace_heal (ec, itable->root);
+out:
+         return ret;
+}
+
+int32_t
+ec_launch_replace_heal (ec_t *ec)
+{
+	int ret = -1;
+
+        if (!ec)
+                return ret;
+        ret = synctask_new (ec->xl->ctx->env, ec_replace_brick_heal_wrap,
+                            ec_replace_heal_done, NULL, ec);
+        if (ret < 0) {
+                gf_msg_debug (ec->xl->name, 0,
+                        "Heal failed for replace brick ret = %d", ret);
+        }
+        return ret;
+}
