@@ -19,7 +19,6 @@
 #include "syncop-utils.h"
 #include "protocol-common.h"
 
-#define SHD_INODE_LRU_LIMIT          10
 #define ASSERT_LOCAL(this, healer)				        \
         do {                                                            \
                 if (!ec_shd_is_subvol_local (this, healer->subvol)) {	\
@@ -138,10 +137,8 @@ ec_shd_inode_find (xlator_t *this, xlator_t *subvol,
 	*inode =  NULL;
 
         *inode = inode_find (this->itable, gfid);
-        if (*inode) {
-                inode_lookup (*inode);
+        if (*inode)
                 goto out;
-        }
 
         loc.inode = inode_new (this->itable);
         if (!loc.inode) {
@@ -158,8 +155,6 @@ ec_shd_inode_find (xlator_t *this, xlator_t *subvol,
         if (!*inode) {
                 ret = -ENOMEM;
                 goto out;
-        } else {
-                inode_lookup (*inode);
         }
 out:
         loc_wipe (&loc);
@@ -268,8 +263,6 @@ out:
                         uuid_utoa(loc.gfid));
                 ec_shd_index_purge (subvol, parent->inode, entry->d_name);
         }
-        if (loc.inode)
-                inode_forget (loc.inode, 0);
         loc_wipe (&loc);
 
         return 0;
@@ -294,11 +287,9 @@ ec_shd_index_sweep (struct subvol_healer *healer)
                 goto out;
         }
 
-        ret = syncop_dir_scan (subvol, &loc, GF_CLIENT_PID_AFR_SELF_HEALD,
+        ret = syncop_dir_scan (subvol, &loc, GF_CLIENT_PID_SELF_HEALD,
                                healer, ec_shd_index_heal);
 out:
-        if (loc.inode)
-                inode_forget (loc.inode, 0);
         loc_wipe (&loc);
 
         return ret;
@@ -337,8 +328,6 @@ ec_shd_full_heal (xlator_t *subvol, gf_dirent_t *entry, loc_t *parent,
         ret = 0;
 
 out:
-        if (loc.inode)
-                inode_forget (loc.inode, 0);
         loc_wipe (&loc);
         return ret;
 }
@@ -352,7 +341,7 @@ ec_shd_full_sweep (struct subvol_healer *healer, inode_t *inode)
         ec = healer->this->private;
         loc.inode = inode;
         return syncop_ftw (ec->xl_list[healer->subvol], &loc,
-                           GF_CLIENT_PID_AFR_SELF_HEALD, healer,
+                           GF_CLIENT_PID_SELF_HEALD, healer,
                            ec_shd_full_heal);
 }
 
@@ -513,10 +502,6 @@ ec_selfheal_daemon_init (xlator_t *this)
         ec = this->private;
         shd = &ec->shd;
 
-        this->itable = inode_table_new (SHD_INODE_LRU_LIMIT, this);
-        if (!this->itable)
-                goto out;
-
         shd->index_healers = GF_CALLOC (sizeof(*shd->index_healers),
                                         ec->nodes,
                                         ec_mt_subvol_healer_t);
