@@ -126,9 +126,9 @@ changelog_init_rpc_threads (xlator_t *this, changelog_priv_t *priv,
  cleanup_connector:
         (void) pthread_cancel (priv->connector);
  cleanup_wait_lock:
-        (void) LOCK_DESTROY (&conn->wait_lock);
+        LOCK_DESTROY (&conn->wait_lock);
  cleanup_active_lock:
-        (void) LOCK_DESTROY (&conn->active_lock);
+        LOCK_DESTROY (&conn->active_lock);
  cleanup_pending_cond:
         (void) pthread_cond_destroy (&conn->pending_cond);
  cleanup_pending_lock:
@@ -183,11 +183,11 @@ changelog_rpc_clnt_cleanup (changelog_rpc_clnt_t *crpc)
         if (!crpc)
                 return;
         crpc->c_clnt = NULL;
-        (void) LOCK_DESTROY (&crpc->lock);
+        LOCK_DESTROY (&crpc->lock);
         GF_FREE (crpc);
 }
 
-static inline changelog_rpc_clnt_t *
+static changelog_rpc_clnt_t *
 changelog_rpc_clnt_init (xlator_t *this,
                          changelog_probe_req *rpc_req, changelog_clnt_t *c_clnt)
 {
@@ -199,7 +199,10 @@ changelog_rpc_clnt_init (xlator_t *this,
                 goto error_return;
         INIT_LIST_HEAD (&crpc->list);
 
-        crpc->ref = 0;
+        /* Take a ref, the last unref will be on RPC_CLNT_DESTROY
+         * which comes as a result of last rpc_clnt_unref.
+         */
+        crpc->ref = 1;
         changelog_set_disconnect_flag (crpc, _gf_false);
 
         crpc->filter = rpc_req->filter;
@@ -246,7 +249,9 @@ changelog_handle_probe (rpcsvc_request_t *req)
         ret = xdr_to_generic (req->msg[0],
                               &rpc_req, (xdrproc_t)xdr_changelog_probe_req);
         if (ret < 0) {
-                gf_log ("", GF_LOG_ERROR, "xdr decoding error");
+                gf_msg ("", GF_LOG_ERROR, 0,
+                        CHANGELOG_MSG_HANDLE_PROBE_ERROR,
+                        "xdr decoding error");
                 req->rpc_err = GARBAGE_ARGS;
                 goto handle_xdr_error;
         }
