@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "gfid-access.h"
 #include "inode.h"
 #include "byte-order.h"
@@ -171,7 +166,6 @@ ga_newfile_parse_args (xlator_t *this, data_t *data)
                         goto err;
                 }
                 args->args.mkdir.umask = ntoh32 (*(uint32_t *)blob);
-                blob += sizeof (uint32_t);
                 blob_len -= sizeof (uint32_t);
                 if (blob_len < 0) {
                         gf_log (this->name, GF_LOG_ERROR,
@@ -193,7 +187,6 @@ ga_newfile_parse_args (xlator_t *this, data_t *data)
                         goto err;
 
                 memcpy (args->args.symlink.linkpath, blob, (len + 1));
-                blob += (len + 1);
                 blob_len -= (len + 1);
         } else {
                 if (blob_len < sizeof (uint32_t)) {
@@ -223,7 +216,6 @@ ga_newfile_parse_args (xlator_t *this, data_t *data)
                         goto err;
                 }
                 args->args.mknod.umask = ntoh32 (*(uint32_t *)blob);
-                blob += sizeof (uint32_t);
                 blob_len -= sizeof (uint32_t);
         }
 
@@ -400,33 +392,6 @@ ga_heal_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         return 0;
 }
 
-static int32_t
-ga_newentry_setattr_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                         int32_t op_ret, int32_t op_errno, struct iatt *statpre,
-                         struct iatt *statpost,
-                         dict_t *xdata)
-{
-        ga_local_t *local = NULL;
-
-        local = frame->local;
-        frame->local = NULL;
-
-        /* don't worry about inode linking and other stuff. They'll happen on
-         * the next lookup.
-         */
-        STACK_DESTROY (frame->root);
-
-        STACK_UNWIND_STRICT (setxattr, local->orig_frame, op_ret,
-                             op_errno, xdata);
-
-        if (local->xdata)
-                dict_unref (local->xdata);
-        loc_wipe (&local->loc);
-        mem_put (local);
-
-        return 0;
-}
-
 static int
 ga_newentry_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  int32_t op_ret, int32_t op_errno,
@@ -435,24 +400,9 @@ ga_newentry_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                  dict_t *xdata)
 {
         ga_local_t *local = NULL;
-        struct iatt temp_stat = {0,};
 
         local = frame->local;
 
-        /* no need to proceed if things don't look good here */
-        if (op_ret == -1)
-                goto done;
-
-        temp_stat.ia_uid = local->uid;
-        temp_stat.ia_gid = local->gid;
-
-        STACK_WIND (frame, ga_newentry_setattr_cbk, FIRST_CHILD (this),
-                    FIRST_CHILD (this)->fops->setattr, &local->loc, &temp_stat,
-                    (GF_SET_ATTR_UID | GF_SET_ATTR_GID), xdata);
-
-        return 0;
-
-done:
         /* don't worry about inode linking and other stuff. They'll happen on
          * the next lookup.
          */
@@ -544,12 +494,11 @@ ga_new_entry (call_frame_t *frame, xlator_t *this, loc_t *loc, data_t *data,
         local = mem_get0 (this->local_pool);
         local->orig_frame = frame;
 
-        local->uid = args->uid;
-        local->gid = args->gid;
-
         loc_copy (&local->loc, &tmp_loc);
 
         new_frame->local = local;
+        new_frame->root->uid = args->uid;
+        new_frame->root->gid = args->gid;
 
         if (S_ISDIR (args->st_mode)) {
                 STACK_WIND (new_frame, ga_newentry_cbk,
@@ -842,6 +791,16 @@ ga_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
         inode_t      *true_inode    = NULL;
         int32_t       op_errno = ENOENT;
 
+        priv = this->private;
+
+        /* Handle nameless lookup on ".gfid" */
+        if (!loc->parent && __is_gfid_access_dir(loc->gfid)) {
+                STACK_UNWIND_STRICT (lookup, frame, 0, 0, loc->inode,
+                                     &priv->gfiddir_stbuf, xdata,
+                                     &priv->root_stbuf);
+                return 0;
+        }
+
         /* if its discover(), no need for any action here */
         if (!loc->name)
                 goto wind;
@@ -874,8 +833,6 @@ ga_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
                 goto wind;
         }
 
-        priv = this->private;
-
         /* need to check if the lookup is on virtual dir */
         if ((loc->name && !strcmp (GF_GFID_DIR, loc->name)) &&
             ((loc->parent && __is_root_gfid (loc->parent->gfid)) ||
