@@ -26,10 +26,20 @@ typedef enum {
         NOTIN
 } index_state_t;
 
+typedef enum {
+        XATTROP_TYPE_UNSET = -1,
+        XATTROP,
+        DIRTY,
+        ENTRY_CHANGES,
+        XATTROP_TYPE_END
+} index_xattrop_type_t;
+
 typedef struct index_inode_ctx {
         gf_boolean_t processing;
         struct list_head callstubs;
-        index_state_t state;
+        int state[XATTROP_TYPE_END];
+        uuid_t virtual_pargfid; /* virtual gfid of dir under
+                                  .glusterfs/indices/entry-changes. */
 } index_inode_ctx_t;
 
 typedef struct index_fd_ctx {
@@ -39,23 +49,38 @@ typedef struct index_fd_ctx {
 
 typedef struct index_priv {
         char *index_basepath;
+        char *dirty_basepath;
         uuid_t index;
         gf_lock_t lock;
-        uuid_t xattrop_vgfid;//virtual gfid of the xattrop index dir
+        uuid_t internal_vgfid[XATTROP_TYPE_END];
         struct list_head callstubs;
         pthread_mutex_t mutex;
         pthread_cond_t  cond;
-        dict_t  *xattrop64_watchlist;
+        dict_t  *dirty_watchlist;
+        dict_t  *pending_watchlist;
+        dict_t  *complete_watchlist;
+        int64_t  pending_count;
 } index_priv_t;
 
+typedef struct index_local {
+        inode_t *inode;
+        dict_t *xdata;
+} index_local_t;
+
 #define INDEX_STACK_UNWIND(fop, frame, params ...)      \
 do {                                                    \
+        index_local_t *__local = NULL;                  \
         if (frame) {                                    \
-                inode_t *_inode = frame->local;         \
+                __local = frame->local;                 \
                 frame->local = NULL;                    \
-                inode_unref (_inode);                   \
         }                                               \
         STACK_UNWIND_STRICT (fop, frame, params);       \
+        if (__local) {                                  \
+                inode_unref (__local->inode);           \
+                if (__local->xdata)                     \
+                        dict_unref (__local->xdata);    \
+                mem_put (__local);                      \
+        }                                               \
 } while (0)
 
 #endif
