@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "glusterfs.h"
 #include "compat.h"
 #include "xlator.h"
@@ -98,7 +93,7 @@ names_conflict (const char *n1, const char *n2)
 }
 
 
-static inline int
+static int
 __same_entrylk_owner (pl_entry_lock_t *l1, pl_entry_lock_t *l2)
 {
         return (is_same_lkowner (&l1->owner, &l2->owner) &&
@@ -106,7 +101,7 @@ __same_entrylk_owner (pl_entry_lock_t *l1, pl_entry_lock_t *l2)
 }
 
 /* Just as in inodelk, allow conflicting name locks from same (lk_owner, conn)*/
-static inline int
+static int
 __conflicting_entrylks (pl_entry_lock_t *l1, pl_entry_lock_t *l2)
 {
         if (names_conflict (l1->basename, l2->basename)
@@ -437,10 +432,10 @@ __unlock_entrylk (pl_dom_list_t *dom, pl_entry_lock_t *lock)
         return ret_lock;
 }
 
-uint32_t
+int32_t
 check_entrylk_on_basename (xlator_t *this, inode_t *parent, char *basename)
 {
-        uint32_t        entrylk = 0;
+        int32_t         entrylk = 0;
         pl_inode_t      *pinode = 0;
         pl_dom_list_t   *dom = NULL;
         pl_entry_lock_t *conf       = NULL;
