@@ -11,11 +11,6 @@
 #ifndef _MARKER_QUOTA_HELPER_H
 #define _MARKER_QUOTA_HELPER_H
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "marker.h"
 
 #define QUOTA_FREE_CONTRIBUTION_NODE(ctx, _contribution)             \
