@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef _QUOTA_H
 #define _QUOTA_H
 
@@ -199,6 +194,7 @@ struct quota_local {
         loc_t                   newloc;
         loc_t                   validate_loc;
         int64_t                 delta;
+        int8_t                  object_delta;
         int32_t                 op_ret;
         int32_t                 op_errno;
         int64_t                 size;
