@@ -8,17 +8,12 @@
    cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "snapview-client.h"
 #include "inode.h"
 #include "byte-order.h"
 
 
-void
+static void
 svc_local_free (svc_local_t *local)
 {
         if (local) {
@@ -31,7 +26,7 @@ svc_local_free (svc_local_t *local)
         }
 }
 
-xlator_t *
+static xlator_t *
 svc_get_subvolume (xlator_t *this, int inode_type)
 {
         xlator_t *subvolume = NULL;
@@ -47,7 +42,7 @@ out:
         return subvolume;
 }
 
-int32_t
+static int32_t
 __svc_inode_ctx_set (xlator_t *this, inode_t *inode, int inode_type)
 {
         uint64_t    value = 0;
@@ -64,7 +59,7 @@ out:
         return ret;
 }
 
-int
+static int
 __svc_inode_ctx_get (xlator_t *this, inode_t *inode, int *inode_type)
 {
         uint64_t    value      = 0;
@@ -83,7 +78,7 @@ out:
         return ret;
 }
 
-int
+static int
 svc_inode_ctx_get (xlator_t *this, inode_t *inode, int *inode_type)
 {
         int          ret        = -1;
@@ -101,7 +96,7 @@ out:
         return ret;
 }
 
-int32_t
+static int32_t
 svc_inode_ctx_set (xlator_t *this, inode_t *inode, int inode_type)
 {
         int32_t   ret = -1;
@@ -119,7 +114,7 @@ out:
         return ret;
 }
 
-svc_fd_t *
+static svc_fd_t *
 svc_fd_new (void)
 {
         svc_fd_t    *svc_fd = NULL;
@@ -129,7 +124,7 @@ svc_fd_new (void)
         return svc_fd;
 }
 
-svc_fd_t *
+static svc_fd_t *
 __svc_fd_ctx_get (xlator_t *this, fd_t *fd)
 {
         svc_fd_t *svc_fd = NULL;
@@ -149,7 +144,7 @@ out:
         return svc_fd;
 }
 
-svc_fd_t *
+static svc_fd_t *
 svc_fd_ctx_get (xlator_t *this, fd_t *fd)
 {
         svc_fd_t *svc_fd = NULL;
@@ -167,7 +162,7 @@ out:
         return svc_fd;
 }
 
-int
+static int
 __svc_fd_ctx_set (xlator_t *this, fd_t *fd, svc_fd_t *svc_fd)
 {
         uint64_t    value = 0;
@@ -185,26 +180,7 @@ out:
         return ret;
 }
 
-int32_t
-svc_fd_ctx_set (xlator_t *this, fd_t *fd, svc_fd_t *svc_fd)
-{
-        int32_t    ret = -1;
-
-        GF_VALIDATE_OR_GOTO ("snapview-client", this, out);
-        GF_VALIDATE_OR_GOTO (this->name, fd, out);
-        GF_VALIDATE_OR_GOTO (this->name, svc_fd, out);
-
-        LOCK (&fd->lock);
-        {
-                ret = __svc_fd_ctx_set (this, fd, svc_fd);
-        }
-        UNLOCK (&fd->lock);
-
-out:
-        return ret;
-}
-
-svc_fd_t *
+static svc_fd_t *
 __svc_fd_ctx_get_or_new (xlator_t *this, fd_t *fd)
 {
         svc_fd_t        *svc_fd    = NULL;
@@ -244,7 +220,7 @@ out:
         return svc_fd;
 }
 
-svc_fd_t *
+static svc_fd_t *
 svc_fd_ctx_get_or_new (xlator_t *this, fd_t *fd)
 {
         svc_fd_t  *svc_fd = NULL;
@@ -264,16 +240,14 @@ out:
 
 
 static int32_t
-svc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                int32_t op_ret, int32_t op_errno, inode_t *inode,
-                struct iatt *buf, dict_t *xdata, struct iatt *postparent)
+gf_svc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                   int32_t op_ret, int32_t op_errno, inode_t *inode,
+                   struct iatt *buf, dict_t *xdata, struct iatt *postparent)
 {
         svc_local_t     *local       = NULL;
-        inode_t         *parent      = NULL;
         xlator_t        *subvolume   = NULL;
         gf_boolean_t     do_unwind   = _gf_true;
         int              inode_type  = -1;
-        int              parent_type = -1;
         int              ret         = -1;
 
         local = frame->local;
@@ -313,20 +287,24 @@ svc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                                 ? GF_LOG_DEBUG:GF_LOG_ERROR,
                                 "Lookup failed on snapview graph with error %s",
                                 strerror (op_errno));
+                        goto out;
                 }
 
                 if ((op_errno == ENOENT || op_errno == ESTALE) &&
                     !gf_uuid_is_null (local->loc.gfid)) {
-                        ret = svc_inode_ctx_get (this, inode, &inode_type);
-                        if (ret < 0 && subvolume == FIRST_CHILD (this)) {
+                        if (inode != NULL)
+                                ret = svc_inode_ctx_get (this, inode,
+                                                                &inode_type);
+
+                        if (ret < 0 || inode == NULL) {
                                 gf_log (this->name, GF_LOG_DEBUG,
                                         "Lookup on normal graph failed. "
                                         "Sending lookup to snapview-server");
 
                                 subvolume = SECOND_CHILD (this);
                                 local->subvolume = subvolume;
-                                STACK_WIND (frame, svc_lookup_cbk, subvolume,
-                                            subvolume->fops->lookup,
+                                STACK_WIND (frame, gf_svc_lookup_cbk,
+                                            subvolume, subvolume->fops->lookup,
                                             &local->loc, xdata);
                                 do_unwind = _gf_false;
                         }
@@ -335,27 +313,6 @@ svc_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 goto out;
         }
 
-        if (local->loc.parent)
-                parent = inode_ref (local->loc.parent);
-        else {
-                parent = inode_parent (inode, NULL, NULL);
-                if (!parent && !gf_uuid_is_null (local->loc.pargfid)) {
-                        parent = inode_find (inode->table,
-                                             local->loc.pargfid);
-                }
-        }
-
-        if (!__is_root_gfid (buf->ia_gfid) && parent) {
-                ret = svc_inode_ctx_get (this, parent, &parent_type);
-                if (ret < 0) {
-                        op_ret = -1;
-                        op_errno = EINVAL;
-                        gf_log (this->name, GF_LOG_WARNING,
-                                "Error fetching parent context");
-                        goto out;
-                }
-        }
-
         if (subvolume == FIRST_CHILD (this))
                 inode_type = NORMAL_INODE;
         else
@@ -372,15 +329,11 @@ out:
                                   xdata, postparent);
         }
 
-        if (parent)
-                inode_unref (parent);
-
         return 0;
 }
 
 static int32_t
-svc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
-            dict_t *xdata)
+gf_svc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
 {
         int32_t        ret         =   -1;
         svc_local_t   *local       = NULL;
@@ -484,8 +437,8 @@ svc_lookup (call_frame_t *frame, xlator_t *this, loc_t *loc,
 
 out:
         if (wind)
-                STACK_WIND (frame, svc_lookup_cbk,
-                            subvolume, subvolume->fops->lookup, loc, xdata);
+                STACK_WIND (frame, gf_svc_lookup_cbk, subvolume,
+                            subvolume->fops->lookup, loc, xdata);
         else
                 SVC_STACK_UNWIND (lookup, frame, op_ret, op_errno, NULL,
                                   NULL, NULL, NULL);
@@ -499,8 +452,7 @@ out:
 }
 
 static int32_t
-svc_statfs (call_frame_t *frame, xlator_t *this, loc_t *loc,
-            dict_t *xdata)
+gf_svc_statfs (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *xdata)
 {
         xlator_t      *subvolume  = NULL;
         int32_t        ret        = -1;
@@ -557,13 +509,37 @@ out:
         return 0;
 }
 
+static int32_t
+gf_svc_stat_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                 int32_t op_ret, int32_t op_errno, struct iatt *buf,
+                 dict_t *xdata)
+{
+        /* Consider a testcase:
+         * #mount -t nfs host1:/vol1 /mnt
+         * #ls /mnt
+         * #ls /mnt/.snaps (As expected this fails)
+         * #gluster volume set vol1 features.uss enable
+         * Now `ls /mnt/.snaps` should work,
+         * but fails with No such file or directory.
+         * This is because NFS client caches the list of files in
+         * a directory. This cache is updated if there are any changes
+         * in the directory attributes. To solve this problem change
+         * a attribute 'ctime' when USS is enabled
+         */
+        if (op_ret == 0 && IA_ISDIR(buf->ia_type))
+                buf->ia_ctime_nsec++;
+
+        SVC_STACK_UNWIND (stat, frame, op_ret, op_errno, buf, xdata);
+        return 0;
+}
+
 /* should all the fops be handled like lookup is supposed to be
    handled? i.e just based on inode type decide where the call should
    be sent and in the call back update the contexts.
 */
 static int32_t
-svc_stat (call_frame_t *frame, xlator_t *this, loc_t *loc,
-          dict_t *xdata)
+gf_svc_stat (call_frame_t *frame, xlator_t *this, loc_t *loc,
+             dict_t *xdata)
 {
         int32_t      ret        = -1;
         int          inode_type = -1;
@@ -580,7 +556,8 @@ svc_stat (call_frame_t *frame, xlator_t *this, loc_t *loc,
         SVC_GET_SUBVOL_FROM_CTX (this, op_ret, op_errno, inode_type, ret,
                                  loc->inode, subvolume, out);
 
-        STACK_WIND_TAIL (frame,subvolume, subvolume->fops->stat, loc, xdata);
+        STACK_WIND (frame, gf_svc_stat_cbk, subvolume,
+                    subvolume->fops->stat, loc, xdata);
 
         wind = _gf_true;
 
@@ -592,7 +569,7 @@ out:
 }
 
 static int32_t
-svc_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
+gf_svc_fstat (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
 {
         int32_t      ret        = -1;
         int          inode_type = -1;
@@ -621,8 +598,8 @@ out:
 }
 
 static int32_t
-svc_opendir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                 int32_t op_ret, int32_t op_errno, fd_t *fd, dict_t *xdata)
+gf_svc_opendir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                    int32_t op_ret, int32_t op_errno, fd_t *fd, dict_t *xdata)
 {
         svc_fd_t        *svc_fd          = NULL;
         svc_local_t     *local           = NULL;
@@ -688,8 +665,8 @@ out:
    svc has to do things that open-behind is doing.
 */
 static int32_t
-svc_opendir (call_frame_t *frame, xlator_t *this, loc_t *loc, fd_t *fd,
-             dict_t *xdata)
+gf_svc_opendir (call_frame_t *frame, xlator_t *this, loc_t *loc, fd_t *fd,
+                dict_t *xdata)
 {
         int32_t        ret        = -1;
         int            inode_type = -1;
@@ -721,8 +698,8 @@ svc_opendir (call_frame_t *frame, xlator_t *this, loc_t *loc, fd_t *fd,
         local->subvolume = subvolume;
         frame->local = local;
 
-        STACK_WIND (frame, svc_opendir_cbk, subvolume, subvolume->fops->opendir,
-                    loc, fd, xdata);
+        STACK_WIND (frame, gf_svc_opendir_cbk, subvolume,
+                    subvolume->fops->opendir, loc, fd, xdata);
 
         wind = _gf_true;
 
@@ -734,8 +711,8 @@ out:
 }
 
 static int32_t
-svc_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
-             struct iatt *stbuf, int32_t valid, dict_t *xdata)
+gf_svc_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
+                struct iatt *stbuf, int32_t valid, dict_t *xdata)
 {
         int32_t      ret        = -1;
         int          inode_type = -1;
@@ -777,10 +754,11 @@ out:
         return 0;
 }
 
-/* XXX: This function is currently not used. Mark it 'static' when required */
-int32_t
-svc_fsetattr (call_frame_t *frame, xlator_t *this, fd_t *fd, struct iatt *stbuf,
-             int32_t valid, dict_t *xdata)
+/* XXX: This function is currently not used. Remove "#if 0" when required */
+#if 0
+static int32_t
+gf_svc_fsetattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
+                 struct iatt *stbuf, int32_t valid, dict_t *xdata)
 {
         int32_t      ret        = -1;
         int          inode_type = -1;
@@ -820,10 +798,11 @@ out:
                                   NULL, NULL, NULL);
         return 0;
 }
+#endif /* gf_svc_fsetattr() is not used */
 
 static int32_t
-svc_getxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, const char *name,
-              dict_t *xdata)
+gf_svc_getxattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
+                 const char *name, dict_t *xdata)
 {
         int32_t          ret                    = -1;
         int              inode_type             = -1;
@@ -901,10 +880,11 @@ out:
         return 0;
 }
 
-/* XXX: This function is currently not used. Mark it 'static' when required */
-int32_t
-svc_fgetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, const char *name,
-              dict_t *xdata)
+/* XXX: This function is currently not used. Mark it '#if 0' when required */
+#if 0
+static int32_t
+gf_svc_fgetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
+                  const char *name, dict_t *xdata)
 {
         int32_t       ret        = -1;
         int           inode_type = -1;
@@ -932,10 +912,11 @@ out:
                                   NULL, NULL);
         return 0;
 }
+#endif /* gf_svc_fgetxattr() is not used */
 
 static int32_t
-svc_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *dict,
-              int32_t flags, dict_t *xdata)
+gf_svc_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc, dict_t *dict,
+                 int32_t flags, dict_t *xdata)
 {
         int32_t      ret         = -1;
         int          inode_type  = -1;
@@ -979,8 +960,8 @@ out:
 }
 
 static int32_t
-svc_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *dict,
-              int32_t flags, dict_t *xdata)
+gf_svc_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *dict,
+                  int32_t flags, dict_t *xdata)
 {
         int32_t      ret        = -1;
         int          inode_type = -1;
@@ -1023,8 +1004,8 @@ out:
 }
 
 static int32_t
-svc_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
-           dict_t *xdata)
+gf_svc_rmdir (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
+              dict_t *xdata)
 {
         int          inode_type = -1;
         int          ret        = -1;
@@ -1067,10 +1048,10 @@ out:
 }
 
 static int32_t
-svc_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-               int32_t op_ret, int32_t op_errno, inode_t *inode,
-               struct iatt *buf, struct iatt *preparent,
-               struct iatt *postparent, dict_t *xdata)
+gf_svc_mkdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                  int32_t op_ret, int32_t op_errno, inode_t *inode,
+                  struct iatt *buf, struct iatt *preparent,
+                  struct iatt *postparent, dict_t *xdata)
 {
         int inode_type = -1;
         int ret        = -1;
@@ -1092,8 +1073,8 @@ out:
 }
 
 static int32_t
-svc_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
-           mode_t umask, dict_t *xdata)
+gf_svc_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
+              mode_t umask, dict_t *xdata)
 {
         int            parent_type = -1;
         int            ret         = -1;
@@ -1120,7 +1101,7 @@ svc_mkdir (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
         }
 
         if (strcmp (loc->name, priv->path) && parent_type == NORMAL_INODE) {
-                STACK_WIND (frame, svc_mkdir_cbk, FIRST_CHILD (this),
+                STACK_WIND (frame, gf_svc_mkdir_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->mkdir, loc, mode,
                             umask, xdata);
         } else {
@@ -1139,10 +1120,10 @@ out:
 }
 
 static int32_t
-svc_mknod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-               int32_t op_ret, int32_t op_errno, inode_t *inode,
-               struct iatt *buf, struct iatt *preparent,
-               struct iatt *postparent, dict_t *xdata)
+gf_svc_mknod_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                  int32_t op_ret, int32_t op_errno, inode_t *inode,
+                  struct iatt *buf, struct iatt *preparent,
+                  struct iatt *postparent, dict_t *xdata)
 {
         int inode_type = -1;
         int ret        = -1;
@@ -1163,8 +1144,8 @@ out:
 }
 
 static int32_t
-svc_mknod (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
-           dev_t rdev, mode_t umask, dict_t *xdata)
+gf_svc_mknod (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
+              dev_t rdev, mode_t umask, dict_t *xdata)
 {
         int            parent_type = -1;
         int            ret         = -1;
@@ -1191,7 +1172,7 @@ svc_mknod (call_frame_t *frame, xlator_t *this, loc_t *loc, mode_t mode,
         }
 
         if (strcmp (loc->name, priv->path) && parent_type == NORMAL_INODE) {
-                STACK_WIND (frame, svc_mknod_cbk, FIRST_CHILD (this),
+                STACK_WIND (frame, gf_svc_mknod_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->mknod, loc, mode,
                             rdev, umask, xdata);
         } else {
@@ -1214,8 +1195,8 @@ out:
    STACK_WIND the call to the first child of svc xlator.
 */
 static int32_t
-svc_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
-          fd_t *fd, dict_t *xdata)
+gf_svc_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
+             fd_t *fd, dict_t *xdata)
 {
         xlator_t    *subvolume  = NULL;
         int          inode_type = -1;
@@ -1259,10 +1240,10 @@ out:
 }
 
 static int32_t
-svc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                int32_t op_ret, int32_t op_errno, fd_t *fd, inode_t *inode,
-                struct iatt *stbuf, struct iatt *preparent,
-                struct iatt *postparent, dict_t *xdata)
+gf_svc_create_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                   int32_t op_ret, int32_t op_errno, fd_t *fd, inode_t *inode,
+                   struct iatt *stbuf, struct iatt *preparent,
+                   struct iatt *postparent, dict_t *xdata)
 {
         int  inode_type = -1;
         int  ret        = -1;
@@ -1284,9 +1265,8 @@ out:
 }
 
 static int32_t
-svc_create (call_frame_t *frame, xlator_t *this,
-            loc_t *loc, int32_t flags, mode_t mode,
-            mode_t umask, fd_t *fd, dict_t *xdata)
+gf_svc_create (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t flags,
+               mode_t mode, mode_t umask, fd_t *fd, dict_t *xdata)
 {
         int            parent_type = -1;
         int            ret         = -1;
@@ -1314,7 +1294,7 @@ svc_create (call_frame_t *frame, xlator_t *this,
         }
 
         if (strcmp (loc->name, priv->path) && parent_type == NORMAL_INODE) {
-                STACK_WIND (frame, svc_create_cbk, FIRST_CHILD (this),
+                STACK_WIND (frame, gf_svc_create_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->create, loc, flags,
                             mode, umask, fd, xdata);
         } else {
@@ -1333,10 +1313,10 @@ out:
 }
 
 static int32_t
-svc_symlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                     int32_t op_ret, int32_t op_errno, inode_t *inode,
-                     struct iatt *buf, struct iatt *preparent,
-                     struct iatt *postparent, dict_t *xdata)
+gf_svc_symlink_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                    int32_t op_ret, int32_t op_errno, inode_t *inode,
+                    struct iatt *buf, struct iatt *preparent,
+                    struct iatt *postparent, dict_t *xdata)
 {
         int inode_type = -1;
         int ret        = -1;
@@ -1358,8 +1338,8 @@ out:
 }
 
 static int32_t
-svc_symlink (call_frame_t *frame, xlator_t *this, const char *linkpath,
-             loc_t *loc, mode_t umask, dict_t *xdata)
+gf_svc_symlink (call_frame_t *frame, xlator_t *this, const char *linkpath,
+                loc_t *loc, mode_t umask, dict_t *xdata)
 {
         int            parent_type = -1;
         int            op_ret      = -1;
@@ -1386,7 +1366,7 @@ svc_symlink (call_frame_t *frame, xlator_t *this, const char *linkpath,
         }
 
         if (strcmp (loc->name, priv->path) && parent_type == NORMAL_INODE) {
-                STACK_WIND (frame, svc_symlink_cbk, FIRST_CHILD (this),
+                STACK_WIND (frame, gf_svc_symlink_cbk, FIRST_CHILD (this),
                             FIRST_CHILD (this)->fops->symlink, linkpath, loc,
                             umask, xdata);
         } else {
@@ -1405,8 +1385,8 @@ out:
 }
 
 static int32_t
-svc_unlink (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
-            dict_t *xdata)
+gf_svc_unlink (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
+               dict_t *xdata)
 {
         int            inode_type   = -1;
         int            op_ret       = -1;
@@ -1448,8 +1428,8 @@ out:
 }
 
 static int32_t
-svc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
-           off_t offset, uint32_t flags, dict_t *xdata)
+gf_svc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
+              off_t offset, uint32_t flags, dict_t *xdata)
 {
         int           inode_type = -1;
         xlator_t     *subvolume  = NULL;
@@ -1479,8 +1459,8 @@ out:
 }
 
 static int32_t
-svc_readlink (call_frame_t *frame, xlator_t *this,
-              loc_t *loc, size_t size, dict_t *xdata)
+gf_svc_readlink (call_frame_t *frame, xlator_t *this, loc_t *loc, size_t size,
+                 dict_t *xdata)
 {
         int              inode_type = -1;
         xlator_t        *subvolume  = NULL;
@@ -1510,8 +1490,8 @@ out:
 }
 
 static int32_t
-svc_access (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t mask,
-            dict_t *xdata)
+gf_svc_access (call_frame_t *frame, xlator_t *this, loc_t *loc, int32_t mask,
+               dict_t *xdata)
 {
         int            ret        = -1;
         int            inode_type = -1;
@@ -1540,13 +1520,49 @@ out:
         return 0;
 }
 
+int32_t
+gf_svc_readdir_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                    int32_t op_ret, int32_t op_errno, gf_dirent_t *entries,
+                    dict_t *xdata)
+{
+        gf_dirent_t   *entry      = NULL;
+        gf_dirent_t   *tmpentry  = NULL;
+        svc_local_t   *local      = NULL;
+        svc_private_t *priv       = NULL;
+
+        if (op_ret < 0)
+                goto out;
+
+        GF_VALIDATE_OR_GOTO (this->name, this->private, out);
+
+        priv = this->private;
+        local = frame->local;
+
+        /* If .snaps pre-exists, then it should not be listed
+         * in the NORMAL INODE directory when USS is enabled,
+         * so filter the .snaps entry if exists.
+         * However it is OK to list .snaps in VIRTUAL world
+         */
+        if (local->subvolume != FIRST_CHILD (this))
+                goto out;
+
+        list_for_each_entry_safe (entry, tmpentry, &entries->list, list) {
+                if (strcmp(priv->path, entry->d_name) == 0)
+                        gf_dirent_entry_free (entry);
+        }
+
+out:
+        SVC_STACK_UNWIND (readdir, frame, op_ret, op_errno, entries, xdata);
+        return 0;
+}
+
 static int32_t
-svc_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd,
-             size_t size, off_t off,
-             dict_t *xdata)
+gf_svc_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
+                off_t off, dict_t *xdata)
 {
         int           inode_type = -1;
         xlator_t     *subvolume  = NULL;
+        svc_local_t  *local      = NULL;
         int           ret        = -1;
         int           op_ret     = -1;
         int           op_errno   = EINVAL;
@@ -1577,8 +1593,16 @@ svc_readdir (call_frame_t *frame, xlator_t *this, fd_t *fd,
         SVC_GET_SUBVOL_FROM_CTX (this, op_ret, op_errno, inode_type, ret,
                                  fd->inode, subvolume, out);
 
-        STACK_WIND_TAIL (frame, subvolume, subvolume->fops->readdir, fd, size,
-                         off, xdata);
+        local = mem_get0 (this->local_pool);
+        if (!local) {
+                gf_log (this->name, GF_LOG_ERROR, "failed to allocate local");
+                goto out;
+        }
+        local->subvolume = subvolume;
+        frame->local = local;
+
+	STACK_WIND (frame, gf_svc_readdir_cbk, subvolume,
+                    subvolume->fops->readdir, fd, size, off, xdata);
 
         wind = _gf_true;
 
@@ -1617,10 +1641,10 @@ out:
  */
 
 static int32_t
-svc_readdirp_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                         int32_t op_ret, int32_t op_errno, inode_t *inode,
-                         struct iatt *buf, dict_t *xdata,
-                         struct iatt *postparent)
+gf_svc_readdirp_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                            int32_t op_ret, int32_t op_errno, inode_t *inode,
+                            struct iatt *buf, dict_t *xdata,
+                            struct iatt *postparent)
 {
         gf_dirent_t    entries;
         gf_dirent_t   *entry      = NULL;
@@ -1638,6 +1662,9 @@ svc_readdirp_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 
         local = frame->local;
 
+        if (local->xdata != NULL)
+                dict_unref (xdata);
+
         if (op_ret) {
                 op_ret = 0;
                 op_errno = ENOENT;
@@ -1688,10 +1715,11 @@ out:
         return 0;
 }
 
-gf_boolean_t
-svc_readdir_on_special_dir (call_frame_t *frame, void *cookie, xlator_t *this,
-                            int32_t op_ret, int32_t op_errno,
-                            gf_dirent_t *entries, dict_t *xdata)
+static gf_boolean_t
+gf_svc_readdir_on_special_dir (call_frame_t *frame, void *cookie,
+                               xlator_t *this, int32_t op_ret,
+                               int32_t op_errno, gf_dirent_t *entries,
+                               dict_t *xdata)
 {
         svc_local_t   *local      = NULL;
         svc_private_t *private    = NULL;
@@ -1774,8 +1802,11 @@ svc_readdir_on_special_dir (call_frame_t *frame, void *cookie, xlator_t *this,
                 }
 
                 local->cookie = cookie;
-                local->xdata = dict_ref (xdata);
-                STACK_WIND (frame, svc_readdirp_lookup_cbk,
+                if (xdata == NULL)
+                        local->xdata = NULL;
+                else
+                        local->xdata = dict_ref (xdata);
+                STACK_WIND (frame, gf_svc_readdirp_lookup_cbk,
                             SECOND_CHILD (this),
                             SECOND_CHILD (this)->fops->lookup, loc, tmp_xdata);
                 unwind = _gf_false;
@@ -1790,23 +1821,25 @@ out:
 }
 
 static int32_t
-svc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                  int32_t op_ret, int32_t op_errno,
-                  gf_dirent_t *entries, dict_t *xdata)
+gf_svc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                     int32_t op_ret, int32_t op_errno,
+                     gf_dirent_t *entries, dict_t *xdata)
 {
         gf_dirent_t   *entry      = NULL;
+        gf_dirent_t   *tmpentry   = NULL;
         svc_local_t   *local      = NULL;
-        gf_boolean_t   real       = _gf_true;
         int            inode_type = -1;
         int            ret        = -1;
         svc_fd_t      *svc_fd     = NULL;
         gf_boolean_t   unwind     = _gf_true;
-
-        GF_VALIDATE_OR_GOTO ("snapview-client", this, out);
+        svc_private_t *priv       = NULL;
 
         if (op_ret < 0)
                 goto out;
 
+        GF_VALIDATE_OR_GOTO ("snapview-client", this, out);
+        GF_VALIDATE_OR_GOTO (this->name, this->private, out);
+        priv = this->private;
         local = frame->local;
 
         svc_fd = svc_fd_ctx_get (this, local->fd);
@@ -1817,19 +1850,25 @@ svc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         }
 
         if (local->subvolume == FIRST_CHILD (this))
-                real = _gf_true;
+                inode_type = NORMAL_INODE;
         else
-                real = _gf_false;
+                inode_type = VIRTUAL_INODE;
+
+        list_for_each_entry_safe (entry, tmpentry, &entries->list, list) {
+                /* If .snaps pre-exists, then it should not be listed
+                 * in the NORMAL INODE directory when USS is enabled,
+                 * so filter the .snaps entry if exists.
+                 * However it is OK to list .snaps in VIRTUAL world
+                 */
+                if (inode_type == NORMAL_INODE &&
+                    !strcmp(priv->path, entry->d_name)) {
+                        gf_dirent_entry_free (entry);
+                        continue;
+                }
 
-        list_for_each_entry (entry, &entries->list, list) {
                 if (!entry->inode)
                         continue;
 
-                if (real)
-                        inode_type = NORMAL_INODE;
-                else
-                        inode_type = VIRTUAL_INODE;
-
                 ret = svc_inode_ctx_set (this, entry->inode, inode_type);
                 if (ret)
                         gf_log (this->name, GF_LOG_ERROR, "failed to set inode "
@@ -1838,8 +1877,8 @@ svc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         svc_fd->last_offset = entry->d_off;
         }
 
-        unwind = svc_readdir_on_special_dir (frame, cookie, this, op_ret,
-                                             op_errno, entries, xdata);
+        unwind = gf_svc_readdir_on_special_dir (frame, cookie, this, op_ret,
+                                                op_errno, entries, xdata);
 
 out:
         if (unwind)
@@ -1850,9 +1889,8 @@ out:
 }
 
 static int32_t
-svc_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd,
-              size_t size, off_t off,
-              dict_t *xdata)
+gf_svc_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
+                 off_t off, dict_t *xdata)
 {
         int            inode_type = -1;
         xlator_t      *subvolume  = NULL;
@@ -1907,7 +1945,7 @@ svc_readdirp (call_frame_t *frame, xlator_t *this, fd_t *fd,
         local->fd = fd_ref (fd);
         frame->local = local;
 
-        STACK_WIND (frame, svc_readdirp_cbk, subvolume,
+        STACK_WIND (frame, gf_svc_readdirp_cbk, subvolume,
                     subvolume->fops->readdirp, fd, size, off, xdata);
 
         wind = _gf_true;
@@ -1926,8 +1964,8 @@ out:
    are read-only.
 */
 static int32_t
-svc_rename (call_frame_t *frame, xlator_t *this, loc_t *oldloc,
-            loc_t *newloc, dict_t *xdata)
+gf_svc_rename (call_frame_t *frame, xlator_t *this, loc_t *oldloc,
+               loc_t *newloc, dict_t *xdata)
 {
         int          src_inode_type  = -1;
         int          dst_inode_type  = -1;
@@ -2004,8 +2042,8 @@ out:
    And so is vise versa.
 */
 static int32_t
-svc_link (call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
-          dict_t *xdata)
+gf_svc_link (call_frame_t *frame, xlator_t *this, loc_t *oldloc, loc_t *newloc,
+             dict_t *xdata)
 {
         int          src_inode_type  = -1;
         int          dst_parent_type = -1;
@@ -2052,8 +2090,8 @@ out:
 }
 
 static int32_t
-svc_removexattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
-                 const char *name, dict_t *xdata)
+gf_svc_removexattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
+                    const char *name, dict_t *xdata)
 {
         int          ret        = -1;
         int          inode_type = -1;
@@ -2097,8 +2135,8 @@ out:
 }
 
 static int
-svc_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd, int datasync,
-           dict_t *xdata)
+gf_svc_fsync (call_frame_t *frame, xlator_t *this, fd_t *fd, int datasync,
+              dict_t *xdata)
 {
         int             inode_type = -1;
         int             ret        = -1;
@@ -2141,8 +2179,7 @@ out:
 }
 
 static int32_t
-svc_flush (call_frame_t *frame, xlator_t *this,
-           fd_t *fd, dict_t *xdata)
+gf_svc_flush (call_frame_t *frame, xlator_t *this, fd_t *fd, dict_t *xdata)
 {
         int32_t           op_ret     = -1;
         int32_t           op_errno   = 0;
@@ -2171,7 +2208,7 @@ out:
 }
 
 static int32_t
-svc_releasedir (xlator_t *this, fd_t *fd)
+gf_svc_releasedir (xlator_t *this, fd_t *fd)
 {
         svc_fd_t *sfd      = NULL;
         uint64_t          tmp_pfd  = 0;
@@ -2194,7 +2231,7 @@ out:
 }
 
 static int32_t
-svc_forget (xlator_t *this, inode_t *inode)
+gf_svc_forget (xlator_t *this, inode_t *inode)
 {
         int            ret      = -1;
         uint64_t       value    = 0;
@@ -2360,37 +2397,37 @@ notify (xlator_t *this, int event, void *data, ...)
 }
 
 struct xlator_fops fops = {
-        .lookup        = svc_lookup,
-        .opendir       = svc_opendir,
-        .stat          = svc_stat,
-        .fstat         = svc_fstat,
-        .statfs        = svc_statfs,
-        .rmdir         = svc_rmdir,
-        .rename        = svc_rename,
-        .mkdir         = svc_mkdir,
-        .open          = svc_open,
-        .unlink        = svc_unlink,
-        .setattr       = svc_setattr,
-        .getxattr      = svc_getxattr,
-        .setxattr      = svc_setxattr,
-        .fsetxattr     = svc_fsetxattr,
-        .readv         = svc_readv,
-        .readdir       = svc_readdir,
-        .readdirp      = svc_readdirp,
-        .create        = svc_create,
-        .readlink      = svc_readlink,
-        .mknod         = svc_mknod,
-        .symlink       = svc_symlink,
-        .flush         = svc_flush,
-        .link          = svc_link,
-        .access        = svc_access,
-        .removexattr   = svc_removexattr,
-        .fsync         = svc_fsync,
+        .lookup        = gf_svc_lookup,
+        .opendir       = gf_svc_opendir,
+        .stat          = gf_svc_stat,
+        .fstat         = gf_svc_fstat,
+        .statfs        = gf_svc_statfs,
+        .rmdir         = gf_svc_rmdir,
+        .rename        = gf_svc_rename,
+        .mkdir         = gf_svc_mkdir,
+        .open          = gf_svc_open,
+        .unlink        = gf_svc_unlink,
+        .setattr       = gf_svc_setattr,
+        .getxattr      = gf_svc_getxattr,
+        .setxattr      = gf_svc_setxattr,
+        .fsetxattr     = gf_svc_fsetxattr,
+        .readv         = gf_svc_readv,
+        .readdir       = gf_svc_readdir,
+        .readdirp      = gf_svc_readdirp,
+        .create        = gf_svc_create,
+        .readlink      = gf_svc_readlink,
+        .mknod         = gf_svc_mknod,
+        .symlink       = gf_svc_symlink,
+        .flush         = gf_svc_flush,
+        .link          = gf_svc_link,
+        .access        = gf_svc_access,
+        .removexattr   = gf_svc_removexattr,
+        .fsync         = gf_svc_fsync,
 };
 
 struct xlator_cbks cbks = {
-        .forget = svc_forget,
-        .releasedir = svc_releasedir,
+        .forget = gf_svc_forget,
+        .releasedir = gf_svc_releasedir,
 };
 
 struct volume_options options[] = {
