@@ -13,11 +13,6 @@
 
 #include "rpc-clnt.h"
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "glusterd-conn-mgmt.h"
 
 glusterd_svc_t *
