@@ -8,14 +8,10 @@
    cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "xlator.h"
 #include "defaults.h"
 #include "glusterfs.h"
+#include "syscall.h"
 #include "compat-errno.h"
 
 #include "glusterd.h"
@@ -160,6 +156,27 @@ out:
         return ret;
 }
 
+int32_t
+glusterd_get_client_per_brick_volfile (glusterd_volinfo_t *volinfo,
+                                       char *filename, char *path, int path_len)
+{
+        char                    workdir[PATH_MAX]      = {0,};
+        glusterd_conf_t        *priv                   = NULL;
+        int32_t                 ret                    = -1;
+
+        GF_VALIDATE_OR_GOTO ("glusterd", THIS, out);
+        priv = THIS->private;
+        GF_VALIDATE_OR_GOTO (THIS->name, priv, out);
+
+        GLUSTERD_GET_VOLUME_DIR (workdir, volinfo, priv);
+
+        snprintf (path, path_len, "%s/%s", workdir, filename);
+
+        ret = 0;
+out:
+        return ret;
+}
+
 static size_t
 build_volfile_path (char *volume_id, char *path,
                     size_t path_len, char *trusted_str)
@@ -268,6 +285,49 @@ build_volfile_path (char *volume_id, char *path,
                 goto out;
         }
 
+        volid_ptr = strstr (volume_id, "client_per_brick/");
+        if (volid_ptr) {
+                volid_ptr = strchr (volid_ptr, '/');
+                if (!volid_ptr) {
+                        ret = -1;
+                        goto out;
+                }
+                volid_ptr++;
+
+                dup_volname = gf_strdup (volid_ptr);
+                if (!dup_volname) {
+                        gf_msg (this->name, GF_LOG_ERROR, ENOMEM,
+                                GD_MSG_NO_MEMORY,
+                                "strdup failed");
+                        ret = -1;
+                        goto out;
+                }
+
+                /* Split the volume name */
+                vol = strtok_r (dup_volname, ".", &save_ptr);
+                if (!vol) {
+                        ret = -1;
+                        goto out;
+                }
+                ret = glusterd_volinfo_find (vol, &volinfo);
+                if (ret == -1) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                GD_MSG_VOLINFO_GET_FAIL,
+                                "Couldn't find volinfo");
+                        goto out;
+                }
+                ret = glusterd_get_client_per_brick_volfile (volinfo, volid_ptr,
+                                                             path, path_len);
+                if (ret < 0) {
+                        gf_msg (this->name, GF_LOG_ERROR, 0,
+                                GD_MSG_NO_MEMORY, "failed to get volinfo path");
+                        goto out;
+                }
+
+                ret = sys_access (path, F_OK);
+                goto out;
+        }
+
         if (volume_id[0] == '/') {
                 /* Normal behavior */
                 volid_ptr = volume_id;
@@ -311,7 +371,7 @@ gotvolinfo:
         if (ret == -1)
                 goto out;
 
-        ret = stat (path, &stbuf);
+        ret = sys_stat (path, &stbuf);
 
         if ((ret == -1) && (errno == ENOENT)) {
                 strncpy (dup_volid, volid_ptr, (PATH_MAX - 1));
@@ -335,7 +395,7 @@ gotvolinfo:
                           path_prefix, volinfo->volname,
                           (trusted_str ? trusted_str : ""),
                           dup_volid);
-                ret = stat (path, &stbuf);
+                ret = sys_stat (path, &stbuf);
         }
 out:
         if (dup_volname)
@@ -790,8 +850,9 @@ __server_getspec (rpcsvc_request_t *req)
          * server, self-heal daemon etc., so that they are not inadvertently
          * blocked by a auth.{allow,reject} setting. The trusted volfile is not
          * meant for external users.
+         * For unix domain socket, address will be empty.
          */
-        if (strlen (addrstr) && gf_is_local_addr (addrstr)) {
+        if (strlen (addrstr) == 0 || gf_is_local_addr (addrstr)) {
 
                 ret = build_volfile_path (volume, filename,
                                           sizeof (filename),
@@ -803,7 +864,7 @@ __server_getspec (rpcsvc_request_t *req)
 
         if (ret == 0) {
                 /* to allocate the proper buffer to hold the file data */
-                ret = stat (filename, &stbuf);
+                ret = sys_stat (filename, &stbuf);
                 if (ret < 0){
                         gf_msg ("glusterd", GF_LOG_ERROR, errno,
                                 GD_MSG_FILE_OP_FAILED,
@@ -833,7 +894,7 @@ __server_getspec (rpcsvc_request_t *req)
                         op_errno = ENOMEM;
                         goto fail;
                 }
-                ret = read (spec_fd, rsp.spec, file_len);
+                ret = sys_read (spec_fd, rsp.spec, file_len);
         }
 
         if (brick_name) {
@@ -852,7 +913,7 @@ __server_getspec (rpcsvc_request_t *req)
         /* convert to XDR */
 fail:
         if (spec_fd > 0)
-                close (spec_fd);
+                sys_close (spec_fd);
 
         rsp.op_ret   = ret;
 
@@ -990,7 +1051,7 @@ out:
 /* Validate if glusterd can serve the management handshake request
  *
  * Requests are allowed if,
- *  - glusterd has no peers, or
+ *  - glusterd has no peers & no volumes, or
  *  - the request came from a known peer
  * A known peer is identified using the following steps
  *  - the dict is checked for a peer uuid, which if present is matched with the
@@ -1010,7 +1071,7 @@ gd_validate_mgmt_hndsk_req (rpcsvc_request_t *req, dict_t *dict)
         this = THIS;
         GF_ASSERT (this);
 
-        if (!glusterd_have_peers ())
+        if (!glusterd_have_peers () && !glusterd_have_volumes ())
                 return _gf_true;
 
         ret = dict_get_str (dict, GD_PEER_ID_KEY, &uuid_str);
@@ -1031,8 +1092,25 @@ gd_validate_mgmt_hndsk_req (rpcsvc_request_t *req, dict_t *dict)
         if (ret)
                 return _gf_false;
 
+        /* If peer object is not found it indicates that request is from an
+         * unknown peer, if its found, validate whether its uuid is also
+         * available in the peerinfo list. There could be a case where hostname
+         * is available in the peerinfo list but the uuid has changed of the
+         * node due to a reinstall, in that case the validation should fail!
+         */
         rcu_read_lock ();
-        ret = (glusterd_peerinfo_find (NULL, hostname) == NULL);
+        peer = glusterd_peerinfo_find (NULL, hostname);
+        if (!peer) {
+                ret = -1;
+        } else if (peer && glusterd_peerinfo_find (peer_uuid, NULL) != NULL) {
+                ret = 0;
+        } else {
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_HANDSHAKE_REQ_REJECTED, "Request from peer %s "
+                        "has an entry in peerinfo, but uuid does not match",
+                        req->trans->peerinfo.identifier);
+                ret = -1;
+        }
         rcu_read_unlock ();
 
         if (ret) {
@@ -2204,5 +2282,8 @@ glusterd_peer_dump_version (xlator_t *this, struct rpc_clnt *rpc,
 unlock:
         rcu_read_unlock ();
 out:
+        if (ret && frame)
+                STACK_DESTROY (frame->root);
+
         return ret;
 }
