@@ -24,6 +24,7 @@
 #include "glusterd-snapshot-utils.h"
 #include "glusterd-messages.h"
 #include "glusterd-errno.h"
+#include "glusterd-hooks.h"
 
 extern struct rpc_clnt_program gd_mgmt_v3_prog;
 
@@ -184,7 +185,17 @@ gd_mgmt_v3_pre_validate_fn (glusterd_op_t op, dict_t *dict,
                 if (ret) {
                         gf_msg (this->name, GF_LOG_WARNING, 0,
                                 GD_MSG_PRE_VALIDATION_FAIL,
-                                "Replace-brick prevalidation failed.");
+                                "ADD-brick prevalidation failed.");
+                        goto out;
+                }
+                break;
+        case GD_OP_START_VOLUME:
+                ret = glusterd_op_stage_start_volume (dict, op_errstr,
+                                                      rsp_dict);
+                if (ret) {
+                        gf_msg (this->name, GF_LOG_WARNING, 0,
+                                GD_MSG_PRE_VALIDATION_FAIL,
+                                "Volume start prevalidation failed.");
                         goto out;
                 }
                 break;
@@ -249,6 +260,7 @@ gd_mgmt_v3_commit_fn (glusterd_op_t op, dict_t *dict,
         GF_VALIDATE_OR_GOTO (this->name, op_errno, out);
         GF_ASSERT (rsp_dict);
 
+        glusterd_op_commit_hook (op, dict, GD_COMMIT_HOOK_PRE);
         switch (op) {
                case GD_OP_SNAP:
                {
@@ -285,6 +297,19 @@ gd_mgmt_v3_commit_fn (glusterd_op_t op, dict_t *dict,
                         break;
 
                 }
+                case GD_OP_START_VOLUME:
+                {
+                        ret = glusterd_op_start_volume (dict, op_errstr);
+                        if (ret) {
+                                gf_msg (this->name, GF_LOG_ERROR, 0,
+                                        GD_MSG_COMMIT_OP_FAIL,
+                                        "Volume start commit failed.");
+                                goto out;
+                        }
+                        break;
+
+                }
+
                default:
                        break;
         }
@@ -311,6 +336,9 @@ gd_mgmt_v3_post_validate_fn (glusterd_op_t op, int32_t op_ret, dict_t *dict,
         GF_ASSERT (op_errstr);
         GF_ASSERT (rsp_dict);
 
+        if (op_ret == 0)
+            glusterd_op_commit_hook (op, dict, GD_COMMIT_HOOK_POST);
+
         switch (op) {
                case GD_OP_SNAP:
                {
@@ -329,7 +357,7 @@ gd_mgmt_v3_post_validate_fn (glusterd_op_t op, int32_t op_ret, dict_t *dict,
                {
                         ret = dict_get_str (dict, "volname", &volname);
                         if (ret) {
-                                gf_msg ("glusterd", GF_LOG_ERROR, errno,
+                                gf_msg ("glusterd", GF_LOG_ERROR, 0,
                                         GD_MSG_DICT_GET_FAILED, "Unable to get"
                                         " volume name");
                                 goto out;
@@ -353,6 +381,36 @@ gd_mgmt_v3_post_validate_fn (glusterd_op_t op, int32_t op_ret, dict_t *dict,
                         break;
 
                }
+               case GD_OP_START_VOLUME:
+               {
+                        ret = dict_get_str (dict, "volname", &volname);
+                        if (ret) {
+                                gf_msg ("glusterd", GF_LOG_ERROR, 0,
+                                        GD_MSG_DICT_GET_FAILED, "Unable to get"
+                                        " volume name");
+                                goto out;
+                        }
+
+                        ret = glusterd_volinfo_find (volname, &volinfo);
+                        if (ret) {
+                                gf_msg ("glusterd", GF_LOG_ERROR, EINVAL,
+                                        GD_MSG_VOL_NOT_FOUND, "Unable to "
+                                        "allocate memory");
+                                goto out;
+                        }
+
+                        if (volinfo->type == GF_CLUSTER_TYPE_TIER) {
+                                if (volinfo->rebal.op != GD_OP_REMOVE_BRICK) {
+                                        glusterd_defrag_info_set (volinfo, dict,
+                                                  GF_DEFRAG_CMD_START_TIER,
+                                                  GF_DEFRAG_CMD_START,
+                                                  GD_OP_REBALANCE);
+                                }
+                                glusterd_restart_rebalance_for_volume (volinfo);
+                        }
+                        break;
+               }
+
                default:
                         break;
         }
@@ -422,7 +480,11 @@ out:
 
         if (rsp.dict.dict_val)
                 free (rsp.dict.dict_val);
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -442,7 +504,6 @@ gd_mgmt_v3_lock (glusterd_op_t op, dict_t *op_ctx,
                  uuid_t recv_uuid)
 {
         gd1_mgmt_v3_lock_req     req  = {{0},};
-        glusterd_conf_t         *conf = THIS->private;
         int32_t                  ret  = -1;
         xlator_t                *this = NULL;
         uuid_t                  *peerid = NULL;
@@ -611,6 +672,7 @@ glusterd_pre_validate_aggr_rsp_dict (glusterd_op_t op,
                         goto out;
                 }
                 break;
+        case GD_OP_START_VOLUME:
         case GD_OP_ADD_BRICK:
                 ret = glusterd_aggr_brick_mount_dirs (aggr, rsp);
                 if (ret) {
@@ -721,8 +783,11 @@ out:
         if (rsp.op_errstr)
                 free (rsp.op_errstr);
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -743,7 +808,6 @@ gd_mgmt_v3_pre_validate_req (glusterd_op_t op, dict_t *op_ctx,
 {
         int32_t                  ret   = -1;
         gd1_mgmt_v3_pre_val_req  req   = {{0},};
-        glusterd_conf_t         *conf  = THIS->private;
         xlator_t                *this  = NULL;
         uuid_t                  *peerid = NULL;
 
@@ -918,6 +982,7 @@ glusterd_mgmt_v3_build_payload (dict_t **req, char **op_errstr, dict_t *dict,
                 case GD_OP_SNAP:
                         dict_copy (dict, req_dict);
                         break;
+                case GD_OP_START_VOLUME:
                 case GD_OP_ADD_BRICK:
                 case GD_OP_REPLACE_BRICK:
                 {
@@ -1008,8 +1073,11 @@ out:
         if (rsp.dict.dict_val)
                 free (rsp.dict.dict_val);
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -1030,7 +1098,6 @@ gd_mgmt_v3_brick_op_req (glusterd_op_t op, dict_t *op_ctx,
 {
         int32_t                   ret  = -1;
         gd1_mgmt_v3_brick_op_req  req  = {{0},};
-        glusterd_conf_t          *conf = THIS->private;
         xlator_t                 *this = NULL;
         uuid_t                   *peerid = {0,};
 
@@ -1255,7 +1322,14 @@ out:
                                   GLUSTERD_MGMT_V3_COMMIT, *peerid, rsp.uuid);
         GF_FREE (peerid);
 
-        STACK_DESTROY (frame->root);
+        if (rsp.op_errstr)
+                free (rsp.op_errstr);
+
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -1276,7 +1350,6 @@ gd_mgmt_v3_commit_req (glusterd_op_t op, dict_t *op_ctx,
 {
         int32_t                  ret  = -1;
         gd1_mgmt_v3_commit_req   req  = {{0},};
-        glusterd_conf_t         *conf = THIS->private;
         xlator_t                *this = NULL;
         uuid_t                  *peerid = NULL;
 
@@ -1482,8 +1555,11 @@ out:
         if (rsp.dict.dict_val)
                 free (rsp.dict.dict_val);
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -1504,7 +1580,6 @@ gd_mgmt_v3_post_validate_req (glusterd_op_t op, int32_t op_ret, dict_t *op_ctx,
 {
         int32_t                   ret  = -1;
         gd1_mgmt_v3_post_val_req  req  = {{0},};
-        glusterd_conf_t          *conf = THIS->private;
         xlator_t                 *this = NULL;
         uuid_t                   *peerid = NULL;
 
@@ -1698,8 +1773,11 @@ out:
         if (rsp.dict.dict_val)
                 free (rsp.dict.dict_val);
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -1720,7 +1798,6 @@ gd_mgmt_v3_unlock (glusterd_op_t op, dict_t *op_ctx,
 {
         int32_t                  ret  = -1;
         gd1_mgmt_v3_unlock_req   req  = {{0},};
-        glusterd_conf_t         *conf = THIS->private;
         xlator_t                *this = NULL;
         uuid_t                  *peerid = NULL;
 
@@ -1879,6 +1956,7 @@ glusterd_mgmt_v3_initiate_all_phases (rpcsvc_request_t *req, glusterd_op_t op,
                 gf_msg (this->name, GF_LOG_ERROR, 0,
                         GD_MSG_DICT_SET_FAILED,
                         "Failed to set originator_uuid.");
+                GF_FREE (originator_uuid);
                 goto out;
         }
 
@@ -2112,6 +2190,7 @@ glusterd_mgmt_v3_initiate_snap_phases (rpcsvc_request_t *req, glusterd_op_t op,
                 gf_msg (this->name, GF_LOG_ERROR, 0,
                         GD_MSG_DICT_SET_FAILED,
                         "Failed to set originator_uuid.");
+                GF_FREE (originator_uuid);
                 goto out;
         }
 
