@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "common-utils.h"
 #include "cli1-xdr.h"
 #include "xdr-generic.h"
@@ -28,6 +23,7 @@
 #include "byte-order.h"
 #include "compat-errno.h"
 #include "quota-common-utils.h"
+#include "glusterd-quota.h"
 
 #include <sys/wait.h>
 #include <dlfcn.h>
@@ -60,12 +56,10 @@ const char *gd_quota_op_list[GF_QUOTA_OPTION_TYPE_MAX + 1] = {
         [GF_QUOTA_OPTION_TYPE_LIST_OBJECTS]       = "list-objects",
         [GF_QUOTA_OPTION_TYPE_REMOVE_OBJECTS]     = "remove-objects",
         [GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS]     = "enable-objects",
+        [GF_QUOTA_OPTION_TYPE_UPGRADE]            = "upgrade",
         [GF_QUOTA_OPTION_TYPE_MAX]                = NULL
 };
 
-int
-glusterd_store_quota_config (glusterd_volinfo_t *volinfo, char *path,
-                             char *gfid_str, int opcode, char **op_errstr);
 
 gf_boolean_t
 glusterd_is_quota_supported (int32_t type, char **op_errstr)
@@ -97,6 +91,20 @@ glusterd_is_quota_supported (int32_t type, char **op_errstr)
              type == GF_QUOTA_OPTION_TYPE_REMOVE))
                 goto out;
 
+        /* Quota xattr version implemented in 3.7.6
+         * quota-version is incremented when quota is enabled
+         * Quota enable and disable performance enhancement has been done
+         * in version 3.7.12.
+         * so don't allow enabling/disabling quota in heterogeneous
+         * cluster during upgrade
+         */
+        if (type == GF_QUOTA_OPTION_TYPE_ENABLE ||
+            type == GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS ||
+            type == GF_QUOTA_OPTION_TYPE_DISABLE) {
+                if (conf->op_version < GD_OP_VERSION_3_7_12)
+                        goto out;
+        }
+
         supported = _gf_true;
 
 out:
@@ -225,32 +233,73 @@ out:
 }
 
 int32_t
-glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv, char *volname,
-                                  int type)
+_glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv,
+                                  glusterd_volinfo_t *volinfo,
+                                  glusterd_brickinfo_t *brick, int type,
+                                  char *pid_dir)
 {
         pid_t                      pid;
-        int32_t                    ret               = 0;
-        int                        status            = 0;
-        char                       mountdir[]        = "/tmp/mntXXXXXX";
-        char                       logfile[PATH_MAX] = {0,};
-        runner_t                   runner            = {0};
+        int32_t                    ret                 = -1;
+        int                        status              = 0;
+        char                       mountdir[PATH_MAX]  = {0,};
+        char                       logfile[PATH_MAX]   = {0,};
+        char                       brickpath[PATH_MAX] = {0,};
+        char                       vol_id[PATH_MAX]    = {0,};
+        char                       pidfile[PATH_MAX]   = {0,};
+        runner_t                   runner              = {0};
+        char                      *volfileserver       = NULL;
+        FILE                      *pidfp               = NULL;
 
-        if (mkdtemp (mountdir) == NULL) {
-                gf_msg_debug ("glusterd", 0,
-                        "failed to create a temporary mount directory");
+        GF_VALIDATE_OR_GOTO ("glusterd", THIS, out);
+
+        GLUSTERD_GET_TMP_PATH (mountdir, "/");
+        ret = sys_mkdir (mountdir, 0777);
+        if (ret && errno != EEXIST) {
+                gf_msg (THIS->name, GF_LOG_WARNING, errno,
+                        GD_MSG_MOUNT_REQ_FAIL, "failed to create temporary "
+                        "directory %s", mountdir);
                 ret = -1;
                 goto out;
         }
+
+        strcat (mountdir, "mntXXXXXX");
+        if (mkdtemp (mountdir) == NULL) {
+                gf_msg (THIS->name, GF_LOG_WARNING, errno,
+                        GD_MSG_MOUNT_REQ_FAIL, "failed to create a temporary "
+                        "mount directory: %s", mountdir);
+                ret = -1;
+                goto out;
+        }
+
+        GLUSTERD_REMOVE_SLASH_FROM_PATH (brick->path, brickpath);
         snprintf (logfile, sizeof (logfile),
-                  DEFAULT_LOG_FILE_DIRECTORY"/%s-quota-crawl.log", volname);
+                  DEFAULT_QUOTA_CRAWL_LOG_DIRECTORY"/%s.log",
+                  brickpath);
+
+        if (dict_get_str (THIS->options, "transport.socket.bind-address",
+                          &volfileserver) != 0)
+                volfileserver = "localhost";
+
+        snprintf (vol_id, sizeof (vol_id), "client_per_brick/%s.%s.%s.%s.vol",
+                  volinfo->volname, "client", brick->hostname, brickpath);
 
         runinit (&runner);
-        runner_add_args (&runner, SBIN_DIR"/glusterfs",
-                         "-s", "localhost",
-                         "--volfile-id", volname,
-			 "--use-readdirp=no",
-                         "--client-pid", QUOTA_CRAWL_PID,
-                         "-l", logfile, mountdir, NULL);
+
+        if (type == GF_QUOTA_OPTION_TYPE_ENABLE ||
+            type == GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS)
+                runner_add_args (&runner, SBIN_DIR"/glusterfs",
+                                 "-s", volfileserver,
+                                 "--volfile-id", vol_id,
+                                 "--use-readdirp=yes",
+                                 "--client-pid", QUOTA_CRAWL_PID,
+                                 "-l", logfile, mountdir, NULL);
+        else
+                runner_add_args (&runner, SBIN_DIR"/glusterfs",
+                                 "-s", volfileserver,
+                                 "--volfile-id", vol_id,
+                                 "--use-readdirp=no",
+                                 "--client-pid", QUOTA_CRAWL_PID,
+                                 "-l", logfile, mountdir, NULL);
 
         synclock_unlock (&priv->big_lock);
         ret = runner_run_reuse (&runner);
@@ -263,7 +312,7 @@ glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv, char *volname,
         runner_end (&runner);
 
         if ((pid = fork ()) < 0) {
-                gf_msg ("glusterd", GF_LOG_WARNING, 0,
+                gf_msg (THIS->name, GF_LOG_WARNING, 0,
                         GD_MSG_FORK_FAIL, "fork from parent failed");
                 ret = -1;
                 goto out;
@@ -277,7 +326,7 @@ glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv, char *volname,
 
                 ret = chdir (mountdir);
                 if (ret == -1) {
-                        gf_msg ("glusterd", GF_LOG_WARNING, errno,
+                        gf_msg (THIS->name, GF_LOG_WARNING, errno,
                                 GD_MSG_DIR_OP_FAILED, "chdir %s failed",
                                 mountdir);
                         exit (EXIT_FAILURE);
@@ -286,9 +335,7 @@ glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv, char *volname,
 
                 if (type == GF_QUOTA_OPTION_TYPE_ENABLE ||
                     type == GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS)
-                        runner_add_args (&runner, "/usr/bin/find", ".",
-                                         "-exec", "/usr/bin/stat",
-                                         "{}", "\\", ";", NULL);
+                        runner_add_args (&runner, "/usr/bin/find", ".", NULL);
 
                 else if (type == GF_QUOTA_OPTION_TYPE_DISABLE) {
 
@@ -312,8 +359,19 @@ glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv, char *volname,
 
                 }
 
-                if (runner_start (&runner) == -1)
+                if (runner_start (&runner) == -1) {
+                        gf_umount_lazy ("glusterd", mountdir, 1);
                         _exit (EXIT_FAILURE);
+                }
+
+                snprintf (pidfile, sizeof (pidfile), "%s/%s.pid", pid_dir,
+                          brickpath);
+                pidfp = fopen (pidfile, "w");
+                if (pidfp) {
+                        fprintf (pidfp, "%d\n", runner.chpid);
+                        fflush (pidfp);
+                        fclose (pidfp);
+                }
 
 #ifndef GF_LINUX_HOST_OS
                 runner_end (&runner); /* blocks in waitpid */
@@ -329,6 +387,100 @@ out:
         return ret;
 }
 
+void
+glusterd_stop_all_quota_crawl_service (glusterd_conf_t *priv,
+                                       glusterd_volinfo_t *volinfo, int type)
+{
+        char                       pid_dir[PATH_MAX]  = {0, };
+        char                       pidfile[PATH_MAX]  = {0,};
+        struct dirent             *entry              = NULL;
+        DIR                       *dir                = NULL;
+
+        GLUSTERD_GET_QUOTA_CRAWL_PIDDIR (pid_dir, volinfo, type);
+
+        dir = sys_opendir (pid_dir);
+        if (dir == NULL)
+                return;
+
+        GF_FOR_EACH_ENTRY_IN_DIR (entry, dir);
+        while (entry) {
+                snprintf (pidfile, sizeof (pidfile), "%s/%s",
+                          pid_dir, entry->d_name);
+
+                glusterd_service_stop_nolock ("quota_crawl", pidfile, SIGKILL,
+                                              _gf_true);
+                sys_unlink (pidfile);
+
+                GF_FOR_EACH_ENTRY_IN_DIR (entry, dir);
+        }
+        sys_closedir (dir);
+}
+
+int32_t
+glusterd_quota_initiate_fs_crawl (glusterd_conf_t *priv,
+                                  glusterd_volinfo_t *volinfo, int type)
+{
+        int32_t                    ret                = -1;
+        glusterd_brickinfo_t      *brick              = NULL;
+        char                       pid_dir[PATH_MAX]  = {0, };
+
+        GF_VALIDATE_OR_GOTO ("glusterd", THIS, out);
+
+        ret = glusterd_generate_client_per_brick_volfile (volinfo);
+        if (ret) {
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                        GD_MSG_GLUSTERD_OP_FAILED,
+                        "failed to generate client volume file");
+                goto out;
+        }
+
+        ret = mkdir_p (DEFAULT_QUOTA_CRAWL_LOG_DIRECTORY, 0777, _gf_true);
+        if (ret) {
+                gf_msg (THIS->name, GF_LOG_ERROR, errno,
+                        GD_MSG_GLUSTERD_OP_FAILED,
+                        "failed to create dir %s: %s",
+                        DEFAULT_QUOTA_CRAWL_LOG_DIRECTORY, strerror (errno));
+                goto out;
+        }
+
+        GLUSTERD_GET_QUOTA_CRAWL_PIDDIR (pid_dir, volinfo, type);
+        ret = mkdir_p (pid_dir, 0777, _gf_true);
+        if (ret) {
+                gf_msg (THIS->name, GF_LOG_ERROR, errno,
+                        GD_MSG_GLUSTERD_OP_FAILED,
+                        "failed to create dir %s: %s",
+                        pid_dir, strerror (errno));
+                goto out;
+        }
+
+        /* When quota enable is performed, stop alreday running enable crawl
+         * process and start fresh crawl process. let disable process continue
+         * if running to cleanup the older xattrs
+         * When quota disable is performed, stop both enable/disable crawl
+         * process and start fresh crawl process to cleanup the xattrs
+         */
+        glusterd_stop_all_quota_crawl_service (priv, volinfo,
+                                               GF_QUOTA_OPTION_TYPE_ENABLE);
+        if (type == GF_QUOTA_OPTION_TYPE_DISABLE)
+                glusterd_stop_all_quota_crawl_service (priv, volinfo,
+                                               GF_QUOTA_OPTION_TYPE_DISABLE);
+
+        cds_list_for_each_entry (brick, &volinfo->bricks, brick_list) {
+                if (gf_uuid_compare (brick->uuid, MY_UUID))
+                        continue;
+
+                ret = _glusterd_quota_initiate_fs_crawl (priv, volinfo, brick,
+                                                         type, pid_dir);
+
+                if (ret)
+                        goto out;
+        }
+
+        ret = 0;
+out:
+        return ret;
+}
+
 int32_t
 glusterd_quota_get_default_soft_limit (glusterd_volinfo_t *volinfo,
                                        dict_t *rsp_dict)
@@ -769,7 +921,7 @@ glusterd_copy_to_tmp_file (int src_fd, int dst_fd)
         this = THIS;
         GF_ASSERT (this);
 
-        while ((bytes_read = read (src_fd, (void *)&buf, entry_sz)) > 0) {
+        while ((bytes_read = sys_read (src_fd, (void *)&buf, entry_sz)) > 0) {
                 if (bytes_read % 16 != 0) {
                         gf_msg (this->name, GF_LOG_ERROR, 0,
                                 GD_MSG_QUOTA_CONF_CORRUPT, "quota.conf "
@@ -777,7 +929,7 @@ glusterd_copy_to_tmp_file (int src_fd, int dst_fd)
                         ret = -1;
                         goto out;
                 }
-                ret = write (dst_fd, (void *) buf, bytes_read);
+                ret = sys_write (dst_fd, (void *) buf, bytes_read);
                 if (ret == -1) {
                         gf_msg (this->name, GF_LOG_ERROR, errno,
                                 GD_MSG_QUOTA_CONF_WRITE_FAIL,
@@ -839,7 +991,7 @@ glusterd_store_quota_conf_upgrade (glusterd_volinfo_t *volinfo)
 
 out:
         if (conf_fd != -1)
-                close (conf_fd);
+                sys_close (conf_fd);
 
         if (ret && (fd > 0)) {
                 gf_store_unlink_tmppath (volinfo->quota_conf_shandle);
@@ -911,7 +1063,7 @@ glusterd_store_quota_config (glusterd_volinfo_t *volinfo, char *path,
 
         if (version < 1.2f && conf->op_version >= GD_OP_VERSION_3_7_0) {
                 /* Upgrade quota.conf file to newer format */
-                close (conf_fd);
+                sys_close (conf_fd);
                 ret = glusterd_store_quota_conf_upgrade(volinfo);
                 if (ret)
                         goto out;
@@ -944,9 +1096,15 @@ glusterd_store_quota_config (glusterd_volinfo_t *volinfo, char *path,
         if (ret)
                 goto out;
 
+
         /* Just create empty quota.conf file if create */
         if (GF_QUOTA_OPTION_TYPE_ENABLE == opcode ||
-            GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS == opcode) {
+            GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS == opcode ||
+            GF_QUOTA_OPTION_TYPE_UPGRADE == opcode) {
+                /* Opcode will be GF_QUOTA_OPTION_TYPE_UPGRADE when there is
+                 * an upgrade from 3.6 to 3.7. Just upgrade the quota.conf
+                 * file even during an op-version bumpup and exit.
+                 */
                 modified = _gf_true;
                 goto out;
         }
@@ -964,7 +1122,7 @@ glusterd_store_quota_config (glusterd_volinfo_t *volinfo, char *path,
                 type = GF_QUOTA_CONF_TYPE_USAGE;
 
         for (;;) {
-                bytes_read = read (conf_fd, (void *)&buf, sizeof (buf));
+                bytes_read = sys_read (conf_fd, (void *)&buf, sizeof (buf));
                 if (bytes_read <= 0) {
                         /*The flag @is_first_read is TRUE when the loop is
                          * entered, and is set to false if the first read
@@ -988,7 +1146,7 @@ glusterd_store_quota_config (glusterd_volinfo_t *volinfo, char *path,
                 found = glusterd_find_gfid_match (gfid, type, buf, bytes_read,
                                                   opcode, &bytes_to_write);
 
-                ret = write (fd, (void *) buf, bytes_to_write);
+                ret = sys_write (fd, (void *) buf, bytes_to_write);
                 if (ret == -1) {
                         gf_msg (this->name, GF_LOG_ERROR, errno,
                                 GD_MSG_QUOTA_CONF_WRITE_FAIL,
@@ -1070,7 +1228,7 @@ glusterd_store_quota_config (glusterd_volinfo_t *volinfo, char *path,
         ret = 0;
 out:
         if (conf_fd != -1) {
-                close (conf_fd);
+                sys_close (conf_fd);
         }
 
         if (ret && (fd > 0)) {
@@ -1499,12 +1657,12 @@ glusterd_op_quota (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
 
 
         if (GF_QUOTA_OPTION_TYPE_ENABLE == type)
-                volinfo->quota_version++;
+                volinfo->quota_xattr_version++;
         ret = glusterd_store_volinfo (volinfo,
                                       GLUSTERD_VOLINFO_VER_AC_INCREMENT);
         if (ret) {
                 if (GF_QUOTA_OPTION_TYPE_ENABLE == type)
-                        volinfo->quota_version--;
+                        volinfo->quota_xattr_version--;
                 goto out;
         }
 
@@ -1515,7 +1673,7 @@ glusterd_op_quota (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
                                                   "volfiles");
                 if (GF_QUOTA_OPTION_TYPE_ENABLE == type) {
                         /* rollback volinfo */
-                        volinfo->quota_version--;
+                        volinfo->quota_xattr_version--;
                         ret = glusterd_store_volinfo (volinfo,
                                       GLUSTERD_VOLINFO_VER_AC_INCREMENT);
                 }
@@ -1530,7 +1688,7 @@ glusterd_op_quota (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         }
 
         if (rsp_dict && start_crawl == _gf_true)
-                glusterd_quota_initiate_fs_crawl (priv, volname, type);
+                glusterd_quota_initiate_fs_crawl (priv, volinfo, type);
 
         ret = 0;
 out:
@@ -1691,6 +1849,94 @@ out:
         return ret;
 }
 
+static int
+glusterd_create_quota_auxiliary_mount (xlator_t *this, char *volname)
+{
+        int                ret                     = -1;
+        int                retry                   = 0;
+        char               mountdir[PATH_MAX]      = {0,};
+        char               pidfile_path[PATH_MAX]  = {0,};
+        char               logfile[PATH_MAX]       = {0,};
+        char               qpid[16]                = {0,};
+        char              *volfileserver           = NULL;
+        glusterd_conf_t   *priv                    = NULL;
+        struct stat        buf                     = {0,};
+
+        GF_VALIDATE_OR_GOTO ("glusterd", this, out);
+        priv = this->private;
+        GF_VALIDATE_OR_GOTO (this->name, priv, out);
+
+        GLUSTERFS_GET_AUX_MOUNT_PIDFILE (pidfile_path, volname);
+
+        if (gf_is_service_running (pidfile_path, NULL)) {
+                gf_msg_debug (this->name, 0, "Aux mount of volume %s is running"
+                              " already", volname);
+                ret = 0;
+                goto out;
+        }
+
+        if (glusterd_is_fuse_available () == _gf_false) {
+                gf_msg (this->name, GF_LOG_ERROR, 0,
+                        GD_MSG_MOUNT_REQ_FAIL, "Fuse unavailable");
+                ret = -1;
+                goto out;
+        }
+
+        GLUSTERD_GET_QUOTA_AUX_MOUNT_PATH (mountdir, volname, "/");
+        ret = sys_mkdir (mountdir, 0777);
+        if (ret && errno != EEXIST) {
+                gf_msg (this->name, GF_LOG_ERROR, errno,
+                        GD_MSG_MOUNT_REQ_FAIL, "Failed to create auxiliary "
+                        "mount directory %s", mountdir);
+                goto out;
+        }
+        snprintf (logfile, PATH_MAX-1, "%s/quota-mount-%s.log",
+                  DEFAULT_LOG_FILE_DIRECTORY, volname);
+        snprintf(qpid, 15, "%d", GF_CLIENT_PID_QUOTA_MOUNT);
+
+        if (dict_get_str (this->options, "transport.socket.bind-address",
+                          &volfileserver) != 0)
+                volfileserver = "localhost";
+
+        synclock_unlock (&priv->big_lock);
+        ret = runcmd (SBIN_DIR"/glusterfs",
+                      "--volfile-server", volfileserver,
+                      "--volfile-id", volname,
+                      "-l", logfile,
+                      "-p", pidfile_path,
+                      "--client-pid", qpid,
+                      mountdir,
+                      NULL);
+        if (ret == 0) {
+                /* Block here till mount process is ready to accept FOPs.
+                 * Else, if glusterd acquires biglock below before
+                 * mount process is ready, then glusterd and mount process
+                 * can get into a deadlock situation.
+                 */
+                ret = sys_stat (mountdir, &buf);
+                if (ret < 0)
+                        ret = -errno;
+        } else {
+                ret = -errno;
+        }
+
+        synclock_lock (&priv->big_lock);
+
+        if (ret) {
+                gf_msg (this->name, GF_LOG_ERROR, -ret,
+                        GD_MSG_MOUNT_REQ_FAIL, "Failed to mount glusterfs "
+                        "client. Please check the log file %s for more details",
+                        logfile);
+                ret = -1;
+                goto out;
+        }
+
+        ret = 0;
+
+out:
+        return ret;
+}
+
 int
 glusterd_op_stage_quota (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
 {
@@ -1778,25 +2024,35 @@ glusterd_op_stage_quota (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         }
 
         switch (type) {
-        case GF_QUOTA_OPTION_TYPE_ENABLE:
-        case GF_QUOTA_OPTION_TYPE_ENABLE_OBJECTS:
         case GF_QUOTA_OPTION_TYPE_LIST:
         case GF_QUOTA_OPTION_TYPE_LIST_OBJECTS:
-                /* Fuse mount req. only for enable & list-usage options*/
-                if (is_origin_glusterd (dict) &&
-                    !glusterd_is_fuse_available ()) {
-                        *op_errstr = gf_strdup ("Fuse unavailable");
-                        ret = -1;
-                        goto out;
+        case GF_QUOTA_OPTION_TYPE_LIMIT_USAGE:
+        case GF_QUOTA_OPTION_TYPE_LIMIT_OBJECTS:
+        case GF_QUOTA_OPTION_TYPE_REMOVE:
+        case GF_QUOTA_OPTION_TYPE_REMOVE_OBJECTS:
+                /* Quota auxiliary mount is needed by CLI
+                 * for list command and need by glusterd for
+                 * setting/removing limit
+                 */
+                if (is_origin_glusterd (dict)) {
+                        ret = glusterd_create_quota_auxiliary_mount (this,
+                                                                     volname);
+                        if (ret) {
+                                *op_errstr = gf_strdup ("Failed to start aux "
+                                                        "mount");
+                                goto out;
+                        }
                 }
                 break;
+        }
 
+        switch (type) {
         case GF_QUOTA_OPTION_TYPE_LIMIT_USAGE:
                 ret = dict_get_str (dict, "hard-limit", &hard_limit_str);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_ERROR, 0,
                                 GD_MSG_DICT_GET_FAILED,
-                                "Faild to get hard-limit from dict");
+                                "Failed to get hard-limit from dict");
                         goto out;
                 }
                 ret = gf_string2bytesize_int64 (hard_limit_str, &hard_limit);
