@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include <inttypes.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -140,8 +135,7 @@ __glusterd_defrag_notify (struct rpc_clnt *rpc, void *mydata,
                 UNLOCK (&defrag->lock);
 
                 if (!gf_is_service_running (pidfile, NULL)) {
-                        if (volinfo->type != GF_CLUSTER_TYPE_TIER &&
-                            volinfo->rebal.defrag_status ==
+                        if (volinfo->rebal.defrag_status ==
                                                 GF_DEFRAG_STATUS_STARTED) {
                                 volinfo->rebal.defrag_status =
                                                    GF_DEFRAG_STATUS_FAILED;
@@ -150,6 +144,7 @@ __glusterd_defrag_notify (struct rpc_clnt *rpc, void *mydata,
 
                 glusterd_store_perform_node_state_store (volinfo);
 
+                rpc_clnt_reconnect_cleanup (&defrag->rpc->conn);
                 glusterd_defrag_rpc_put (defrag);
                 if (defrag->cbk_fn)
                         defrag->cbk_fn (volinfo,
@@ -270,7 +265,8 @@ glusterd_handle_defrag_start (glusterd_volinfo_t *volinfo, char *op_errstr,
                                                    priv);
                 snprintf (logfile, PATH_MAX, "%s/%s-%s-%s.log",
                           DEFAULT_LOG_FILE_DIRECTORY, volinfo->volname,
-                          (cmd == GF_DEFRAG_CMD_START_TIER ? "tier":"rebalance"),
+                          (cmd == GF_DEFRAG_CMD_START_TIER ?
+                           "tier":"rebalance"),
                           uuid_utoa(MY_UUID));
 
         } else {
@@ -331,6 +327,40 @@ out:
         return ret;
 }
 
+int
+glusterd_rebalance_defrag_init (glusterd_volinfo_t *volinfo,
+                                defrag_cbk_fn_t cbk)
+
+{
+        glusterd_defrag_info_t  *defrag         = NULL;
+        int                      ret            = -1;
+
+        if (!volinfo->rebal.defrag) {
+                volinfo->rebal.defrag =
+                        GF_CALLOC (1, sizeof (*volinfo->rebal.defrag),
+                                   gf_gld_mt_defrag_info);
+        } else {
+                /*
+                 * if defrag variable is already initialized,
+                 * we skip the initialization.
+                 */
+                ret = 0;
+                goto out;
+        }
+
+        if (!volinfo->rebal.defrag)
+                goto out;
+        defrag = volinfo->rebal.defrag;
+
+        defrag->cmd = volinfo->rebal.defrag_cmd;
+        LOCK_INIT (&defrag->lock);
+        if (cbk)
+                defrag->cbk_fn = cbk;
+        ret = 0;
+out:
+        return ret;
+
+}
 
 int
 glusterd_rebalance_rpc_create (glusterd_volinfo_t *volinfo,
@@ -528,7 +558,8 @@ __glusterd_handle_defrag_volume (rpcsvc_request_t *req)
         if ((cmd == GF_DEFRAG_CMD_STATUS) ||
             (cmd == GF_DEFRAG_CMD_STATUS_TIER) ||
             (cmd == GF_DEFRAG_CMD_STOP_DETACH_TIER) ||
-              (cmd == GF_DEFRAG_CMD_STOP)) {
+            (cmd == GF_DEFRAG_CMD_STOP) ||
+            (cmd == GF_DEFRAG_CMD_DETACH_STATUS)) {
                 ret = glusterd_op_begin (req, GD_OP_DEFRAG_BRICK_VOLUME,
                                          dict, msg, sizeof (msg));
         } else
@@ -572,7 +603,8 @@ glusterd_brick_validation  (dict_t *dict, char *key, data_t *value,
         GF_ASSERT (this);
 
         ret = glusterd_volume_brickinfo_get_by_brick (value->data, volinfo,
-                                                      &brickinfo);
+                                                      &brickinfo,
+                                                      _gf_false);
         if (ret) {
                 gf_msg (this->name, GF_LOG_ERROR, EINVAL,
                         GD_MSG_BRICK_NOT_FOUND,
@@ -601,9 +633,12 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
         int32_t                 cmd          = 0;
         char                    msg[2048]    = {0};
         glusterd_volinfo_t      *volinfo     = NULL;
+        glusterd_brickinfo_t    *brickinfo   = NULL;
+        glusterd_peerinfo_t     *peerinfo    = NULL;
         char                    *task_id_str = NULL;
         dict_t                  *op_ctx      = NULL;
         xlator_t                *this        = 0;
+        int32_t                 is_force     = 0;
 
         this = THIS;
         GF_ASSERT (this);
@@ -628,16 +663,20 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
         }
         switch (cmd) {
         case GF_DEFRAG_CMD_START_TIER:
+                ret = dict_get_int32 (dict, "force", &is_force);
+                if (ret)
+                        is_force = 0;
+
                 if (volinfo->type != GF_CLUSTER_TYPE_TIER) {
                         gf_asprintf (op_errstr, "volume %s is not a tier "
                                      "volume.", volinfo->volname);
                         ret = -1;
                         goto out;
                 }
-                if (glusterd_is_tier_daemon_running (volinfo)) {
+                if ((!is_force) && glusterd_is_tier_daemon_running (volinfo)) {
                         ret = gf_asprintf (op_errstr, "A Tier daemon is "
-                                           "already running on volume %s",
-                                           volname);
+                                        "already running on volume %s",
+                                        volname);
                         ret = -1;
                         goto out;
                 }
@@ -663,6 +702,43 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                         goto out;
                 }
 
+                cds_list_for_each_entry (brickinfo, &volinfo->bricks,
+                                         brick_list) {
+                        if (glusterd_is_local_brick (THIS, volinfo, brickinfo)) {
+                                if (brickinfo->status != GF_BRICK_STARTED) {
+                                        gf_asprintf (op_errstr, "Received"
+                                                     " rebalance on volume with "
+                                                     " stopped brick %s",
+                                                     brickinfo->path);
+                                        ret = -1;
+                                        goto out;
+                                }
+                        } else {
+                                rcu_read_lock ();
+                                peerinfo = glusterd_peerinfo_find_by_uuid
+                                           (brickinfo->uuid);
+                                if (!peerinfo) {
+                                        gf_asprintf (op_errstr, "Host node %s "
+                                                     "of brick %s doesn't "
+                                                     "belong to cluster",
+                                                     brickinfo->hostname,
+                                                     brickinfo->path);
+                                        ret = -1;
+                                        rcu_read_unlock ();
+                                        goto out;
+                                } else if (!peerinfo->connected) {
+                                        gf_asprintf (op_errstr, "Host node %s "
+                                                     "of brick %s is down",
+                                                     brickinfo->hostname,
+                                                     brickinfo->path);
+                                        ret = -1;
+                                        rcu_read_unlock ();
+                                        goto out;
+                                }
+                                rcu_read_unlock ();
+                        }
+                }
+
         case GF_DEFRAG_CMD_START_FORCE:
                 if (is_origin_glusterd (dict)) {
                         op_ctx = glusterd_op_get_ctx ();
@@ -705,7 +781,7 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
         case GF_DEFRAG_CMD_STATUS_TIER:
         case GF_DEFRAG_CMD_STATUS:
         case GF_DEFRAG_CMD_STOP:
-        case GF_DEFRAG_CMD_STOP_DETACH_TIER:
+
                 ret = dict_get_str (dict, "cmd-str", &cmd_str);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_ERROR, 0,
@@ -720,6 +796,7 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                         ret = -1;
                         goto out;
                 }
+
                 if (strstr(cmd_str, "remove-brick") != NULL) {
                         if (volinfo->rebal.op != GD_OP_REMOVE_BRICK) {
                                 snprintf (msg, sizeof(msg), "remove-brick not "
@@ -740,22 +817,33 @@ glusterd_op_stage_rebalance (dict_t *dict, char **op_errstr)
                                 goto out;
                         }
                 }
-                if (strstr(cmd_str, "detach-tier") != NULL) {
+                if (cmd == GF_DEFRAG_CMD_STATUS_TIER) {
                         if (volinfo->type != GF_CLUSTER_TYPE_TIER) {
                                 snprintf (msg, sizeof(msg), "volume %s is not "
                                           "a tier volume.", volinfo->volname);
                                 ret = -1;
                                 goto out;
                         }
-                        if (volinfo->rebal.op != GD_OP_REMOVE_BRICK) {
-                                snprintf (msg, sizeof(msg), "Detach-tier"
-                                          "not started");
-                                ret = -1;
-                                goto out;
-                        }
                 }
 
                 break;
+
+        case GF_DEFRAG_CMD_STOP_DETACH_TIER:
+        case GF_DEFRAG_CMD_DETACH_STATUS:
+                if (volinfo->type != GF_CLUSTER_TYPE_TIER) {
+                        snprintf (msg, sizeof(msg), "volume %s is not "
+                                  "a tier volume.", volinfo->volname);
+                        ret = -1;
+                        goto out;
+                }
+
+                if (volinfo->rebal.op != GD_OP_REMOVE_BRICK) {
+                        snprintf (msg, sizeof(msg), "Detach-tier "
+                                  "not started");
+                        ret = -1;
+                        goto out;
+                }
+                break;
         default:
                 break;
         }
@@ -784,6 +872,7 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         dict_t                  *ctx = NULL;
         xlator_t                *this = NULL;
         uint32_t                commit_hash;
+        int32_t                 is_force    = 0;
 
         this = THIS;
         GF_ASSERT (this);
@@ -847,39 +936,79 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
         case GF_DEFRAG_CMD_START_LAYOUT_FIX:
         case GF_DEFRAG_CMD_START_FORCE:
         case GF_DEFRAG_CMD_START_TIER:
-                /* Reset defrag status to 'NOT STARTED' whenever a
-                 * remove-brick/rebalance command is issued to remove
-                 * stale information from previous run.
-                 */
-                volinfo->rebal.defrag_status = GF_DEFRAG_STATUS_NOT_STARTED;
 
-                ret = dict_get_str (dict, GF_REBALANCE_TID_KEY, &task_id_str);
-                if (ret) {
-                        gf_msg_debug (this->name, 0, "Missing rebalance "
-                                "id");
-                        ret = 0;
-                } else {
-                        gf_uuid_parse (task_id_str, volinfo->rebal.rebalance_id) ;
-                        volinfo->rebal.op = GD_OP_REBALANCE;
-                }
-                if (!gd_should_i_start_rebalance (volinfo)) {
-                        /* Store the rebalance-id and rebalance command even if
-                         * the peer isn't starting a rebalance process. On peers
-                         * where a rebalance process is started,
-                         * glusterd_handle_defrag_start performs the storing.
-                         *
-                         * Storing this is needed for having 'volume status'
-                         * work correctly.
+
+                ret = dict_get_int32 (dict, "force", &is_force);
+                if (ret)
+                        is_force = 0;
+                if (!is_force) {
+                        /* Reset defrag status to 'NOT STARTED' whenever a
+                         * remove-brick/rebalance command is issued to remove
+                         * stale information from previous run.
                          */
-                        glusterd_store_perform_node_state_store (volinfo);
+                        volinfo->rebal.defrag_status =
+                               GF_DEFRAG_STATUS_NOT_STARTED;
+
+                        ret = dict_get_str (dict, GF_REBALANCE_TID_KEY,
+                                            &task_id_str);
+                        if (ret) {
+                                gf_msg_debug (this->name, 0, "Missing rebalance"
+                                              " id");
+                                ret = 0;
+                        } else {
+                                gf_uuid_parse (task_id_str,
+                                               volinfo->rebal.rebalance_id);
+                                volinfo->rebal.op = GD_OP_REBALANCE;
+                        }
+                        if (!gd_should_i_start_rebalance (volinfo)) {
+                                /* Store the rebalance-id and rebalance command
+                                 * even if the peer isn't starting a rebalance
+                                 * process. On peers where a rebalance process
+                                 * is started, glusterd_handle_defrag_start
+                                 * performs the storing.
+                                 * Storing this is needed for having
+                                 * 'volume status' work correctly.
+                                 */
+                                glusterd_store_perform_node_state_store
+                                        (volinfo);
+                                break;
+                        }
+                        if (dict_get_uint32 (dict, "commit-hash", &commit_hash)
+                                             == 0) {
+                                volinfo->rebal.commit_hash = commit_hash;
+                        }
+                        ret = glusterd_handle_defrag_start (volinfo, msg,
+                                        sizeof (msg),
+                                        cmd, NULL, GD_OP_REBALANCE);
+                        break;
+                } else {
+                        /* Reset defrag status to 'STARTED' so that the
+                         * pid is checked and restarted accordingly.
+                         * If the pid is not running it executes the
+                         * "NOT_STARTED" case and restarts the process
+                         */
+                        volinfo->rebal.defrag_status = GF_DEFRAG_STATUS_STARTED;
+                        volinfo->rebal.defrag_cmd = cmd;
+                        volinfo->rebal.op = GD_OP_REBALANCE;
+
+                        ret = dict_get_str (dict, GF_REBALANCE_TID_KEY,
+                                            &task_id_str);
+                        if (ret) {
+                                gf_msg_debug (this->name, 0, "Missing rebalance"
+                                              " id");
+                                ret = 0;
+                        } else {
+                                gf_uuid_parse (task_id_str,
+                                               volinfo->rebal.rebalance_id);
+                                volinfo->rebal.op = GD_OP_REBALANCE;
+                        }
+                        if (dict_get_uint32 (dict, "commit-hash", &commit_hash)
+                                             == 0) {
+                                volinfo->rebal.commit_hash = commit_hash;
+                        }
+                        ret = glusterd_restart_rebalance_for_volume (volinfo);
                         break;
                 }
-                if (dict_get_uint32 (dict, "commit-hash", &commit_hash) == 0) {
-                        volinfo->rebal.commit_hash = commit_hash;
-                }
-                ret = glusterd_handle_defrag_start (volinfo, msg, sizeof (msg),
-                                                    cmd, NULL, GD_OP_REBALANCE);
-                break;
         case GF_DEFRAG_CMD_STOP:
         case GF_DEFRAG_CMD_STOP_DETACH_TIER:
                 /* Clear task-id only on explicitly stopping rebalance.
@@ -920,6 +1049,15 @@ glusterd_op_rebalance (dict_t *dict, char **op_errstr, dict_t *rsp_dict)
                         goto out;
                 }
 
+                if (volinfo->type == GF_CLUSTER_TYPE_TIER &&
+                                cmd == GF_OP_CMD_STOP_DETACH_TIER) {
+                        glusterd_defrag_info_set (volinfo, dict,
+                                        GF_DEFRAG_CMD_START_TIER,
+                                        GF_DEFRAG_CMD_START,
+                                        GD_OP_REBALANCE);
+                        glusterd_restart_rebalance_for_volume (volinfo);
+                }
+
                 ret = 0;
                 break;
 
@@ -969,7 +1107,7 @@ glusterd_defrag_event_notify_handle (dict_t *dict)
         } else {
                 gf_msg (this->name, GF_LOG_ERROR, 0,
                         GD_MSG_NO_REBALANCE_PFX_IN_VOLNAME,
-                        "volname recieved (%s) is not prefixed with rebalance.",
+                        "volname received (%s) is not prefixed with rebalance.",
                         volname);
                 ret = -1;
                 goto out;
