@@ -8,11 +8,6 @@
    cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "rpc-clnt.h"
 #include "glusterd1-xdr.h"
 #include "cli1-xdr.h"
@@ -147,6 +142,7 @@ glusterd_op_send_cli_response (glusterd_op_t op, int32_t op_ret,
         case GD_OP_SNAP:
         case GD_OP_BARRIER:
         case GD_OP_BITROT:
+        case GD_OP_SCRUB_STATUS:
         {
                 /*nothing specific to be done*/
                 break;
@@ -329,8 +325,8 @@ __glusterd_probe_cbk (struct rpc_req *req, struct iovec *iov,
                         goto reply;
                 }
 
-                /* Injecting LOCAL_ACC to send update */
-                ret = glusterd_friend_sm_new_event (GD_FRIEND_EVENT_LOCAL_ACC,
+                /* Injecting EVENT_NEW_NAME to send update */
+                ret = glusterd_friend_sm_new_event (GD_FRIEND_EVENT_NEW_NAME,
                                                     &event);
                 if (!ret) {
                         event->peername = gf_strdup (peerinfo->hostname);
@@ -423,7 +419,7 @@ out:
         GLUSTERD_STACK_DESTROY (((call_frame_t *)myframe));
 
         /* Attempt to start the state machine. Needed as no state machine could
-         * be running at time this RPC reply was recieved
+         * be running at time this RPC reply was received
          */
         if (!ret) {
                 glusterd_friend_sm ();
@@ -536,6 +532,7 @@ out:
                 glusterd_friend_sm ();
                 glusterd_op_sm ();
         }
+
         if (ctx)
                 glusterd_destroy_probe_ctx (ctx);
         free (rsp.hostname);//malloced by xdr
@@ -562,7 +559,7 @@ __glusterd_friend_remove_cbk (struct rpc_req * req, struct iovec *iov,
         glusterd_friend_sm_event_type_t event_type = GD_FRIEND_EVENT_NONE;
         glusterd_peerinfo_t             *peerinfo = NULL;
         int32_t                         op_ret = -1;
-        int32_t                         op_errno = -1;
+        int32_t                         op_errno = 0;
         glusterd_probe_ctx_t            *ctx = NULL;
         gf_boolean_t                    move_sm_now = _gf_true;
 
@@ -788,6 +785,15 @@ __glusterd_cluster_lock_cbk (struct rpc_req *req, struct iovec *iov,
         }
 
 out:
+
+        ret = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
+
         ret = glusterd_op_sm_inject_event (event_type, txn_id, NULL);
 
         if (!ret) {
@@ -898,6 +904,14 @@ glusterd_mgmt_v3_lock_peers_cbk_fn (struct rpc_req *req, struct iovec *iov,
         }
 
 out:
+
+        ret = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
         ret = glusterd_op_sm_inject_event (event_type, txn_id, NULL);
         if (!ret) {
                 glusterd_friend_sm ();
@@ -1001,6 +1015,14 @@ glusterd_mgmt_v3_unlock_peers_cbk_fn (struct rpc_req *req, struct iovec *iov,
         }
 
 out:
+
+        ret = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
         ret = glusterd_op_sm_inject_event (event_type, txn_id, NULL);
 
         if (!ret) {
@@ -1099,6 +1121,14 @@ __glusterd_cluster_unlock_cbk (struct rpc_req *req, struct iovec *iov,
         }
 
 out:
+
+        ret = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
         ret = glusterd_op_sm_inject_event (event_type, txn_id, NULL);
 
         if (!ret) {
@@ -1233,6 +1263,13 @@ out:
         rcu_read_unlock ();
 
 
+        ret = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
         ret = glusterd_op_sm_inject_event (event_type, txn_id, NULL);
 
         if (!ret) {
@@ -1408,6 +1445,14 @@ unlock:
         rcu_read_unlock ();
 
 out:
+
+        ret = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
         ret = glusterd_op_sm_inject_event (event_type, txn_id, NULL);
 
         if (!ret) {
@@ -1671,6 +1716,9 @@ glusterd_rpc_friend_update (call_frame_t *frame, xlator_t *this,
 out:
         GF_FREE (req.friends.friends_val);
 
+        if (ret && dummy_frame)
+                STACK_DESTROY (dummy_frame->root);
+
         gf_msg_debug ("glusterd", 0, "Returning %d", ret);
         return ret;
 }
@@ -1706,6 +1754,9 @@ glusterd_cluster_lock (call_frame_t *frame, xlator_t *this,
                                        (xdrproc_t)xdr_gd1_mgmt_cluster_lock_req);
 out:
         gf_msg_debug ("glusterd", 0, "Returning %d", ret);
+
+        if (ret && dummy_frame)
+                STACK_DESTROY (dummy_frame->root);
         return ret;
 }
 
@@ -1781,6 +1832,10 @@ glusterd_mgmt_v3_lock_peers (call_frame_t *frame, xlator_t *this,
                                        (xdrproc_t)xdr_gd1_mgmt_v3_lock_req);
 out:
         gf_msg_debug (this->name, 0, "Returning %d", ret);
+        if (dict)
+                dict_unref (dict);
+        if (req.dict.dict_val)
+                GF_FREE (req.dict.dict_val);
         return ret;
 }
 
@@ -1858,6 +1913,11 @@ glusterd_mgmt_v3_unlock_peers (call_frame_t *frame, xlator_t *this,
                                        xdr_gd1_mgmt_v3_unlock_req);
 out:
         gf_msg_debug (this->name, 0, "Returning %d", ret);
+        if (dict)
+                dict_unref(dict);
+
+        if (req.dict.dict_val)
+                GF_FREE (req.dict.dict_val);
         return ret;
 }
 
@@ -1892,6 +1952,10 @@ glusterd_cluster_unlock (call_frame_t *frame, xlator_t *this,
                                        (xdrproc_t)xdr_gd1_mgmt_cluster_unlock_req);
 out:
         gf_msg_debug (this ? this->name : "glusterd", 0, "Returning %d", ret);
+
+        if (ret && dummy_frame)
+                STACK_DESTROY (dummy_frame->root);
+
         return ret;
 }
 
@@ -2192,6 +2256,7 @@ glusterd_brick_op (call_frame_t *frame, xlator_t *this,
 
         gd1_mgmt_brick_op_req           *req = NULL;
         int                             ret = 0;
+        int                             ret1 = 0;
         glusterd_conf_t                 *priv = NULL;
         call_frame_t                    *dummy_frame = NULL;
         char                            *op_errstr = NULL;
@@ -2239,6 +2304,7 @@ glusterd_brick_op (call_frame_t *frame, xlator_t *this,
                 if ((pending_node->type == GD_NODE_NFS) ||
                     (pending_node->type == GD_NODE_QUOTAD) ||
                     (pending_node->type == GD_NODE_SNAPD) ||
+                    (pending_node->type == GD_NODE_SCRUB) ||
                     ((pending_node->type == GD_NODE_SHD) &&
                      (req_ctx->op == GD_OP_STATUS_VOLUME)))
                         ret = glusterd_node_op_build_payload
@@ -2314,6 +2380,17 @@ glusterd_brick_op (call_frame_t *frame, xlator_t *this,
         opinfo.brick_pending_count = pending_bricks;
 
 out:
+
+        if (ret)
+                opinfo.op_ret = ret;
+
+        ret1 = glusterd_set_txn_opinfo (txn_id, &opinfo);
+        if (ret1)
+                gf_msg (THIS->name, GF_LOG_ERROR, 0,
+                                GD_MSG_TRANS_OPINFO_SET_FAIL,
+                                "Unable to set "
+                                "transaction's opinfo");
+
         if (ret) {
                 glusterd_op_sm_inject_event (GD_OP_EVENT_RCVD_RJT,
                                              txn_id, data);
@@ -2321,7 +2398,6 @@ out:
         }
 
         gf_msg_debug (this ? this->name : "glusterd", 0, "Returning %d", ret);
-
         return ret;
 }
 
