@@ -20,6 +20,7 @@
 #include "glusterd-snapd-svc.h"
 #include "glusterd-snapd-svc-helper.h"
 #include "glusterd-snapshot-utils.h"
+#include "syscall.h"
 
 char *snapd_svc_name = "snapd";
 
@@ -251,7 +252,7 @@ glusterd_snapdsvc_start (glusterd_svc_t *svc, int flags)
                 goto out;
         }
 
-        ret = access (svc->proc.volfile, F_OK);
+        ret = sys_access (svc->proc.volfile, F_OK);
         if (ret) {
                 gf_msg (this->name, GF_LOG_DEBUG, 0,
                         GD_MSG_VOLINFO_GET_FAIL,
@@ -294,18 +295,9 @@ glusterd_snapdsvc_start (glusterd_svc_t *svc, int flags)
                          "--brick-name", snapd_id,
                          "-S", svc->conn.sockpath, NULL);
 
-        snapd_port = volinfo->snapd.port;
-        if (!snapd_port) {
-                snapd_port = pmap_registry_alloc (THIS);
-                if (!snapd_port) {
-                        snprintf (msg, sizeof (msg), "Could not allocate port "
-                                  "for snapd service for volume %s",
-                                  volinfo->volname);
-                        runner_log (&runner, this->name, GF_LOG_DEBUG, msg);
-                        ret = -1;
-                        goto out;
-                }
-        }
+        snapd_port = pmap_assign_port (THIS, volinfo->snapd.port, snapd_id);
+        volinfo->snapd.port = snapd_port;
+
         runner_add_arg (&runner, "--brick-port");
         runner_argprintf (&runner, "%d", snapd_port);
         runner_add_arg (&runner, "--xlator-option");
@@ -326,7 +318,6 @@ glusterd_snapdsvc_start (glusterd_svc_t *svc, int flags)
                 }
                 synclock_lock (&priv->big_lock);
         }
-        volinfo->snapd.port = snapd_port;
 
 out:
         return ret;
@@ -348,15 +339,14 @@ glusterd_snapdsvc_restart ()
 
         cds_list_for_each_entry (volinfo, &conf->volumes, vol_list) {
                 /* Start per volume snapd svc */
-                if (volinfo->status == GLUSTERD_STATUS_STARTED &&
-                    glusterd_is_snapd_enabled (volinfo)) {
+                if (volinfo->status == GLUSTERD_STATUS_STARTED) {
                         svc = &(volinfo->snapd.svc);
-                        ret = svc->start (svc, PROC_START_NO_WAIT);
+                        ret = svc->manager (svc, volinfo, PROC_START_NO_WAIT);
                         if (ret) {
                                 gf_msg (this->name, GF_LOG_ERROR, 0,
                                         GD_MSG_SNAPD_START_FAIL,
-                                        "Couldn't start snapd for "
-                                        "vol: %s", volinfo->volname);
+                                        "Couldn't resolve snapd for "
+                                        "vol: %s on restart", volinfo->volname);
                                 goto out;
                         }
                 }
