@@ -215,6 +215,8 @@ out:
         iobref_unref (iobref);
         iobuf_unref (iobuf);
 
+        if (ret && frame)
+                STACK_DESTROY (frame->root);
         return ret;
 }
 
@@ -304,6 +306,9 @@ glusterd_syncop_aggr_rsp_dict (glusterd_op_t op, dict_t *aggr, dict_t *rsp)
                         goto out;
         break;
 
+        case GD_OP_SCRUB_STATUS:
+                ret = glusterd_volume_bitrot_scrub_use_rsp_dict (aggr, rsp);
+        break;
         default:
         break;
         }
@@ -358,8 +363,11 @@ out:
                                    GLUSTERD_MGMT_V3_LOCK, *peerid, rsp.uuid);
 
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -380,7 +388,6 @@ gd_syncop_mgmt_v3_lock (glusterd_op_t op, dict_t *op_ctx,
 {
         int                      ret  = -1;
         gd1_mgmt_v3_lock_req     req  = {{0},};
-        glusterd_conf_t         *conf = THIS->private;
         uuid_t                  *peerid = NULL;
 
         GF_ASSERT(op_ctx);
@@ -408,6 +415,7 @@ gd_syncop_mgmt_v3_lock (glusterd_op_t op, dict_t *op_ctx,
                                         (xdrproc_t)
                                         xdr_gd1_mgmt_v3_lock_req);
 out:
+        GF_FREE (req.dict.dict_val);
         gf_msg_debug ("glusterd", 0, "Returning %d", ret);
         return ret;
 }
@@ -459,8 +467,11 @@ out:
                                    GLUSTERD_MGMT_V3_UNLOCK, *peerid, rsp.uuid);
 
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -480,7 +491,6 @@ gd_syncop_mgmt_v3_unlock (dict_t *op_ctx, glusterd_peerinfo_t *peerinfo,
 {
         int                          ret  = -1;
         gd1_mgmt_v3_unlock_req       req  = {{0},};
-        glusterd_conf_t             *conf = THIS->private;
         uuid_t                      *peerid = NULL;
 
         GF_ASSERT(op_ctx);
@@ -507,6 +517,7 @@ gd_syncop_mgmt_v3_unlock (dict_t *op_ctx, glusterd_peerinfo_t *peerinfo,
                                         (xdrproc_t)
                                         xdr_gd1_mgmt_v3_unlock_req);
 out:
+        GF_FREE (req.dict.dict_val);
         gf_msg_debug ("glusterd", 0, "Returning %d", ret);
         return ret;
 }
@@ -571,8 +582,11 @@ out:
                            GLUSTERD_MGMT_CLUSTER_LOCK, *peerid, rsp.uuid);
 
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -591,7 +605,6 @@ gd_syncop_mgmt_lock (glusterd_peerinfo_t *peerinfo, struct syncargs *args,
 {
         int                       ret = -1;
         gd1_mgmt_cluster_lock_req req  = {{0},};
-        glusterd_conf_t           *conf = THIS->private;
         uuid_t                    *peerid = NULL;
 
         gf_uuid_copy (req.uuid, my_uuid);
@@ -665,8 +678,11 @@ out:
                            GLUSTERD_MGMT_CLUSTER_UNLOCK, *peerid, rsp.uuid);
 
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -686,7 +702,6 @@ gd_syncop_mgmt_unlock (glusterd_peerinfo_t *peerinfo, struct syncargs *args,
 {
         int                         ret     = -1;
         gd1_mgmt_cluster_unlock_req req     = {{0},};
-        glusterd_conf_t             *conf   = THIS->private;
         uuid_t                      *peerid = NULL;
 
         gf_uuid_copy (req.uuid, my_uuid);
@@ -794,8 +809,11 @@ out:
         if (rsp_dict)
                 dict_unref (rsp_dict);
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
         return 0;
 }
@@ -815,7 +833,6 @@ gd_syncop_mgmt_stage_op (glusterd_peerinfo_t *peerinfo, struct syncargs *args,
                          dict_t *dict_out, dict_t *op_ctx)
 {
         gd1_mgmt_stage_op_req *req  = NULL;
-        glusterd_conf_t       *conf = THIS->private;
         int                   ret   = -1;
         uuid_t                *peerid = NULL;
 
@@ -903,7 +920,11 @@ out:
                 free (rsp.op_errstr);
         free (rsp.output.output_val);
 
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         __wake (args);
 
         return 0;
@@ -932,7 +953,7 @@ gd_syncop_mgmt_brick_op (struct rpc_clnt *rpc, glusterd_pending_node_t *pnode,
         args.op_errno = ENOTCONN;
 
         if ((pnode->type == GD_NODE_NFS) ||
-            (pnode->type == GD_NODE_QUOTAD) ||
+            (pnode->type == GD_NODE_QUOTAD) || (pnode->type == GD_NODE_SCRUB) ||
             ((pnode->type == GD_NODE_SHD) && (op == GD_OP_STATUS_VOLUME))) {
                 ret = glusterd_node_op_build_payload (op, &req, dict_out);
 
@@ -1080,8 +1101,11 @@ out:
         if (rsp_dict)
                 dict_unref (rsp_dict);
         GF_FREE (peerid);
-
-        STACK_DESTROY (frame->root);
+        /* req->rpc_status set to -1 means, STACK_DESTROY will be called from
+         * the caller function.
+         */
+        if (req->rpc_status != -1)
+                STACK_DESTROY (frame->root);
         synctask_barrier_wake(args);
 
         return 0;
@@ -1101,7 +1125,6 @@ gd_syncop_mgmt_commit_op (glusterd_peerinfo_t *peerinfo, struct syncargs *args,
                           uuid_t my_uuid, uuid_t recv_uuid,
                           int op, dict_t *dict_out, dict_t *op_ctx)
 {
-        glusterd_conf_t        *conf = THIS->private;
         gd1_mgmt_commit_op_req *req  = NULL;
         int                    ret  = -1;
         uuid_t                 *peerid = NULL;
@@ -1717,7 +1740,7 @@ gd_sync_task_begin (dict_t *op_ctx, rpcsvc_request_t * req)
         int                         op_ret           = -1;
         dict_t                      *req_dict        = NULL;
         glusterd_conf_t             *conf            = NULL;
-        glusterd_op_t               op               = 0;
+        glusterd_op_t               op               = GD_OP_NONE;
         int32_t                     tmp_op           = 0;
         char                        *op_errstr       = NULL;
         char                        *tmp             = NULL;
