@@ -10,11 +10,6 @@
 #ifndef _GLUSTERD_UTILS_H
 #define _GLUSTERD_UTILS_H
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include <pthread.h>
 #include "compat-uuid.h"
 
@@ -70,6 +65,9 @@ glusterd_unlock (uuid_t owner);
 int32_t
 glusterd_get_uuid (uuid_t *uuid);
 
+char*
+gd_get_shd_key (int type);
+
 int
 glusterd_submit_reply (rpcsvc_request_t *req, void *arg,
                        struct iovec *payload, int payloadcount,
@@ -115,7 +113,10 @@ int32_t
 glusterd_brickinfo_new (glusterd_brickinfo_t **brickinfo);
 
 int32_t
-glusterd_brickinfo_new_from_brick (char *brick, glusterd_brickinfo_t **brickinfo);
+glusterd_brickinfo_new_from_brick (char *brick,
+                                   glusterd_brickinfo_t **brickinfo,
+                                   gf_boolean_t construct_real_path,
+                                   char **op_errstr);
 
 int32_t
 glusterd_volinfo_find (char *volname, glusterd_volinfo_t **volinfo);
@@ -127,6 +128,10 @@ int32_t
 glusterd_service_stop(const char *service, char *pidfile, int sig,
                       gf_boolean_t force_kill);
 
+int32_t
+glusterd_service_stop_nolock (const char *service, char *pidfile, int sig,
+                              gf_boolean_t force_kill);
+
 int
 glusterd_get_next_available_brickid (glusterd_volinfo_t *volinfo);
 
@@ -161,7 +166,8 @@ glusterd_is_cli_op_req (int32_t op);
 int32_t
 glusterd_volume_brickinfo_get_by_brick (char *brick,
                                         glusterd_volinfo_t *volinfo,
-                                        glusterd_brickinfo_t **brickinfo);
+                                        glusterd_brickinfo_t **brickinfo,
+                                        gf_boolean_t construct_real_path);
 
 int32_t
 glusterd_add_volumes_to_export_dict (dict_t **peer_data);
@@ -200,12 +206,11 @@ glusterd_add_volume_to_dict (glusterd_volinfo_t *volinfo,
                              char *prefix);
 int
 glusterd_get_brickinfo (xlator_t *this, const char *brickname,
-                        int port, gf_boolean_t localhost,
-                        glusterd_brickinfo_t **brickinfo);
+                        int port, glusterd_brickinfo_t **brickinfo);
 
 void
 glusterd_set_brick_status (glusterd_brickinfo_t  *brickinfo,
-                            gf_brick_status_t status);
+                           gf_brick_status_t status);
 
 gf_boolean_t
 glusterd_is_brick_started (glusterd_brickinfo_t  *brickinfo);
@@ -315,7 +320,9 @@ glusterd_get_local_brickpaths (glusterd_volinfo_t *volinfo,
 int32_t
 glusterd_recreate_bricks (glusterd_conf_t *conf);
 int32_t
-glusterd_handle_upgrade_downgrade (dict_t *options, glusterd_conf_t *conf);
+glusterd_handle_upgrade_downgrade (dict_t *options, glusterd_conf_t *conf,
+                                   gf_boolean_t upgrade,
+                                   gf_boolean_t downgrade);
 
 int
 glusterd_add_brick_detail_to_dict (glusterd_volinfo_t *volinfo,
@@ -371,7 +378,7 @@ glusterd_create_sub_tier_volinfo (glusterd_volinfo_t *volinfo,
                                    glusterd_volinfo_t **dup_volinfo,
                                    gf_boolean_t is_hot_tier,
                                    const char *new_name);
-void
+int
 glusterd_restart_rebalance_for_volume (glusterd_volinfo_t *volinfo);
 
 void
@@ -441,6 +448,11 @@ int32_t
 glusterd_handle_node_rsp (dict_t *req_ctx, void *pending_entry,
                           glusterd_op_t op, dict_t *rsp_dict, dict_t *op_ctx,
                           char **op_errstr, gd_node_type type);
+int
+glusterd_volume_bitrot_scrub_use_rsp_dict (dict_t *aggr, dict_t *rsp_dict);
+
+int
+glusterd_volume_heal_use_rsp_dict (dict_t *aggr, dict_t *rsp_dict);
 
 int32_t
 glusterd_check_if_quota_trans_enabled (glusterd_volinfo_t *volinfo);
@@ -623,6 +635,9 @@ glusterd_check_client_op_version_support (char *volname, uint32_t op_version,
 gf_boolean_t
 glusterd_have_peers ();
 
+gf_boolean_t
+glusterd_have_volumes ();
+
 void
 glusterd_get_rebalance_volfile (glusterd_volinfo_t *volinfo,
                                 char *path, int path_len);
@@ -667,6 +682,9 @@ void
 glusterd_list_add_order (struct cds_list_head *new, struct cds_list_head *head,
                         int (*compare)(struct cds_list_head *,
                                        struct cds_list_head *));
+int
+glusterd_disallow_op_for_tier (glusterd_volinfo_t *volinfo, glusterd_op_t op,
+                               int cmd);
 
 struct rpc_clnt*
 glusterd_defrag_rpc_get (glusterd_defrag_info_t *defrag);
@@ -674,10 +692,27 @@ glusterd_defrag_rpc_get (glusterd_defrag_info_t *defrag);
 struct rpc_clnt*
 glusterd_defrag_rpc_put (glusterd_defrag_info_t *defrag);
 
-int
-glusterd_disallow_op_for_tier (glusterd_volinfo_t *volinfo, glusterd_op_t op,
-                               int cmd);
 int32_t
 glusterd_count_connected_peers (int32_t *count);
 
+int
+glusterd_volume_brick_for_each (glusterd_volinfo_t *volinfo, void *data,
+               int (*fn) (glusterd_volinfo_t *, glusterd_brickinfo_t *,
+                          dict_t *mod_dict, void *));
+
+int
+glusterd_get_dummy_client_filepath (char *filepath,
+                                    glusterd_volinfo_t *volinfo,
+                                    gf_transport_type type);
+
+int
+glusterd_handle_replicate_brick_ops (glusterd_volinfo_t *volinfo,
+                                     glusterd_brickinfo_t *brickinfo,
+                                     glusterd_op_t op);
+void
+assign_brick_groups (glusterd_volinfo_t *volinfo);
+
+glusterd_brickinfo_t*
+get_last_brick_of_brick_group (glusterd_volinfo_t *volinfo,
+                               glusterd_brickinfo_t *brickinfo);
 #endif
