@@ -8,11 +8,6 @@
   cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "glusterfs.h"
 #include "logging.h"
 #include "dict.h"
@@ -31,7 +26,7 @@ ioc_get_priority (ioc_table_t *table, const char *path);
 struct volume_options options[];
 
 
-static inline uint32_t
+static uint32_t
 ioc_hashfn (void *data, int len)
 {
         off_t offset;
@@ -44,7 +39,7 @@ ioc_hashfn (void *data, int len)
 /* TODO: This function is not used, uncomment when we find a
          usage for this function.
 
-static inline ioc_inode_t *
+static ioc_inode_t *
 ioc_inode_reupdate (ioc_inode_t *ioc_inode)
 {
         ioc_table_t *table = NULL;
@@ -58,7 +53,7 @@ ioc_inode_reupdate (ioc_inode_t *ioc_inode)
 }
 
 
-static inline ioc_inode_t *
+static ioc_inode_t *
 ioc_get_inode (dict_t *dict, char *name)
 {
         ioc_inode_t *ioc_inode      = NULL;
@@ -499,7 +494,7 @@ out:
         return ret;
 }
 
-static inline uint32_t
+static uint32_t
 is_match (const char *path, const char *pattern)
 {
         int32_t ret = 0;
@@ -898,29 +893,6 @@ ioc_release (xlator_t *this, fd_t *fd)
         return 0;
 }
 
-/*
- * ioc_readv_disabled_cbk
- * @frame:
- * @cookie:
- * @this:
- * @op_ret:
- * @op_errno:
- * @vector:
- * @count:
- *
- */
-int32_t
-ioc_readv_disabled_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
-                        int32_t op_ret,	int32_t op_errno, struct iovec *vector,
-                        int32_t count, struct iatt *stbuf,
-                        struct iobref *iobref, dict_t *xdata)
-{
-        STACK_UNWIND_STRICT (readv, frame, op_ret, op_errno, vector, count,
-                             stbuf, iobref, xdata);
-        return 0;
-}
-
-
 int32_t
 ioc_need_prune (ioc_table_t *table)
 {
@@ -1133,10 +1105,17 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
         ioc_inode = (ioc_inode_t *)(long)tmp_ioc_inode;
         if (!ioc_inode) {
                 /* caching disabled, go ahead with normal readv */
-                STACK_WIND (frame, ioc_readv_disabled_cbk,
-                            FIRST_CHILD (frame->this),
-                            FIRST_CHILD (frame->this)->fops->readv, fd, size,
-                            offset, flags, xdata);
+                STACK_WIND_TAIL (frame, FIRST_CHILD (frame->this),
+                                 FIRST_CHILD (frame->this)->fops->readv, fd,
+                                 size, offset, flags, xdata);
+                return 0;
+        }
+
+        if (flags & O_DIRECT) {
+                /* disable caching for this fd, if O_DIRECT is used */
+                STACK_WIND_TAIL (frame, FIRST_CHILD (frame->this),
+                                 FIRST_CHILD (frame->this)->fops->readv, fd,
+                                 size, offset, flags, xdata);
                 return 0;
         }
 
@@ -1170,10 +1149,9 @@ ioc_readv (call_frame_t *frame, xlator_t *this, fd_t *fd,
 
         if (!fd_ctx_get (fd, this, NULL)) {
                 /* disable caching for this fd, go ahead with normal readv */
-                STACK_WIND (frame, ioc_readv_disabled_cbk,
-                            FIRST_CHILD (frame->this),
-                            FIRST_CHILD (frame->this)->fops->readv, fd, size,
-                            offset, flags, xdata);
+                STACK_WIND_TAIL (frame, FIRST_CHILD (frame->this),
+                                 FIRST_CHILD (frame->this)->fops->readv, fd,
+                                 size, offset, flags, xdata);
                 return 0;
         }
 
