@@ -8,12 +8,8 @@
   cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "glusterfs.h"
+#include "defaults.h"
 #include "logging.h"
 #include "dict.h"
 #include "xlator.h"
@@ -36,6 +32,7 @@ struct mdc_conf {
 	gf_boolean_t cache_posix_acl;
 	gf_boolean_t cache_selinux;
 	gf_boolean_t force_readdirp;
+        gf_boolean_t cache_swift_metadata;
 };
 
 
@@ -69,6 +66,11 @@ static struct mdc_key {
 		.load = 0,
 		.check = 1,
 	},
+        {
+                .name = "user.swift.metadata",
+                .load = 0,
+                .check = 1,
+        },
 	{
 		.name = "security.capability",
 		.load = 0,
@@ -737,6 +739,50 @@ mdc_load_reqs (xlator_t *this, dict_t *dict)
 }
 
 
+static char*
+mdc_serialize_loaded_key_names (xlator_t *this)
+{
+        int             max_len = 0;
+        int             len = 0;
+        int             i = 0;
+        char           *mdc_key_names = NULL;
+        const char     *mdc_key = NULL;
+        gf_boolean_t    at_least_one_key_loaded = _gf_false;
+
+        for (mdc_key = mdc_keys[i].name; (mdc_key = mdc_keys[i].name); i++) {
+                max_len += (strlen(mdc_keys[i].name) + 1);
+                if (mdc_keys[i].load)
+                        at_least_one_key_loaded = _gf_true;
+        }
+
+        if (!at_least_one_key_loaded)
+                goto out;
+
+        mdc_key_names = GF_CALLOC (1, max_len + 1, gf_common_mt_char);
+        if (!mdc_key_names)
+                goto out;
+
+        i = 0;
+        for (mdc_key = mdc_keys[i].name; (mdc_key = mdc_keys[i].name); i++) {
+                if (!mdc_keys[i].load)
+                        continue;
+                strcat (mdc_key_names, mdc_keys[i].name);
+                strcat (mdc_key_names, " ");
+        }
+
+        len = strlen (mdc_key_names);
+        if (len > 0) {
+                mdc_key_names[len - 1] = '\0';
+        } else {
+                GF_FREE (mdc_key_names);
+                mdc_key_names = NULL;
+        }
+
+out:
+        return mdc_key_names;
+}
+
+
 struct checkpair {
 	int  ret;
 	dict_t *rsp;
@@ -1460,6 +1506,53 @@ mdc_create (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
 }
 
 
+static int
+mdc_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
+                int32_t op_ret, int32_t op_errno, fd_t *fd,
+                dict_t *xdata)
+{
+        mdc_local_t *local = NULL;
+
+        local = frame->local;
+
+        if (op_ret || !local)
+                goto out;
+
+        if (local->fd->flags & O_TRUNC) {
+                /* O_TRUNC modifies file size. Hence invalidate the
+                 * cache entry to fetch latest attributes. */
+                mdc_inode_iatt_invalidate (this, local->fd->inode);
+        }
+
+out:
+        MDC_STACK_UNWIND (open, frame, op_ret, op_errno, fd, xdata);
+        return 0;
+}
+
+
+static int
+mdc_open (call_frame_t *frame, xlator_t *this, loc_t *loc, int flags,
+          fd_t *fd, dict_t *xdata)
+{
+        mdc_local_t  *local = NULL;
+
+        if (!fd || !IA_ISREG(fd->inode->ia_type) ||
+            !(fd->flags & O_TRUNC)) {
+                goto out;
+        }
+
+        local = mdc_local_get (frame);
+
+        local->fd = fd_ref (fd);
+
+out:
+        STACK_WIND (frame, mdc_open_cbk,
+                    FIRST_CHILD(this), FIRST_CHILD(this)->fops->open,
+                    loc, flags, fd, xdata);
+        return 0;
+}
+
+
 int
 mdc_readv_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                int32_t op_ret, int32_t op_errno,
@@ -1973,11 +2066,46 @@ mdc_fremovexattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
 }
 
 
+int
+mdc_opendir(call_frame_t *frame, xlator_t *this, loc_t *loc,
+            fd_t *fd, dict_t *xdata)
+{
+        int          ret = -1;
+        char        *mdc_key_names = NULL;
+        dict_t      *xattr_alloc = NULL;
+
+        if (!xdata)
+                xdata = xattr_alloc = dict_new ();
+
+        if (xdata) {
+                /* Tell readdir-ahead to include these keys in xdata when it
+                 * internally issues readdirp() in it's opendir_cbk */
+                mdc_key_names = mdc_serialize_loaded_key_names(this);
+                if (!mdc_key_names)
+                        goto wind;
+                ret = dict_set_dynstr (xdata, GF_MDC_LOADED_KEY_NAMES,
+                                       mdc_key_names);
+                if (ret)
+                        goto wind;
+        }
+
+wind:
+        STACK_WIND (frame, default_opendir_cbk, FIRST_CHILD(this),
+                    FIRST_CHILD(this)->fops->opendir, loc, fd, xdata);
+
+        if (xattr_alloc)
+                dict_unref (xattr_alloc);
+
+        return 0;
+}
+
+
 int
 mdc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
 		  int op_ret, int op_errno, gf_dirent_t *entries, dict_t *xdata)
 {
-        gf_dirent_t *entry      = NULL;
+        gf_dirent_t     *entry      = NULL;
+        struct md_cache *mdc        = NULL;
 
 	if (op_ret <= 0)
 		goto unwind;
@@ -1985,6 +2113,8 @@ mdc_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
         list_for_each_entry (entry, &entries->list, list) {
                 if (!entry->inode)
 			continue;
+                if (mdc_inode_ctx_get (this, entry->inode, &mdc) != 0)
+                        continue;
                 mdc_inode_iatt_set (this, entry->inode, &entry->d_stat);
                 mdc_inode_xatt_set (this, entry->inode, entry->dict);
         }
@@ -2226,6 +2356,12 @@ reconfigure (xlator_t *this, dict_t *options)
 	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
 	mdc_key_load_set (mdc_keys, "glusterfs.posix_acl.", conf->cache_posix_acl);
 
+        GF_OPTION_RECONF ("cache-swift-metadata", conf->cache_swift_metadata,
+                          options, bool, out);
+        mdc_key_load_set (mdc_keys, "user.swift.metadata",
+                          conf->cache_swift_metadata);
+
+
 	GF_OPTION_RECONF("force-readdirp", conf->force_readdirp, options, bool, out);
 
 out:
@@ -2262,6 +2398,11 @@ init (xlator_t *this)
 	mdc_key_load_set (mdc_keys, "system.posix_acl_", conf->cache_posix_acl);
 	mdc_key_load_set (mdc_keys, "glusterfs.posix_acl.", conf->cache_posix_acl);
 
+        GF_OPTION_INIT ("cache-swift-metadata",
+                        conf->cache_swift_metadata, bool, out);
+        mdc_key_load_set (mdc_keys, "user.swift.metadata",
+                          conf->cache_swift_metadata);
+
 	GF_OPTION_INIT("force-readdirp", conf->force_readdirp, bool, out);
 out:
 	this->private = conf;
@@ -2291,6 +2432,7 @@ struct xlator_fops fops = {
         .rename      = mdc_rename,
         .link        = mdc_link,
         .create      = mdc_create,
+        .open        = mdc_open,
         .readv       = mdc_readv,
         .writev      = mdc_writev,
         .setattr     = mdc_setattr,
@@ -2302,6 +2444,7 @@ struct xlator_fops fops = {
         .fgetxattr   = mdc_fgetxattr,
 	.removexattr = mdc_removexattr,
 	.fremovexattr= mdc_fremovexattr,
+        .opendir     = mdc_opendir,
 	.readdirp    = mdc_readdirp,
 	.readdir     = mdc_readdir,
 	.fallocate   = mdc_fallocate,
@@ -2319,6 +2462,11 @@ struct volume_options options[] = {
 	  .type = GF_OPTION_TYPE_BOOL,
 	  .default_value = "false",
 	},
+        { .key = {"cache-swift-metadata"},
+          .type = GF_OPTION_TYPE_BOOL,
+          .default_value = "true",
+          .description = "Cache swift metadata (user.swift.metadata xattr)",
+        },
 	{ .key = {"cache-posix-acl"},
 	  .type = GF_OPTION_TYPE_BOOL,
 	  .default_value = "false",
