@@ -9,11 +9,6 @@
 */
 
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include "client.h"
 #include "xlator.h"
 #include "defaults.h"
@@ -36,6 +31,13 @@ int client_init_rpc (xlator_t *this);
 int client_destroy_rpc (xlator_t *this);
 int client_mark_fd_bad (xlator_t *this);
 
+static void
+client_filter_o_direct (clnt_conf_t *conf, int32_t *flags)
+{
+        if (conf->filter_o_direct)
+                *flags = (*flags & ~O_DIRECT);
+}
+
 static int
 client_fini_complete (xlator_t *this)
 {
@@ -55,7 +57,7 @@ out:
         return 0;
 }
 
-static int
+int
 client_notify_dispatch_uniq (xlator_t *this, int32_t event, void *data, ...)
 {
         clnt_conf_t     *conf = this->private;
@@ -182,10 +184,14 @@ int32_t
 client_register_grace_timer (xlator_t *this, clnt_conf_t *conf)
 {
         int32_t  ret = -1;
+        struct timespec grace_ts = {0, };
 
         GF_VALIDATE_OR_GOTO ("client", this, out);
         GF_VALIDATE_OR_GOTO (this->name, conf, out);
 
+        grace_ts.tv_sec = conf->grace_timeout;
+        grace_ts.tv_nsec = 0;
+
         pthread_mutex_lock (&conf->lock);
         {
                 if (conf->grace_timer || !conf->grace_timer_needed) {
@@ -201,7 +207,7 @@ client_register_grace_timer (xlator_t *this, clnt_conf_t *conf)
 
                         conf->grace_timer =
                                 gf_timer_call_after (this->ctx,
-                                                     conf->grace_ts,
+                                                     grace_ts,
                                                      client_grace_timeout,
                                                      conf->rpc);
                 }
@@ -229,8 +235,6 @@ client_submit_request (xlator_t *this, void *req, call_frame_t *frame,
         struct iobref  *new_iobref = NULL;
         ssize_t         xdr_size   = 0;
         struct rpc_req  rpcreq     = {0, };
-        uint64_t        ngroups    = 0;
-        uint64_t        gid        = 0;
 
         GF_VALIDATE_OR_GOTO ("client", this, out);
         GF_VALIDATE_OR_GOTO (this->name, prog, out);
@@ -301,14 +305,11 @@ client_submit_request (xlator_t *this, void *req, call_frame_t *frame,
 
         /* do not send all groups if they are resolved server-side */
         if (!conf->send_gids) {
-                /* copy some values for restoring later */
-                ngroups = frame->root->ngrps;
-                frame->root->ngrps = 1;
-                if (ngroups <= SMALL_GROUP_COUNT) {
-                        gid = frame->root->groups_small[0];
+                if (frame->root->ngrps <= SMALL_GROUP_COUNT) {
                         frame->root->groups_small[0] = frame->root->gid;
                         frame->root->groups = frame->root->groups_small;
                 }
+                frame->root->ngrps = 1;
         }
 
         /* Send the msg */
@@ -320,13 +321,6 @@ client_submit_request (xlator_t *this, void *req, call_frame_t *frame,
                 gf_msg_debug (this->name, 0, "rpc_clnt_submit failed");
         }
 
-        if (!conf->send_gids) {
-                /* restore previous values */
-                frame->root->ngrps = ngroups;
-                if (ngroups <= SMALL_GROUP_COUNT)
-                        frame->root->groups_small[0] = gid;
-        }
-
         ret = 0;
 
         if (new_iobref)
@@ -352,6 +346,123 @@ out:
 }
 
 
+int
+client_submit_compound_request (xlator_t *this, void *req, call_frame_t *frame,
+                       rpc_clnt_prog_t *prog, int procnum, fop_cbk_fn_t cbkfn,
+                       struct iovec *req_payload, int req_count,
+                       struct iobref *iobref,  struct iovec *rsphdr,
+                       int rsphdr_count, struct iovec *rsp_payload,
+                       int rsp_payload_count, struct iobref *rsp_iobref,
+                       xdrproc_t xdrproc)
+{
+        int             ret        = -1;
+        clnt_conf_t    *conf       = NULL;
+        struct iovec    iov        = {0, };
+        struct iobuf   *iobuf      = NULL;
+        int             count      = 0;
+        struct iobref  *new_iobref = NULL;
+        ssize_t         xdr_size   = 0;
+        struct rpc_req  rpcreq     = {0, };
+
+        GF_VALIDATE_OR_GOTO ("client", this, out);
+        GF_VALIDATE_OR_GOTO (this->name, prog, out);
+        GF_VALIDATE_OR_GOTO (this->name, frame, out);
+
+        conf = this->private;
+
+        /* If 'setvolume' is not successful, we should not send frames to
+         * server
+         */
+
+        if (!conf->connected) {
+                gf_msg_debug (this->name, 0,
+                              "connection in disconnected state");
+                goto out;
+        }
+
+        if (req && xdrproc) {
+                xdr_size = xdr_sizeof (xdrproc, req);
+                iobuf = iobuf_get2 (this->ctx->iobuf_pool, xdr_size);
+                if (!iobuf) {
+                        goto out;
+                };
+
+                new_iobref = iobref_new ();
+                if (!new_iobref) {
+                        goto out;
+                }
+
+                if (iobref != NULL) {
+                        ret = iobref_merge (new_iobref, iobref);
+                        if (ret != 0) {
+                                goto out;
+                        }
+                }
+
+                ret = iobref_add (new_iobref, iobuf);
+                if (ret != 0) {
+                        goto out;
+                }
+
+                iov.iov_base = iobuf->ptr;
+                iov.iov_len  = iobuf_size (iobuf);
+
+                /* Create the xdr payload */
+                ret = xdr_serialize_generic (iov, req, xdrproc);
+                if (ret == -1) {
+                        /* callingfn so that, we can get to know which xdr
+                           function was called */
+                        gf_log_callingfn (this->name, GF_LOG_WARNING,
+                                          "XDR payload creation failed");
+                        goto out;
+                }
+                iov.iov_len = ret;
+                count = 1;
+        }
+
+        /* do not send all groups if they are resolved server-side */
+        if (!conf->send_gids) {
+                if (frame->root->ngrps <= SMALL_GROUP_COUNT) {
+                        frame->root->groups_small[0] = frame->root->gid;
+                        frame->root->groups = frame->root->groups_small;
+                }
+                frame->root->ngrps = 1;
+        }
+
+        /* Send the msg */
+        ret = rpc_clnt_submit (conf->rpc, prog, procnum, cbkfn, &iov, count,
+                               req_payload, req_count, new_iobref, frame,
+                               rsphdr, rsphdr_count,
+                               rsp_payload, rsp_payload_count, rsp_iobref);
+
+        if (ret < 0) {
+                gf_msg_debug (this->name, 0, "rpc_clnt_submit failed");
+        }
+
+        ret = 0;
+
+        if (new_iobref)
+                iobref_unref (new_iobref);
+
+        if (iobuf)
+                iobuf_unref (iobuf);
+
+        return ret;
+
+out:
+        rpcreq.rpc_status = -1;
+
+        cbkfn (&rpcreq, NULL, 0, frame);
+
+        if (new_iobref)
+                iobref_unref (new_iobref);
+
+        if (iobuf)
+                iobuf_unref (iobuf);
+
+        return 0;
+}
+
 int32_t
 client_forget (xlator_t *this, inode_t *inode)
 {
@@ -809,11 +920,9 @@ client_create (call_frame_t *frame, xlator_t *this, loc_t *loc,  int32_t flags,
         args.fd = fd;
         args.umask = umask;
         args.xdata = xdata;
+        args.flags = flags;
 
-        if (!conf->filter_o_direct)
-                args.flags = flags;
-        else
-                args.flags = (flags & ~O_DIRECT);
+        client_filter_o_direct (conf, &args.flags);
 
         proc = &conf->fops->proctable[GF_FOP_CREATE];
         if (proc->fn)
@@ -844,11 +953,9 @@ client_open (call_frame_t *frame, xlator_t *this, loc_t *loc,
         args.loc = loc;
         args.fd = fd;
         args.xdata = xdata;
+        args.flags = flags;
 
-        if (!conf->filter_o_direct)
-                args.flags = flags;
-        else
-                args.flags = (flags & ~O_DIRECT);
+        client_filter_o_direct (conf, &args.flags);
 
         proc = &conf->fops->proctable[GF_FOP_OPEN];
         if (proc->fn)
@@ -882,6 +989,8 @@ client_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
         args.flags  = flags;
         args.xdata = xdata;
 
+        client_filter_o_direct (conf, &args.flags);
+
         proc = &conf->fops->proctable[GF_FOP_READ];
         if (proc->fn)
                 ret = proc->fn (frame, this, &args);
@@ -920,6 +1029,8 @@ client_writev (call_frame_t *frame, xlator_t *this, fd_t *fd,
         args.iobref = iobref;
         args.xdata = xdata;
 
+        client_filter_o_direct (conf, &args.flags);
+
         proc = &conf->fops->proctable[GF_FOP_WRITE];
         if (proc->fn)
                 ret = proc->fn (frame, this, &args);
@@ -1490,6 +1601,34 @@ out:
 	return 0;
 }
 
+int32_t
+client_lease (call_frame_t *frame, xlator_t *this, loc_t *loc,
+              struct gf_lease *lease, dict_t *xdata)
+{
+        int                   ret  = -1;
+        clnt_conf_t          *conf = NULL;
+        rpc_clnt_procedure_t *proc = NULL;
+        clnt_args_t           args = {0,};
+
+        conf = this->private;
+        if (!conf || !conf->fops)
+                goto out;
+
+        args.loc = loc;
+        args.lease = lease;
+        args.xdata = xdata;
+
+        proc = &conf->fops->proctable[GF_FOP_LEASE];
+        if (proc->fn)
+                ret = proc->fn (frame, this, &args);
+out:
+        if (ret)
+                STACK_UNWIND_STRICT (lk, frame, -1, ENOTCONN, NULL, NULL);
+
+        return 0;
+}
+
+
 int32_t
 client_lk (call_frame_t *frame, xlator_t *this, fd_t *fd, int32_t cmd,
            struct gf_flock *lock, dict_t *xdata)
@@ -1906,6 +2045,90 @@ out:
 }
 
 
+int32_t
+client_seek (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
+             gf_seek_what_t what, dict_t *xdata)
+{
+        int          ret              = -1;
+        clnt_conf_t *conf             = NULL;
+        rpc_clnt_procedure_t *proc    = NULL;
+        clnt_args_t  args             = {0,};
+
+        conf = this->private;
+        if (!conf || !conf->fops)
+                goto out;
+
+        args.fd = fd;
+        args.offset = offset;
+        args.what = what;
+        args.xdata = xdata;
+
+        proc = &conf->fops->proctable[GF_FOP_SEEK];
+        if (proc->fn)
+                ret = proc->fn (frame, this, &args);
+out:
+        if (ret)
+                STACK_UNWIND_STRICT(seek, frame, -1, ENOTCONN, 0, NULL);
+
+        return 0;
+}
+
+int32_t
+client_getactivelk (call_frame_t *frame, xlator_t *this, loc_t *loc,
+                     dict_t *xdata)
+{
+        int          ret                = -1;
+        clnt_conf_t *conf               = NULL;
+        rpc_clnt_procedure_t *proc      = NULL;
+        clnt_args_t  args               = {0,};
+
+        conf = this->private;
+        if (!conf || !conf->fops)
+                goto out;
+
+
+        args.loc = loc;
+        args.xdata = xdata;
+
+        proc = &conf->fops->proctable[GF_FOP_GETACTIVELK];
+        if (proc->fn)
+                ret = proc->fn (frame, this, &args);
+out:
+        if (ret)
+                STACK_UNWIND_STRICT (getactivelk, frame, -1, ENOTCONN, NULL,
+                                     NULL);
+
+        return 0;
+}
+
+int32_t
+client_setactivelk (call_frame_t *frame, xlator_t *this, loc_t *loc,
+                      lock_migration_info_t *locklist, dict_t *xdata)
+{
+        int          ret  = -1;
+        clnt_conf_t *conf = NULL;
+        rpc_clnt_procedure_t *proc = NULL;
+        clnt_args_t  args = {0,};
+
+        conf = this->private;
+        if (!conf || !conf->fops)
+                goto out;
+
+
+        args.loc = loc;
+        args.xdata = xdata;
+        args.locklist = locklist;
+
+        proc = &conf->fops->proctable[GF_FOP_SETACTIVELK];
+        if (proc->fn)
+                ret = proc->fn (frame, this, &args);
+out:
+        if (ret)
+                STACK_UNWIND_STRICT (setactivelk, frame, -1, ENOTCONN, NULL);
+
+        return 0;
+}
+
 int32_t
 client_getspec (call_frame_t *frame, xlator_t *this, const char *key,
                 int32_t flags)
@@ -1936,6 +2159,32 @@ out:
 }
 
 
+int32_t
+client_compound (call_frame_t *frame, xlator_t *this,
+                 void *data, dict_t *xdata)
+{
+        int          ret  = -1;
+        clnt_conf_t *conf = NULL;
+        compound_args_t *args = data;
+        rpc_clnt_procedure_t *proc = NULL;
+
+        conf = this->private;
+        if (!conf || !conf->fops)
+                goto out;
+
+        args->xdata = xdata;
+
+        proc = &conf->fops->proctable[GF_FOP_COMPOUND];
+        if (proc->fn)
+                ret = proc->fn (frame, this, args);
+out:
+        if (ret)
+                STACK_UNWIND_STRICT (compound, frame, -1, ENOTCONN,
+                                     NULL, NULL);
+
+	return 0;
+}
+
 int
 client_mark_fd_bad (xlator_t *this)
 {
@@ -1982,27 +2231,14 @@ client_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
         {
                 conf->connected = 1;
                 // connect happened, send 'get_supported_versions' mop
-                ret = dict_get_str (this->options, "disable-handshake",
-                                    &handshake);
 
                 gf_msg_debug (this->name, 0, "got RPC_CLNT_CONNECT");
 
-                if ((ret < 0) || (strcasecmp (handshake, "on"))) {
-                        ret = client_handshake (this, rpc);
-                        if (ret)
-                                gf_msg (this->name, GF_LOG_WARNING, 0,
-                                        PC_MSG_HANDSHAKE_RETURN, "handshake "
-                                        "msg returned %d", ret);
-                } else {
-                        //conf->rpc->connected = 1;
-                        ret = client_notify_dispatch_uniq (this,
-                                                           GF_EVENT_CHILD_UP,
-                                                           NULL);
-                        if (ret)
-                                gf_msg (this->name, GF_LOG_INFO, 0,
-                                        PC_MSG_CHILD_UP_NOTIFY_FAILED,
-                                        "CHILD_UP notify failed");
-                }
+                ret = client_handshake (this, rpc);
+                if (ret)
+                        gf_msg (this->name, GF_LOG_WARNING, 0,
+                                PC_MSG_HANDSHAKE_RETURN, "handshake "
+                                "msg returned %d", ret);
 
                 /* Cancel grace timer if set */
                 pthread_mutex_lock (&conf->lock);
@@ -2025,6 +2261,8 @@ client_rpc_notify (struct rpc_clnt *rpc, void *mydata, rpc_clnt_event_t event,
                 break;
         }
         case RPC_CLNT_DISCONNECT:
+                gf_msg_debug (this->name, 0, "got RPC_CLNT_DISCONNECT");
+
                 if (!conf->lk_heal)
                         client_mark_fd_bad (this);
                 else
@@ -2323,36 +2561,22 @@ int
 client_init_grace_timer (xlator_t *this, dict_t *options,
                          clnt_conf_t *conf)
 {
-        char      timestr[64]    = {0,};
-        char     *lk_heal        = NULL;
         int32_t   ret            = -1;
-        int32_t   grace_timeout  = -1;
 
         GF_VALIDATE_OR_GOTO ("client", this, out);
         GF_VALIDATE_OR_GOTO (this->name, options, out);
         GF_VALIDATE_OR_GOTO (this->name, conf, out);
 
-        conf->lk_heal = _gf_false;
-
-        ret = dict_get_str (options, "lk-heal", &lk_heal);
-        if (!ret)
-                gf_string2boolean (lk_heal, &conf->lk_heal);
+        GF_OPTION_RECONF ("lk-heal", conf->lk_heal, options, bool, out);
 
         gf_msg_debug (this->name, 0, "lk-heal = %s",
                       (conf->lk_heal) ? "on" : "off");
 
-        ret = dict_get_int32 (options, "grace-timeout", &grace_timeout);
-        if (!ret)
-                conf->grace_ts.tv_sec = grace_timeout;
-        else
-                conf->grace_ts.tv_sec = 10;
+        GF_OPTION_RECONF ("grace-timeout", conf->grace_timeout,
+                                                options, uint32, out);
 
-        conf->grace_ts.tv_nsec  = 0;
-
-        gf_time_fmt (timestr, sizeof timestr, conf->grace_ts.tv_sec,
-                     gf_timefmt_s);
-        gf_msg_debug (this->name, 0, "Client grace timeout value = %s",
-                      timestr);
+        gf_msg_debug (this->name, 0, "Client grace timeout value = %d",
+                                conf->grace_timeout);
 
         ret = 0;
 out:
@@ -2476,6 +2700,8 @@ init (xlator_t *this)
         pthread_mutex_init (&conf->lock, NULL);
         INIT_LIST_HEAD (&conf->saved_fds);
 
+        conf->child_up = _gf_false;
+
         /* Initialize parameters for lock self healing*/
         conf->lk_version         = 1;
         conf->grace_timer        = NULL;
@@ -2726,6 +2952,11 @@ struct xlator_fops fops = {
         .zerofill    = client_zerofill,
         .getspec     = client_getspec,
         .ipc         = client_ipc,
+        .seek        = client_seek,
+        .lease       = client_lease,
+        .compound    = client_compound,
+        .getactivelk = client_getactivelk,
+        .setactivelk = client_setactivelk,
 };
 
 
