@@ -17,10 +17,6 @@
   cases as published by the Free Software Foundation.
 */
 
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
 #include <lvm2app.h>
 #include <openssl/md5.h>
 #include <time.h>
@@ -124,7 +120,7 @@ bd_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this, int op_ret,
         int           ret    = -1;
         bd_attr_t    *bdatt  = NULL;
         uint64_t      size   = 0;
-        char         *type   = BD_TYPE_NONE;
+        char         *type   = NULL;
 
         /* only regular files are part of BD object */
         if (op_ret < 0 || buf->ia_type != IA_IFREG)
@@ -489,7 +485,7 @@ bd_readv (call_frame_t *frame, xlator_t *this, fd_t *fd, size_t size,
                 goto out;
         }
         _fd = bd_fd->fd;
-        op_ret = pread (_fd, iobuf->ptr, size, offset);
+        op_ret = sys_pread (_fd, iobuf->ptr, size, offset);
         if (op_ret == -1) {
                 op_errno = errno;
                 gf_log (this->name, GF_LOG_ERROR,
@@ -618,7 +614,7 @@ bd_open_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         "bd_fd is NULL from fd=%p", fd);
                 goto out;
         }
-        close (bd_fd->fd);
+        sys_close (bd_fd->fd);
         GF_FREE (bd_fd);
 
 out:
@@ -694,7 +690,7 @@ out:
         GF_FREE (devpath);
         if (ret) {
                 if (_fd >= 0)
-                        close (_fd);
+                        sys_close (_fd);
                 GF_FREE (bd_fd);
         }
 
@@ -896,7 +892,7 @@ bd_release (xlator_t *this, fd_t *fd)
         }
         bd_fd = (bd_fd_t *)(long)tmp_bfd;
 
-        close (bd_fd->fd);
+        sys_close (bd_fd->fd);
         GF_FREE (bd_fd);
 out:
         return 0;
@@ -1791,7 +1787,7 @@ __bd_pwritev (int fd, struct iovec *vector, int count, off_t offset,
         if (!vector)
                 return -EFAULT;
 
-        retval = pwritev (fd, vector, count, offset);
+        retval = sys_pwritev (fd, vector, count, offset);
         if (retval == -1) {
                 int64_t off = offset;
                 gf_log (THIS->name, GF_LOG_WARNING,
@@ -1814,8 +1810,8 @@ __bd_pwritev (int fd, struct iovec *vector, int count, off_t offset,
                         vector[index].iov_len = bd_size - internal_offset;
                         no_space = 1;
                 }
-                retval = pwritev (fd, vector[index].iov_base,
-                                vector[index].iov_len, internal_offset);
+                retval = sys_pwritev (fd, vector[index].iov_base,
+                                      vector[index].iov_len, internal_offset);
                 if (retval == -1) {
                         gf_log (THIS->name, GF_LOG_WARNING,
                                 "base %p, length %ld, offset %ld, message %s",
@@ -2178,7 +2174,7 @@ bd_rchecksum (call_frame_t *frame, xlator_t *this, fd_t *fd, off_t offset,
 
         LOCK (&fd->lock);
         {
-                ret = pread (_fd, buf, len, offset);
+                ret = sys_pread (_fd, buf, len, offset);
                 if (ret < 0) {
                         gf_log (this->name, GF_LOG_WARNING,
                                 "pread of %d bytes returned %d (%s)",
