@@ -7,11 +7,6 @@
    later), or the GNU General Public License, version 2 (GPLv2), in all
    cases as published by the Free Software Foundation.
 */
-#ifndef _CONFIG_H
-#define _CONFIG_H
-#include "config.h"
-#endif
-
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -21,6 +16,8 @@
 #include <alloca.h>
 #endif
 
+#include "common-utils.h"
+
 #include "posix-handle.h"
 #include "posix.h"
 #include "xlator.h"
@@ -33,24 +30,42 @@ inode_t *
 posix_resolve (xlator_t *this, inode_table_t *itable, inode_t *parent,
                char *bname, struct iatt *iabuf)
 {
-        inode_t     *inode = NULL, *linked_inode = NULL;
+        inode_t     *inode = NULL;
         int          ret   = -1;
 
         ret = posix_istat (this, parent->gfid, bname, iabuf);
-        if (ret < 0)
+        if (ret < 0) {
+                gf_log (this->name, GF_LOG_WARNING, "gfid: %s, bname: %s "
+                        "failed", uuid_utoa (parent->gfid), bname);
                 goto out;
-
-        inode = inode_find (itable, iabuf->ia_gfid);
-        if (inode == NULL) {
-                inode = inode_new (itable);
         }
 
-        linked_inode = inode_link (inode, parent, bname, iabuf);
+        if (__is_root_gfid (iabuf->ia_gfid) && !strcmp (bname, "/")) {
+                inode = itable->root;
+        } else {
+                inode = inode_find (itable, iabuf->ia_gfid);
+                if (inode == NULL) {
+                        inode = inode_new (itable);
+                        gf_uuid_copy (inode->gfid, iabuf->ia_gfid);
+                }
+        }
 
-        inode_unref (inode);
+        /* Linking an inode here, can cause a race in posix_acl.
+           Parent inode gets linked here, but before
+           it reaches posix_acl_readdirp_cbk, create/lookup can
+           come on a leaf-inode, as parent-inode-ctx not yet updated
+           in posix_acl_readdirp_cbk, create and lookup can fail
+           with EACCESS. So do the inode linking in the quota xlator
+
+        if (__is_root_gfid (iabuf->ia_gfid) && !strcmp (bname, "/"))
+                linked_inode = itable->root;
+        else
+                linked_inode = inode_link (inode, parent, bname, iabuf);
+
+        inode_unref (inode);*/
 
 out:
-        return linked_inode;
+        return inode;
 }
 
 int
@@ -70,6 +85,8 @@ posix_make_ancestral_node (const char *priv_base_path, char *path, int pathsize,
         }
 
         strcat (path, dir_name);
+        if (*dir_name != '/')
+                strcat (path, "/");
 
         if (type & POSIX_ANCESTRY_DENTRY) {
                 entry = gf_dirent_for_name (dir_name);
@@ -130,11 +147,16 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
                         *parent = inode_ref (itable->root);
                 }
 
-                inode = itable->root;
 
-                memset (&iabuf, 0, sizeof (iabuf));
-                gf_uuid_copy (iabuf.ia_gfid, inode->gfid);
-                iabuf.ia_type = inode->ia_type;
+                inode = posix_resolve (this, itable, *parent, "/", &iabuf);
+                if (!inode) {
+                        gf_msg (this->name, GF_LOG_ERROR,
+                                P_MSG_INODE_RESOLVE_FAILED, 0,
+                                "posix resolve on the root inode %s failed",
+                                uuid_utoa (gfid));
+                        *op_errno = ESTALE;
+                        goto out;
+                }
 
                 ret = posix_make_ancestral_node (priv_base_path, path, pathsize,
                                                  head, "/", &iabuf, inode, type,
@@ -150,7 +172,7 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
                   priv_base_path, GF_HIDDEN_PATH, gfid[0], gfid[1],
                   uuid_utoa (gfid));
 
-        len = readlink (dir_handle, linkname, PATH_MAX);
+        len = sys_readlink (dir_handle, linkname, PATH_MAX);
         if (len < 0) {
                 gf_msg (this->name, (errno == ENOENT || errno == ESTALE)
                         ? GF_LOG_DEBUG:GF_LOG_ERROR, errno,
@@ -180,12 +202,14 @@ posix_make_ancestryfromgfid (xlator_t *this, char *path, int pathsize,
 
         inode = posix_resolve (this, itable, *parent, dir_name, &iabuf);
         if (inode == NULL) {
+                gf_msg (this->name, GF_LOG_ERROR, P_MSG_INODE_RESOLVE_FAILED,
+                        0, "posix resolve on the root inode %s failed",
+                        uuid_utoa (gfid));
                 *op_errno = ESTALE;
                 ret = -1;
                 goto out;
         }
 
-        strcat (dir_name, "/");
         ret = posix_make_ancestral_node (priv_base_path, path, pathsize, head,
                                          dir_name, &iabuf, inode, type, xdata);
         if (*parent != NULL) {
@@ -237,6 +261,42 @@ posix_handle_relpath (xlator_t *this, uuid_t gfid, const char *basename,
 /*
   TODO: explain how this pump fixes ELOOP
 */
+gf_boolean_t
+posix_is_malformed_link (xlator_t *this, char *base_str, char *linkname,
+                         size_t len)
+{
+        if ((len == 8) && strcmp (linkname, "../../..")) /*for root*/
+                goto err;
+
+        if (len < 50 || len >= 512)
+                goto err;
+
+        if (memcmp (linkname, "../../", 6) != 0)
+                goto err;
+
+        if ((linkname[2] != '/') ||
+            (linkname[5] != '/') ||
+            (linkname[8] != '/') ||
+            (linkname[11] != '/') ||
+            (linkname[48] != '/')) {
+                goto err;
+        }
+
+        if ((linkname[20] != '-') ||
+            (linkname[25] != '-') ||
+            (linkname[30] != '-') ||
+            (linkname[35] != '-')) {
+                goto err;
+        }
+
+        return _gf_false;
+
+err:
+        gf_log_callingfn (this->name, GF_LOG_ERROR, "malformed internal link "
+                          "%s for %s", linkname, base_str);
+        return _gf_true;
+}
+
 int
 posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
                    char *base_str, int base_len, int pfx_len)
@@ -247,7 +307,7 @@ posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
         int        link_len = 0;
 
         /* is a directory's symlink-handle */
-        ret = readlink (base_str, linkname, 512);
+        ret = sys_readlink (base_str, linkname, 512);
         if (ret == -1) {
                 gf_msg (this->name, GF_LOG_ERROR, errno, P_MSG_READLINK_FAILED,
                         "internal readlink failed on %s ",
@@ -267,40 +327,8 @@ posix_handle_pump (xlator_t *this, char *buf, int len, int maxlen,
                 goto out;
         }
 
-        if (ret < 50 || ret >= 512) {
-                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_LINK_FAILED,
-                        "malformed internal link %s for %s",
-                        linkname, base_str);
+        if (posix_is_malformed_link (this, base_str, linkname, ret))
                 goto err;
-        }
-
-        if (memcmp (linkname, "../../", 6) != 0) {
-                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
-                        "malformed internal link %s for %s",
-                        linkname, base_str);
-                goto err;
-        }
-
-        if ((linkname[2] != '/') ||
-            (linkname[5] != '/') ||
-            (linkname[8] != '/') ||
-            (linkname[11] != '/') ||
-            (linkname[48] != '/')) {
-                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
-                        "malformed internal link %s for %s",
-                        linkname, base_str);
-                goto err;
-        }
-
-        if ((linkname[20] != '-') ||
-            (linkname[25] != '-') ||
-            (linkname[30] != '-') ||
-            (linkname[35] != '-')) {
-                gf_msg (this->name, GF_LOG_ERROR, 0, P_MSG_HANDLEPATH_FAILED,
-                        "malformed internal link %s for %s",
-                        linkname, base_str);
-                goto err;
-        }
 
         blen = link_len - 48;
 
@@ -374,7 +402,7 @@ posix_handle_path (xlator_t *this, uuid_t gfid, const char *basename,
                 len = snprintf (buf, maxlen, "%s", base_str);
         }
 
-        ret = lstat (base_str, &stat);
+        ret = sys_lstat (base_str, &stat);
 
         if (!(ret == 0 && S_ISLNK(stat.st_mode) && stat.st_nlink == 1))
                 goto out;
@@ -388,7 +416,7 @@ posix_handle_path (xlator_t *this, uuid_t gfid, const char *basename,
                 if (ret == -1)
                         break;
 
-                ret = lstat (buf, &stat);
+                ret = sys_lstat (buf, &stat);
         } while ((ret == -1) && errno == ELOOP);
 
 out:
@@ -448,7 +476,6 @@ out:
         return len;
 }
 
-
 int
 posix_handle_init (xlator_t *this)
 {
@@ -463,7 +490,7 @@ posix_handle_init (xlator_t *this)
 
         priv = this->private;
 
-        ret = stat (priv->base_path, &exportbuf);
+        ret = sys_stat (priv->base_path, &exportbuf);
         if (ret || !S_ISDIR (exportbuf.st_mode)) {
                 gf_msg (this->name, GF_LOG_ERROR, 0,
                         P_MSG_HANDLE_CREATE,
@@ -476,11 +503,11 @@ posix_handle_init (xlator_t *this)
 
         sprintf (handle_pfx, "%s/%s", priv->base_path, GF_HIDDEN_PATH);
 
-        ret = stat (handle_pfx, &stbuf);
+        ret = sys_stat (handle_pfx, &stbuf);
         switch (ret) {
         case -1:
                 if (errno == ENOENT) {
-                        ret = mkdir (handle_pfx, 0600);
+                        ret = sys_mkdir (handle_pfx, 0600);
                         if (ret != 0) {
                                 gf_msg (this->name, GF_LOG_ERROR, errno,
                                         P_MSG_HANDLE_CREATE,
@@ -509,11 +536,11 @@ posix_handle_init (xlator_t *this)
                 break;
         }
 
-        stat (handle_pfx, &priv->handledir);
+        sys_stat (handle_pfx, &priv->handledir);
 
         MAKE_HANDLE_ABSPATH(rootstr, this, gfid);
 
-        ret = stat (rootstr, &rootbuf);
+        ret = sys_stat (rootstr, &rootbuf);
         switch (ret) {
         case -1:
                 if (errno != ENOENT) {
@@ -531,7 +558,7 @@ posix_handle_init (xlator_t *this)
                         return -1;
                 }
 
-                ret = symlink ("../../..", rootstr);
+                ret = sys_symlink ("../../..", rootstr);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_ERROR, errno,
                                 P_MSG_HANDLE_CREATE,
@@ -567,7 +594,7 @@ posix_does_old_trash_exists (char *old_trash)
         struct stat           stbuf = {0};
         int                   ret = 0;
 
-        ret = lstat (old_trash, &stbuf);
+        ret = sys_lstat (old_trash, &stbuf);
         if ((ret == 0) && S_ISDIR (stbuf.st_mode)) {
                 ret = sys_lgetxattr (old_trash, "trusted.gfid", gfid, 16);
                 if ((ret < 0) && (errno == ENODATA || errno == ENOATTR) )
@@ -582,11 +609,11 @@ posix_handle_new_trash_init (xlator_t *this, char *trash)
         int                     ret = 0;
         struct stat             stbuf = {0};
 
-        ret = lstat (trash, &stbuf);
+        ret = sys_lstat (trash, &stbuf);
         switch (ret) {
         case -1:
                 if (errno == ENOENT) {
-                        ret = mkdir (trash, 0755);
+                        ret = sys_mkdir (trash, 0755);
                         if (ret != 0) {
                                 gf_msg (this->name, GF_LOG_ERROR, errno,
                                         P_MSG_HANDLE_TRASH_CREATE,
@@ -625,7 +652,7 @@ posix_mv_old_trash_into_new_trash (xlator_t *this, char *old, char *new)
         gf_uuid_generate (dest_name);
         snprintf (dest_old, sizeof (dest_old), "%s/%s", new,
                   uuid_utoa (dest_name));
-        ret = rename (old, dest_old);
+        ret = sys_rename (old, dest_old);
         if (ret < 0) {
                 gf_msg (this->name, GF_LOG_ERROR, errno,
                         P_MSG_HANDLE_TRASH_CREATE,
@@ -676,7 +703,7 @@ posix_handle_mkdir_hashes (xlator_t *this, const char *newpath)
         parpath = dirname (duppath);
         parpath = dirname (duppath);
 
-        ret = mkdir (parpath, 0700);
+        ret = sys_mkdir (parpath, 0700);
         if (ret == -1 && errno != EEXIST) {
                 gf_msg (this->name, GF_LOG_ERROR, errno,
                         P_MSG_HANDLE_CREATE,
@@ -687,7 +714,7 @@ posix_handle_mkdir_hashes (xlator_t *this, const char *newpath)
         strcpy (duppath, newpath);
         parpath = dirname (duppath);
 
-        ret = mkdir (parpath, 0700);
+        ret = sys_mkdir (parpath, 0700);
         if (ret == -1 && errno != EEXIST) {
                 gf_msg (this->name, GF_LOG_ERROR, errno,
                         P_MSG_HANDLE_CREATE,
@@ -709,7 +736,7 @@ posix_handle_hard (xlator_t *this, const char *oldpath, uuid_t gfid, struct stat
 
         MAKE_HANDLE_ABSPATH (newpath, this, gfid);
 
-        ret = lstat (newpath, &newbuf);
+        ret = sys_lstat (newpath, &newbuf);
         if (ret == -1 && errno != ENOENT) {
                 gf_msg (this->name, GF_LOG_WARNING, errno,
                         P_MSG_HANDLE_CREATE,
@@ -735,7 +762,7 @@ posix_handle_hard (xlator_t *this, const char *oldpath, uuid_t gfid, struct stat
                         return -1;
                 }
 
-                ret = lstat (newpath, &newbuf);
+                ret = sys_lstat (newpath, &newbuf);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_WARNING, errno,
                                 P_MSG_HANDLE_CREATE,
@@ -771,7 +798,7 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
         MAKE_HANDLE_ABSPATH (newpath, this, gfid);
         MAKE_HANDLE_RELPATH (oldpath, this, loc->pargfid, loc->name);
 
-        ret = lstat (newpath, &newbuf);
+        ret = sys_lstat (newpath, &newbuf);
         if (ret == -1 && errno != ENOENT) {
                 gf_msg (this->name, GF_LOG_WARNING, errno,
                         P_MSG_HANDLE_CREATE, "%s", newpath);
@@ -779,6 +806,12 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
         }
 
         if (ret == -1 && errno == ENOENT) {
+                if (posix_is_malformed_link (this, newpath, oldpath,
+                                             strlen (oldpath))) {
+                        GF_ASSERT (!"Malformed link");
+                        errno = EINVAL;
+                        return -1;
+                }
                 ret = posix_handle_mkdir_hashes (this, newpath);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_WARNING, errno,
@@ -787,7 +820,7 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
                         return -1;
                 }
 
-                ret = symlink (oldpath, newpath);
+                ret = sys_symlink (oldpath, newpath);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_WARNING, errno,
                                 P_MSG_HANDLE_CREATE,
@@ -796,7 +829,7 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
                         return -1;
                 }
 
-                ret = lstat (newpath, &newbuf);
+                ret = sys_lstat (newpath, &newbuf);
                 if (ret) {
                         gf_msg (this->name, GF_LOG_WARNING, errno,
                                 P_MSG_HANDLE_CREATE,
@@ -805,7 +838,7 @@ posix_handle_soft (xlator_t *this, const char *real_path, loc_t *loc,
                 }
         }
 
-        ret = stat (real_path, &newbuf);
+        ret = sys_stat (real_path, &newbuf);
         if (ret) {
                 gf_msg (this->name, GF_LOG_WARNING, errno,
                         P_MSG_HANDLE_CREATE,
@@ -840,7 +873,7 @@ posix_handle_unset_gfid (xlator_t *this, uuid_t gfid)
 
         MAKE_HANDLE_GFID_PATH (path, this, gfid, NULL);
 
-        ret = lstat (path, &stat);
+        ret = sys_lstat (path, &stat);
 
         if (ret == -1) {
                 if (errno != ENOENT) {
@@ -850,7 +883,7 @@ posix_handle_unset_gfid (xlator_t *this, uuid_t gfid)
                 goto out;
         }
 
-        ret = unlink (path);
+        ret = sys_unlink (path);
         if (ret == -1) {
                 gf_msg (this->name, GF_LOG_WARNING, errno,
                         P_MSG_HANDLE_DELETE, "unlink %s failed ", path);
@@ -897,12 +930,18 @@ posix_handle_unset (xlator_t *this, uuid_t gfid, const char *basename)
 
 
 int
-posix_create_link_if_gfid_exists (xlator_t *this, uuid_t gfid,
-                                  char *real_path)
+posix_create_link_if_gfid_exists (xlator_t *this, uuid_t gfid, char *real_path,
+                                  inode_table_t *itable)
 {
-        int ret = -1;
-        struct stat stbuf = {0,};
-        char *newpath = NULL;
+        int                    ret         = -1;
+        char                  *newpath     = NULL;
+        char                  *unlink_path = NULL;
+        uint64_t               ctx_int     = 0;
+        inode_t               *inode       = NULL;
+        struct stat            stbuf       = {0,};
+        struct posix_private  *priv        = NULL;
+
+        priv = this->private;
 
         MAKE_HANDLE_PATH (newpath, this, gfid, NULL);
         if (!newpath) {
@@ -912,9 +951,46 @@ posix_create_link_if_gfid_exists (xlator_t *this, uuid_t gfid,
                 return ret;
         }
 
-        ret = lstat (newpath, &stbuf);
+        ret = sys_lstat (newpath, &stbuf);
         if (!ret) {
                 ret = sys_link (newpath, real_path);
+        } else {
+                inode = inode_find (itable, gfid);
+                if (!inode)
+                        return -1;
+
+                LOCK (&inode->lock);
+                {
+                        ret = __inode_ctx_get0 (inode, this, &ctx_int);
+                        if (ret)
+                                goto unlock;
+
+                        if (ctx_int != GF_UNLINK_TRUE)
+                                goto unlock;
+
+                        POSIX_GET_FILE_UNLINK_PATH (priv->base_path, gfid,
+                                                    unlink_path);
+                        ret = sys_link (unlink_path, real_path);
+                        if (ret) {
+                                gf_msg (this->name, GF_LOG_WARNING, errno,
+                                        P_MSG_HANDLE_CREATE, "Failed to link "
+                                        "%s with %s", real_path, unlink_path);
+                                goto unlock;
+                        }
+                        ret = sys_rename (unlink_path, newpath);
+                        if (ret) {
+                                gf_msg (this->name, GF_LOG_WARNING, errno,
+                                        P_MSG_HANDLE_CREATE, "Failed to link "
+                                        "%s with %s", real_path, unlink_path);
+                                goto unlock;
+                        }
+                        ctx_int = GF_UNLINK_FALSE;
+                        ret = __inode_ctx_set0 (inode, this, &ctx_int);
+                }
+unlock:
+                UNLOCK (&inode->lock);
+
+                inode_unref (inode);
         }
 
         return ret;
