@@ -158,12 +158,17 @@ sticky_permits (call_frame_t *frame, inode_t *parent, inode_t *inode)
         struct posix_acl_ctx  *par = NULL;
         struct posix_acl_ctx  *ctx = NULL;
 
-        par = posix_acl_ctx_get (parent, frame->this);
-        ctx = posix_acl_ctx_get (inode, frame->this);
-
-        if (frame_is_super_user (frame))
+        if ((0 > frame->root->pid) || frame_is_super_user (frame))
                 return 1;
 
+        par = posix_acl_ctx_get (parent, frame->this);
+        if (par == NULL)
+                return 0;
+
+        ctx = posix_acl_ctx_get (inode, frame->this);
+        if (ctx == NULL)
+                return 0;
+
         if (!(par->perm & S_ISVTX))
                 return 1;
 
@@ -192,13 +197,13 @@ acl_permits (call_frame_t *frame, inode_t *inode, int want)
 
         conf = frame->this->private;
 
+        if ((0 > frame->root->pid) || frame_is_super_user (frame))
+                goto green;
+
         ctx = posix_acl_ctx_get (inode, frame->this);
         if (!ctx)
                 goto red;
 
-        if (frame_is_super_user (frame))
-                goto green;
-
         posix_acl_get (inode, frame->this, &acl, NULL);
         if (!acl) {
                 acl = posix_acl_ref (frame->this, conf->minimal_acl);
@@ -287,25 +292,72 @@ out:
 
 
 struct posix_acl_ctx *
-posix_acl_ctx_get (inode_t *inode, xlator_t *this)
+__posix_acl_ctx_get (inode_t *inode, xlator_t *this, gf_boolean_t create)
 {
         struct posix_acl_ctx *ctx = NULL;
         uint64_t              int_ctx = 0;
         int                   ret = 0;
 
-        ret = inode_ctx_get (inode, this, &int_ctx);
+        ret = __inode_ctx_get (inode, this, &int_ctx);
         if ((ret == 0) && (int_ctx))
                 return PTR(int_ctx);
 
+        if (create == _gf_false)
+                return NULL;
+
         ctx = GF_CALLOC (1, sizeof (*ctx), gf_posix_acl_mt_ctx_t);
         if (!ctx)
                 return NULL;
 
-        ret = inode_ctx_put (inode, this, UINT64 (ctx));
+        ret = __inode_ctx_put (inode, this, UINT64 (ctx));
 
         return ctx;
 }
 
+struct posix_acl_ctx *
+posix_acl_ctx_new (inode_t *inode, xlator_t *this)
+{
+        struct posix_acl_ctx *ctx = NULL;
+
+        if (inode == NULL) {
+                gf_log_callingfn (this->name, GF_LOG_WARNING, "inode is NULL");
+                return NULL;
+        }
+
+        LOCK (&inode->lock);
+        {
+                ctx = __posix_acl_ctx_get (inode, this, _gf_true);
+        }
+        UNLOCK (&inode->lock);
+
+        if (ctx == NULL)
+                gf_log_callingfn (this->name, GF_LOG_ERROR, "creating inode ctx"
+                                  "failed for %s", uuid_utoa (inode->gfid));
+        return ctx;
+}
+
+struct posix_acl_ctx *
+posix_acl_ctx_get (inode_t *inode, xlator_t *this)
+{
+        struct posix_acl_ctx *ctx = NULL;
+
+        if (inode == NULL) {
+                gf_log_callingfn (this->name, GF_LOG_WARNING, "inode is NULL");
+                return NULL;
+        }
+
+        LOCK (&inode->lock);
+        {
+                ctx = __posix_acl_ctx_get (inode, this, _gf_false);
+        }
+        UNLOCK (&inode->lock);
+
+        if (ctx == NULL)
+                gf_log_callingfn (this->name, GF_LOG_ERROR, "inode ctx is NULL "
+                                  "for %s", uuid_utoa (inode->gfid));
+        return ctx;
+}
+
 int
 __posix_acl_set_specific (inode_t *inode, xlator_t *this,
                           gf_boolean_t is_access, struct posix_acl *acl)
@@ -636,7 +688,7 @@ posix_acl_inherit (xlator_t *this, loc_t *loc, dict_t *params, mode_t mode,
         if (!par_default)
                 goto out;
 
-        ctx = posix_acl_ctx_get (loc->inode, this);
+        ctx = posix_acl_ctx_new (loc->inode, this);
 
         acl_access = posix_acl_dup (this, par_default);
         if (!acl_access)
@@ -659,6 +711,7 @@ posix_acl_inherit (xlator_t *this, loc_t *loc, dict_t *params, mode_t mode,
                             size_access);
         if (ret) {
                 gf_log (this->name, GF_LOG_ERROR, "out of memory");
+                GF_FREE (xattr_access);
                 ret = -1;
                 goto out;
         }
@@ -682,6 +735,7 @@ posix_acl_inherit (xlator_t *this, loc_t *loc, dict_t *params, mode_t mode,
                             size_default);
         if (ret) {
                 gf_log (this->name, GF_LOG_ERROR, "out of memory");
+                GF_FREE (xattr_default);
                 ret = -1;
                 goto out;
         }
@@ -740,14 +794,14 @@ posix_acl_ctx_update (inode_t *inode, xlator_t *this, struct iatt *buf)
         int                   ret = 0;
 	int                   i = 0;
 
-        ctx = posix_acl_ctx_get (inode, this);
-        if (!ctx) {
-                ret = -1;
-                goto out;
-        }
-
         LOCK(&inode->lock);
         {
+                ctx = __posix_acl_ctx_get (inode, this, _gf_true);
+                if (!ctx) {
+                        ret = -1;
+                        goto unlock;
+                }
+
                 ctx->uid   = buf->ia_uid;
                 ctx->gid   = buf->ia_gid;
                 ctx->perm  = st_mode_from_ia (buf->ia_prot, buf->ia_type);
@@ -792,7 +846,6 @@ posix_acl_ctx_update (inode_t *inode, xlator_t *this, struct iatt *buf)
         }
 unlock:
         UNLOCK(&inode->lock);
-out:
         return ret;
 }
 
@@ -802,19 +855,30 @@ posix_acl_lookup_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                       int op_ret, int op_errno, inode_t *inode,
                       struct iatt *buf, dict_t *xattr, struct iatt *postparent)
 {
-        struct posix_acl     *acl_access = NULL;
+        struct posix_acl     *acl_access  = NULL;
         struct posix_acl     *acl_default = NULL;
-        struct posix_acl     *old_access = NULL;
+        struct posix_acl     *old_access  = NULL;
         struct posix_acl     *old_default = NULL;
-        data_t               *data = NULL;
-        int                   ret = 0;
-        dict_t               *my_xattr = NULL;
+        struct posix_acl_ctx *ctx         = NULL;
+        data_t               *data        = NULL;
+        int                   ret         = 0;
+        dict_t               *my_xattr    = NULL;
 
         if (op_ret != 0)
                 goto unwind;
 
+        ctx = posix_acl_ctx_new (inode, this);
+        if (!ctx) {
+                op_ret = -1;
+                op_errno = ENOMEM;
+                goto unwind;
+        }
+
         ret = posix_acl_get (inode, this, &old_access, &old_default);
 
+        if (xattr == NULL)
+                goto acl_set;
+
         data = dict_get (xattr, POSIX_ACL_ACCESS_XATTR);
         if (!data)
                 goto acl_default;
@@ -1067,7 +1131,7 @@ green:
                     loc, flags, fd, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (open, frame, -1, EACCES, NULL, xdata);
+        STACK_UNWIND_STRICT (open, frame, -1, EACCES, NULL, NULL);
         return 0;
 }
 
@@ -1102,7 +1166,8 @@ green:
                     fd, size, offset, flags, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (readv, frame, -1, EACCES, NULL, 0, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (readv, frame, -1, EACCES, NULL, 0, NULL, NULL,
+                             NULL);
         return 0;
 }
 
@@ -1137,7 +1202,7 @@ green:
                     fd, vector, count, offset, flags, iobref, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (writev, frame, -1, EACCES, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (writev, frame, -1, EACCES, NULL, NULL, NULL);
         return 0;
 }
 
@@ -1172,7 +1237,7 @@ green:
                     fd, offset, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (ftruncate, frame, -1, EACCES, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (ftruncate, frame, -1, EACCES, NULL, NULL, NULL);
         return 0;
 }
 
@@ -1200,7 +1265,7 @@ green:
                     loc, fd, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (opendir, frame, -1, EACCES, NULL, xdata);
+        STACK_UNWIND_STRICT (opendir, frame, -1, EACCES, NULL, NULL);
         return 0;
 }
 
@@ -1367,7 +1432,7 @@ green:
         return 0;
 red:
         STACK_UNWIND_STRICT (symlink, frame, -1, EACCES, NULL, NULL, NULL,
-                             NULL, xdata);
+                             NULL, NULL);
         return 0;
 }
 
@@ -1403,7 +1468,7 @@ green:
                     loc, xflag, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (unlink, frame, -1, EACCES, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (unlink, frame, -1, EACCES, NULL, NULL, NULL);
         return 0;
 }
 
@@ -1438,7 +1503,7 @@ green:
                     loc, flags, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (rmdir, frame, -1, EACCES, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (rmdir, frame, -1, EACCES, NULL, NULL, NULL);
         return 0;
 }
 
@@ -1529,7 +1594,8 @@ posix_acl_link (call_frame_t *frame, xlator_t *this, loc_t *old, loc_t *new, dic
                     old, new, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (link, frame, -1, op_errno, NULL, NULL, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (link, frame, -1, op_errno, NULL, NULL, NULL, NULL,
+                             NULL);
 
         return 0;
 }
@@ -1562,7 +1628,7 @@ green:
                     fd, size, offset, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (readdir, frame, -1, EACCES, NULL, xdata);
+        STACK_UNWIND_STRICT (readdir, frame, -1, EACCES, NULL, NULL);
 
         return 0;
 }
@@ -1573,11 +1639,12 @@ posix_acl_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                         int op_ret, int op_errno, gf_dirent_t *entries,
                         dict_t *xdata)
 {
-        gf_dirent_t      *entry       = NULL;
-        struct posix_acl *acl_access  = NULL;
-        struct posix_acl *acl_default = NULL;
-        data_t           *data        = NULL;
-        int               ret         = 0;
+        gf_dirent_t          *entry       = NULL;
+        struct posix_acl     *acl_access  = NULL;
+        struct posix_acl     *acl_default = NULL;
+        struct posix_acl_ctx *ctx         = NULL;
+        data_t               *data        = NULL;
+        int                   ret         = 0;
 
         if (op_ret <= 0)
                 goto unwind;
@@ -1587,6 +1654,13 @@ posix_acl_readdirp_cbk (call_frame_t *frame, void *cookie, xlator_t *this,
                 if (!entry->dict || !entry->inode)
                         continue;
 
+                ctx = posix_acl_ctx_new (entry->inode, this);
+                if (!ctx) {
+                        op_ret = -1;
+                        op_errno = ENOMEM;
+                        goto unwind;
+                }
+
                 ret = posix_acl_get (entry->inode, this,
                                      &acl_access, &acl_default);
 
@@ -1786,7 +1860,7 @@ posix_acl_setattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
                     loc, buf, valid, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (setattr, frame, -1, op_errno, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (setattr, frame, -1, op_errno, NULL, NULL, NULL);
 
         return 0;
 }
@@ -1832,7 +1906,7 @@ posix_acl_fsetattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
                     fd, buf, valid, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (fsetattr, frame, -1, EACCES, NULL, NULL, xdata);
+        STACK_UNWIND_STRICT (fsetattr, frame, -1, EACCES, NULL, NULL, NULL);
 
         return 0;
 }
@@ -1945,10 +2019,12 @@ handling_other_acl_related_xattr (xlator_t *this, inode_t *inode, dict_t *xattr)
         data_t                 *data     = NULL;
         int                     ret      = 0;
 
+        if (!this || !xattr || !inode)
+                goto out;
+
         data = dict_get (xattr, POSIX_ACL_ACCESS_XATTR);
         if (data) {
 
-
                 acl = posix_acl_from_xattr (this, data->data, data->len);
                 if (!acl) {
                         ret = -1;
@@ -2026,7 +2102,7 @@ posix_acl_setxattr (call_frame_t *frame, xlator_t *this, loc_t *loc,
                            loc, xattr, flags, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (setxattr, frame, -1, op_errno, xdata);
+        STACK_UNWIND_STRICT (setxattr, frame, -1, op_errno, NULL);
 
         return 0;
 }
@@ -2060,7 +2136,7 @@ posix_acl_fsetxattr (call_frame_t *frame, xlator_t *this, fd_t *fd,
                     fd, xattr, flags, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (fsetxattr, frame, -1, op_errno, xdata);
+        STACK_UNWIND_STRICT (fsetxattr, frame, -1, op_errno, NULL);
 
         return 0;
 }
@@ -2095,7 +2171,7 @@ green:
         return 0;
 
 red:
-        STACK_UNWIND_STRICT (getxattr, frame, -1, EACCES, NULL, xdata);
+        STACK_UNWIND_STRICT (getxattr, frame, -1, EACCES, NULL, NULL);
 
         return 0;
 }
@@ -2128,7 +2204,7 @@ green:
                     fd, name, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (fgetxattr, frame, -1, EACCES, NULL, xdata);
+        STACK_UNWIND_STRICT (fgetxattr, frame, -1, EACCES, NULL, NULL);
 
         return 0;
 }
@@ -2177,7 +2253,7 @@ green:
                     loc, name, xdata);
         return 0;
 red:
-        STACK_UNWIND_STRICT (removexattr, frame, -1, op_errno, xdata);
+        STACK_UNWIND_STRICT (removexattr, frame, -1, op_errno, NULL);
 
         return 0;
 }
